@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@
 
 #include "udm_unicode.h"
 #include "udm_uniconv.h"
+#include "udm_unidata.h"
 
 /* Some constants */
 #define UDM_LANGPERDOC				16		/* FIXME */
@@ -84,6 +85,7 @@
 #define UDM_DBMODE_SINGLE	0
 #define UDM_DBMODE_MULTI	1
 #define UDM_DBMODE_BLOB		6
+#define UDM_DBMODE_RAWBLOB	7
 
 
 /* database open modes */
@@ -162,16 +164,21 @@
 #define UDM_WORD_ORIGIN_SYNONYM 4
 #define UDM_WORD_ORIGIN_STOP    8
 #define UDM_WORD_ORIGIN_SUGGEST 16
+#define UDM_WORD_ORIGIN_COLLATION 32
 
 /* Locking mutex numbers */
-#define UDM_LOCK_CONF		0
-#define UDM_LOCK_WIN		1
-#define UDM_LOCK_THREAD         2
-#define UDM_LOCK_SQL            3
-#define UDM_LOCK_SEGMENTER      4
-#define UDM_LOCK_DB             5
-#define UDM_LOCK_LOG		6
-#define UDM_LOCK_MAX		7
+#define UDM_LOCK_DUMMY          0
+#define UDM_LOCK_CONF		1
+#define UDM_LOCK_WIN		2
+#define UDM_LOCK_THREAD         3
+#define UDM_LOCK_SQL            4
+#define UDM_LOCK_SEGMENTER      5
+#define UDM_LOCK_DB             6
+#define UDM_LOCK_LOG		7
+#define UDM_LOCK_ROBOT_FIRST	8
+#define UDM_LOCK_ROBOT_LAST	31
+#define UDM_LOCK_ROBOT_COUNT    (UDM_LOCK_ROBOT_LAST - UDM_LOCK_ROBOT_FIRST +1)
+#define UDM_LOCK_MAX		(UDM_LOCK_ROBOT_LAST + 1)
 
 /************************ Statistics **********************/
 typedef struct stat_struct {
@@ -191,6 +198,9 @@ typedef struct stat_list_struct{
 #define UDM_VARFLAG_NOCLONE	1  /* Ignore for clone detection        */ 
 #define UDM_VARFLAG_USERDEF	2  /* User defined section              */
 #define UDM_VARFLAG_KEEPVAL	4  /* If value doesn't need to be freed */
+#define UDM_VARFLAG_HTMLSOURCE	8  /* If apply HTML parser, for HTDB    */
+#define UDM_VARFLAG_WIKI       16  /* If to remove text between [ and ] */
+#define UDM_VARFLAG_HL         32  /* If variable has highlight markers */
 
 /* Variable types */
 #define UDM_VAR_INT       1
@@ -219,8 +229,9 @@ typedef struct udm_var_st {
 
 typedef struct udm_varlist_st {
 	int		freeme;
-	size_t		nvars;
-	size_t		mvars;
+	size_t		nvars; /* Number of registered variables */
+	size_t		mvars; /* Number of allocated variables  */
+	size_t          svars; /* Number of sorted variables     */
 	UDM_VAR		*Var;
 } UDM_VARLIST;
 
@@ -260,12 +271,60 @@ typedef struct {
 	size_t		max_word_len;
 } UDM_WORDPARAM;
 
+typedef uint4 udm_pos_t;
+typedef unsigned char udm_secno_t;
+typedef unsigned char udm_wordnum_t;
+
+typedef struct udm_coord2_st
+{
+  udm_pos_t pos;
+} UDM_COORD2;
+
+
+typedef struct udm_section_st
+{
+  UDM_COORD2 *Coord;
+  urlid_t url_id;
+  uint4 ncoords;
+  udm_pos_t seclen;
+  udm_pos_t minpos;
+  udm_pos_t maxpos;
+  udm_secno_t secno;
+  udm_wordnum_t wordnum;
+  udm_wordnum_t order;
+} UDM_SECTION;
+
+
+typedef struct udm_sectionlist_st
+{
+  size_t mcoords;
+  size_t ncoords;
+  UDM_COORD2 *Coord;
+  size_t msections;
+  size_t nsections;
+  UDM_SECTION *Section;
+} UDM_SECTIONLIST;
+
+
+typedef struct udm_sectionlistlist_st
+{
+  size_t nitems;
+  size_t mitems;
+  UDM_SECTIONLIST *Item;
+} UDM_SECTIONLISTLIST;
+
+
 
 /** Main search structure */
-typedef struct{
-	urlid_t		url_id;
-	uint4		coord;
-} UDM_URL_CRD;
+typedef struct {
+  urlid_t url_id;
+  uint4   score;
+} UDM_URL_SCORE;
+
+typedef struct {
+  size_t nitems;
+  UDM_URL_SCORE	*Item;
+} UDM_URLSCORELIST;
 
 typedef struct {
         urlid_t         url_id;
@@ -278,30 +337,22 @@ typedef struct {
         char            *section;
 } UDM_URLDATA;
 
+#define UDM_COORD2DBNUM(score) (255 - (int) ((score) & 0xFF))
+
 typedef struct {
 	size_t		nitems;
 	UDM_URLDATA	*Item;
 } UDM_URLDATALIST;
 
-typedef struct {
-        size_t          acoords;
-	size_t		ncoords;
-	size_t		order;
-	char		*word;
-	UDM_URL_CRD	*Coords;
-        UDM_URLDATA	*Data;
-} UDM_URLCRDLIST;
-
-typedef struct {
-	int		freeme;
-	size_t		nlists;
-	UDM_URLCRDLIST	*List;
-} UDM_URLCRDLISTLIST;
 
 /** Word list unit */
-typedef struct {
-	int		coord;
-	char		*word;
+typedef struct
+{
+  char		*word;
+  udm_pos_t     pos;
+  udm_secno_t   secno;
+  unsigned char hash;
+  unsigned char seclen_marker;
 } UDM_WORD;
 
 typedef struct {
@@ -317,11 +368,6 @@ typedef struct {
 } UDM_WORDLIST;
 
 
-#define UDM_WRDCOORD(p,w)	( (((unsigned int)(p))<<16)+(((unsigned int)(w))<<8) )
-#define UDM_WRDSEC(c)		( (((unsigned int)(c))>>8)&0xFF )
-#define UDM_WRDPOS(c)		( ((unsigned int)(c))>>16 )
-#define UDM_WRDNUM(c)		( ((unsigned int)(c))&0xFF )
-#define UDM_WRDMASK(c)		( 1L << (((unsigned int)(c))&0xFF) )
 
 
 /***************************************************************/
@@ -338,7 +384,7 @@ typedef struct {
 typedef struct {
 	size_t		ncrosswords;
 	size_t		mcrosswords;
-	size_t		wordpos;
+	size_t		wordpos[256];
 	UDM_CROSSWORD	*CrossWord;
 } UDM_CROSSLIST;
 
@@ -356,6 +402,7 @@ typedef struct {
 	int	use_robots;	/**< Whether to use robots.txt*/
 	int	use_clones;	/**< Whether to detect clones */
         int	doc_per_site;
+        int	collect_links;
 } UDM_SPIDERPARAM;
 
 /*****************************************************************/
@@ -365,6 +412,7 @@ typedef struct {
 	int		case_sense;
 	char		*section;
 	char		*pattern;
+	size_t          pattern_length;
 	regex_t		*reg;
 	char		*arg;
 	char		*arg1;
@@ -411,6 +459,7 @@ typedef struct {
 	uint4	hops;
 	int	stored;	
 	int	method;
+	int	collect_links;
         urlid_t site_id;
         urlid_t server_id;
         urlid_t rec_id;
@@ -463,7 +512,7 @@ typedef struct udm_conn_struct {
         size_t	buf_len_total;
         int	len;
         char	*buf;
-        UDM_HOST_ADDR *Host;
+        int     net_errors;
         struct	udm_conn_struct *connp;
 } UDM_CONN;
 
@@ -489,6 +538,7 @@ typedef struct {
 	char	*content;	/**< Pointer to content, after headers       */
 	size_t	size;		/**< Number of bytes loaded                  */
 	size_t	maxsize;	/**< Maximum bytes to load into buf          */
+	size_t  content_length; /**  For HTDB **/
 } UDM_HTTPBUF;
 
 typedef struct {
@@ -622,35 +672,24 @@ typedef struct{
 	UDM_ROBOT	*Robot;
 } UDM_ROBOTS;
 
-#define MAX_SEARCH_LIMIT 32
-
-typedef struct{
-	int		type;
-	char		file_name[1024];
-	uint4		hi;
-	uint4		lo;
-        uint4           f_hi;
-        uint4           f_lo;
-} UDM_SEARCH_LIMIT;
-
 
 typedef struct {
 	size_t		order;
 	size_t		count;
 	char		*word;
-	int		*uword;
 	size_t		len;
-	size_t		ulen;
         int     	origin;
         int             weight; /* origin-dependent weight */
         int		match;
         size_t		secno;  /* Which section to search in */
         size_t		phrpos; /* 0 means "not in phrase"    */
         size_t		phrlen; /* phase length               */
+        size_t          phrwidth; /* How many additional parts in a multi-word */
 } UDM_WIDEWORD;
 
 typedef struct {
-	int		wm;    /* Search mode: wrd, sub, beg, end */
+	int		wm;              /* Search mode: wrd, sub, beg, end */
+	int             strip_noaccents; /* If accent insensitive comparison*/
 	size_t		nuniq;
 	size_t		nwords;
 	UDM_WIDEWORD	*Word;
@@ -658,8 +697,8 @@ typedef struct {
 
 
 typedef struct {
-	UDM_WIDEWORD	p;
-	UDM_WIDEWORD	s;
+	char	*p;
+	char	*s;
 } UDM_SYNONYM;
 
 typedef struct {
@@ -718,6 +757,13 @@ typedef struct {
 	unsigned long	arg;
 } UDM_STACK_ITEM;
 
+typedef struct {
+        size_t nitems;
+        size_t mitems;
+        size_t ncmds;
+        UDM_STACK_ITEM *items;
+} UDM_STACKITEMLIST;
+
 typedef struct {
 	size_t			work_time;
 	size_t			first;
@@ -731,11 +777,8 @@ typedef struct {
 	UDM_DOCUMENT		*Doc;
 	
 	UDM_WIDEWORDLIST	WWList;
-	UDM_URLCRDLIST		CoordList;
-	
-	/* Bool stuff */
-	size_t			nitems, mitems, ncmds;
-	UDM_STACK_ITEM		*items;
+	UDM_URLDATALIST		URLData;
+	UDM_STACKITEMLIST	ItemList;
 	
 } UDM_RESULT;
 
@@ -807,17 +850,29 @@ typedef struct udm_config_struct {
 #ifdef MECAB
         mecab_t         *mecab;
 #endif
+        UDM_UNIDATA *unidata;
 
 } UDM_ENV;
 
 
+#define UDM_AGENT_STATE_MUTEXES 5
+
+typedef struct udm_agent_state_t
+{
+  const char *task;
+  const char *param;
+  const char *extra;
+  time_t start_time;
+  size_t mutex_owned[UDM_AGENT_STATE_MUTEXES];
+  size_t nmutexes;
+} UDM_AGENT_STATE;
 
 /** Indexer */
 typedef struct udm_indexer_struct{
 	int		freeme;		/**< whenever it was allocated    */
 	int		handle;		/**< Handler for threaded version */
 	time_t		start_time;	/**< Time of allocation, for stat */
-	size_t		nbytes;		/**< Number of bytes downloaded   */
+	unsigned long long nbytes;	/**< Number of bytes downloaded   */
 	size_t		ndocs;		/**< Number of documents          */
         size_t          nsleepsecs;     /**> Number of sleep seconds      */
 	int		flags;		/**< Callback function to request action*/
@@ -826,11 +881,9 @@ typedef struct udm_indexer_struct{
 	UDM_ENV		*Conf;		/**< Configuration                */
 	UDM_LANGMAP	*LangMap;	/**< LangMap for current document */
 	UDM_RESULT	Indexed;	/**< Indexed cache              */
-	
-	/** Cache mode limits */
-	UDM_SEARCH_LIMIT	limits[MAX_SEARCH_LIMIT];
-	unsigned int		nlimits;
 
+	UDM_AGENT_STATE State;
+	
         char    *UdmFindURLCache[UDM_FINDURL_CACHE_SIZE];
         urlid_t UdmFindURLCacheId[UDM_FINDURL_CACHE_SIZE];
         size_t  pURLCache;
@@ -887,30 +940,10 @@ typedef struct {
 } UDM_UINT8URLIDLIST;
 
 
-#define UDM_SEARCHD_CMD_ERROR	   1
-#define UDM_SEARCHD_CMD_MESSAGE	   2
-#define UDM_SEARCHD_CMD_WORDS	   3
-#define UDM_SEARCHD_CMD_GOODBYE	   4
-#define UDM_SEARCHD_CMD_DOCINFO	   5
-#define UDM_SEARCHD_CMD_WITHOFFSET 7
-#define UDM_SEARCHD_CMD_WWL        8
-#define UDM_SEARCHD_CMD_CATINFO    9
-#define UDM_SEARCHD_CMD_URLACTION  10
-#define UDM_SEARCHD_CMD_DOCCOUNT   11
-#define UDM_SEARCHD_CMD_PERSITE    12
-#define UDM_SEARCHD_CMD_DATA       13
-#define UDM_SEARCHD_CMD_CLONES     14
-
 #define UDM_LOGD_CMD_WORD  0
 #define UDM_LOGD_CMD_DATA  1
 #define UDM_LOGD_CMD_CHECK 2
 
-typedef struct {
-	size_t	cmd;
-	size_t	len;
-} UDM_SEARCHD_PACKET_HEADER;
-
-
 #define UDM_MAXTAGVAL	64
 
 typedef struct {
@@ -923,12 +956,8 @@ typedef struct {
 	int	index;
 	int	comment;
 	int	nonbreaking;
+	int     skip_attribute_sections;
 	char	*lasthref;
-        void    (*next_b)(void *t);
-        void    (*next_e)(void *t);
-        const char *e;
-        const char *b;
-        const char *s;
 	size_t	ntoks;
 	struct {
 		const char *name;
