@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,7 +28,8 @@ extern int  UdmSearchMode(const char * mode);
 extern int  UdmMatchMode(const char * mode);
 extern int  UdmFollowType(const char * follow);
 extern int  UdmMethod(const char * method);
-extern void UdmWeightFactorsInit(const char *wf, int *res);
+extern void UdmWeightFactorsInit(char *res, const char *wf, size_t numsections);
+extern size_t UdmWeightFactorsInit2(char *res, UDM_VARLIST *V1, UDM_VARLIST *V2, const char *name);
 
 extern const char	*UdmMethodStr(int method);
 extern __C_LINK const char * __UDMCALL UdmFollowStr(int follow);
