@@ -42,6 +42,7 @@
 #define __INDLIB__		__declspec(dllexport) /* export functions for DLL  */
 #define UDMSLASH		'\\'
 #define UDMSLASHSTR		"\\"
+#define UDM_TMP_DIR		"."
 #define UDMSLEEP(x)		_sleep(x*1000)
 #define getpid()		_getpid()
 #define UDM_IWRITE		S_IWRITE
@@ -79,6 +80,7 @@ typedef __int32 int4;
 
 #define UDMSLASH		'/'
 #define UDMSLASHSTR		"/"
+#define UDM_TMP_DIR		"/tmp"
 #define UDMSLEEP(x)		sleep(x)
 #define closesocket(x)		close(x)
 #define UDM_IWRITE		0644
@@ -153,12 +155,6 @@ typedef uint4 udmcrc32_t;
 #include "udm_efence.h"
 #endif
 
-#ifdef HAVE_MERGESORT
-#define UdmSort mergesort
-#else
-#define UdmSort qsort
-#endif
-
 #ifdef USE_PARANOIA
 extern void *UdmViolationEnter(void);
 extern void UdmViolationExit(void *);
@@ -166,6 +162,10 @@ extern void UdmViolationExit(void *);
 #define UdmViolationExit(x)
 #endif
 
+#ifndef PATH_MAX
+#define PATH_MAX 512
+#endif
+
 #ifdef HAVE_DEBUG
   #define UDM_ASSERT(x)  assert(x)
   #ifdef HAVE_PTHREAD
@@ -178,5 +178,4 @@ extern void UdmViolationExit(void *);
   #define UDM_ASSERT_MUTEX_OWNER(x) 
 #endif
 
-
 #endif /* _UDM_CONFIG_H */
