@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,9 @@ extern udmcrc32_t UdmCRC32(const char * buf, size_t size);
 /* Updates previous value with new data block*/
 extern udmcrc32_t UdmCRC32Update(int prev,const char * buf, size_t size);
 
+/* Updates previous value with new Unicode block*/
+extern udmcrc32_t UdmCRC32UpdateUnicode(int prev, const int *buf, size_t charlen);
+
 /* Defines for null-terminated string */
 
 #define UdmStrCRC32(buf)		UdmCRC32((buf),strlen(buf))
