@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -73,26 +73,6 @@
 #define UDM_SRV_ACTION_ID               4
 #define UDM_SRV_ACTION_POPRANK          5
 
-#define UDM_IFIELD_TYPE_HOUR     0
-#define UDM_IFIELD_TYPE_MIN      1
-#define UDM_IFIELD_TYPE_HOSTNAME 2
-#define UDM_IFIELD_TYPE_STRCRC32 3
-#define UDM_IFIELD_TYPE_INT      4
-#define UDM_IFIELD_TYPE_HEX8STR  5
-
-#define UDM_LIMTYPE_NESTED     0
-#define UDM_LIMTYPE_TIME       1
-#define UDM_LIMTYPE_LINEAR_INT 2
-#define UDM_LIMTYPE_LINEAR_CRC 3
-
-#define UDM_LIMFNAME_CAT   "lim_cat"
-#define UDM_LIMFNAME_TAG   "lim_tag"
-#define UDM_LIMFNAME_TIME  "lim_time"
-#define UDM_LIMFNAME_HOST  "lim_host"
-#define UDM_LIMFNAME_LANG  "lim_lang"
-#define UDM_LIMFNAME_CTYPE "lim_ctype"
-#define UDM_LIMFNAME_SITE  "lim_site"
-
 
 extern void	UdmDBFree(void *db);
 
@@ -103,8 +83,6 @@ extern __C_LINK int __UDMCALL UdmCatAction(UDM_AGENT *A, UDM_CATEGORY   *C, int
 extern __C_LINK int __UDMCALL UdmSrvAction(UDM_AGENT *A, UDM_SERVERLIST *S, int cmd);
 extern __C_LINK int __UDMCALL UdmTargets(UDM_AGENT *Indexer);
 
-extern __C_LINK int __UDMCALL UdmLimit8(UDM_ENV *Conf, UDM_UINT8URLIDLIST *L,const char *field, int type, void *db);
-extern __C_LINK int __UDMCALL UdmLimit4(UDM_ENV *Conf, UDM_UINT4URLIDLIST *L,const char *field, int type, void *db);
 
 extern __C_LINK UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *);
 extern __C_LINK int __UDMCALL UdmClearDatabase(UDM_AGENT *A);
@@ -117,7 +95,6 @@ extern void		UdmDBListFree(UDM_DBLIST *List);
 
 extern unsigned int     UdmGetCategoryId(UDM_ENV *Conf, char *category);
 
-extern int UdmFindWords(UDM_AGENT *A, UDM_RESULT *Res);
 extern int UdmHTDBGet(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc);
 
 extern int UdmCheckUrlid(UDM_AGENT *A, urlid_t id);
@@ -144,10 +121,8 @@ extern int   UdmResActionSQL(UDM_AGENT  *A, UDM_RESULT     *R, int cmd, UDM_DB *
 extern int   UdmCatActionSQL(UDM_AGENT  *A, UDM_CATEGORY   *C, int cmd, UDM_DB *db);
 extern int   UdmSrvActionSQL(UDM_AGENT  *A, UDM_SERVERLIST *S, int cmd, UDM_DB *db);
 extern int   UdmStatActionSQL(UDM_AGENT *A, UDM_STATLIST   *S, UDM_DB *db);
-extern int   UdmFindWordsSQL(UDM_AGENT *, UDM_RESULT *Res, UDM_DB *db);
+extern int   UdmFindWordsSQL(UDM_AGENT *, UDM_RESULT *Res, UDM_DB *db, size_t num_best_rows);
 extern int   UdmURLDataSQL(UDM_ENV *Conf, UDM_URLDATALIST *L, UDM_DB *db);
-extern int   UdmLimit8SQL(UDM_ENV *Conf, UDM_UINT8URLIDLIST *L,const char *field, int type, UDM_DB *db);
-extern int   UdmLimit4SQL(UDM_ENV *Conf, UDM_UINT4URLIDLIST *L,const char *field, int type, UDM_DB *db);
 extern unsigned int   UdmGetCategoryIdSQL(UDM_ENV *Conf, char *category, UDM_DB *db);
 extern int UdmResAddDocInfoSQL(UDM_AGENT *query, UDM_DB *db, UDM_RESULT *Res, size_t i);
 extern int UdmTrackSQL(UDM_AGENT * query, UDM_RESULT *Res, UDM_DB *db);
@@ -162,9 +137,7 @@ extern UDM_MULTI_CACHE *UdmMultiCacheInit (UDM_MULTI_CACHE *cache);
 extern void UdmMultiCacheFree (UDM_MULTI_CACHE *cache);
 extern size_t UdmMultiCacheAdd (UDM_MULTI_CACHE *cache, urlid_t url_id, unsigned char reindex, UDM_WORD *word);
 
-extern int UdmMulti2BlobSQL (UDM_AGENT *Indexer, UDM_DB *db);
-extern int UdmSingle2BlobSQL (UDM_AGENT *Indexer, UDM_DB *db);
-extern int UdmBlob2BlobSQL (UDM_AGENT *Indexer, UDM_DB *db);
+extern int UdmConvert2BlobSQL(UDM_AGENT *Indexer, UDM_DB *db);
 extern __C_LINK int __UDMCALL UdmMulti2Blob (UDM_AGENT *Indexer);
 extern __C_LINK int __UDMCALL UdmRewriteURL(UDM_AGENT *Indexer);
 extern __C_LINK int __UDMCALL UdmRewriteLimits(UDM_AGENT *Indexer);
@@ -187,7 +160,7 @@ extern void UdmBlobCacheSort (UDM_BLOB_CACHE *cache);
 extern UDM_WORD_CACHE *UdmWordCacheInit (UDM_WORD_CACHE *cache);
 extern void UdmWordCacheFree (UDM_WORD_CACHE *cache);
 extern void UdmWordCacheSort (UDM_WORD_CACHE *cache);
-extern int UdmWordCacheAdd (UDM_WORD_CACHE *cache, urlid_t url_id, const char *word, int intag);
+extern int UdmWordCacheAdd (UDM_WORD_CACHE *cache, urlid_t url_id, UDM_WORD *W);
 extern int UdmWordCacheAddURL (UDM_WORD_CACHE *cache, urlid_t url_id);
 extern int UdmWordCacheWrite (UDM_AGENT *Indexer, UDM_DB *db, size_t limit);
 extern int UdmWordCacheFlush (UDM_AGENT *Indexer);
