@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 /* Multi-dict mode defines */
 #define MULTI_DICTS	0xff
 
-#define	URL_DELETE_CACHE		128
+#define	URL_DELETE_CACHE		256
 #define	URL_SELECT_CACHE		256
 #define URL_LOCK_TIME			4*60*60
 #define UDM_MAX_MULTI_INSERT_QSIZE	16*1024
@@ -72,6 +72,18 @@ typedef struct {
 	struct udm_db_st *db;
 } UDM_SQLRES;
 
+
+typedef struct udm_sql_res_list_st
+{
+  size_t nitems;
+  UDM_SQLRES *Item;
+} UDM_SQLRESLIST;
+
+void UdmSQLResListInit(UDM_SQLRESLIST *List);
+int UdmSQLResListAdd(UDM_SQLRESLIST *List, UDM_SQLRES *Res);
+void UdmSQLResListFree(UDM_SQLRESLIST *List);
+
+
 typedef struct {
 	char *word;
 	size_t nintags;
@@ -124,10 +136,10 @@ typedef struct {
 
 typedef struct {
 	urlid_t url_id;
+	char    *word;
+	udm_pos_t pos;
 	unsigned char secno;
 	unsigned char seed;
-	unsigned short coord;
-	char *word;
 } UDM_WORD_CACHE_WORD;
 
 typedef struct {
