@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,8 +23,4 @@ extern __C_LINK void __UDMCALL UdmDocFree(UDM_DOCUMENT*);
 extern int           UdmDocToTextBuf(UDM_DOCUMENT * Doc,char *d,size_t maxlen);
 extern __C_LINK int __UDMCALL UdmDocFromTextBuf(UDM_DOCUMENT * Doc,const char *d);
 
-extern void UdmURLCRDListListInit(UDM_URLCRDLISTLIST *Lst);
-extern void UdmURLCRDListListFree(UDM_URLCRDLISTLIST *Lst);
-extern int UdmURLCRDListListAdd(UDM_URLCRDLISTLIST *Lst, UDM_URLCRDLIST *I);
-
 #endif
