@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,11 @@
 #ifndef _UDM_HOST_H
 #define _UDM_HOST_H
 
+extern int             UdmHostLookup2(UDM_AGENT *,UDM_HOSTLIST *,UDM_CONN *);
 extern int             UdmHostLookup(UDM_HOSTLIST *,UDM_CONN *);
 extern void            UdmHostListFree(UDM_HOSTLIST *);
+extern UDM_HOST_ADDR  *UdmHostFind(UDM_HOSTLIST *,const char *hostname);
+extern void            UdmHostErrorReset(UDM_HOSTLIST *, const char *hostname);
+extern void            UdmHostErrorIncrement(UDM_HOSTLIST *, const char *hostname);
 
 #endif
