@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,6 @@ extern UDM_RESULT   *UdmResultInit(UDM_RESULT*);
 extern __C_LINK void __UDMCALL UdmResultFree(UDM_RESULT*);
 extern int           UdmResultToTextBuf(UDM_RESULT *R,char *buf,size_t len);
 extern int           UdmResultFromTextBuf(UDM_RESULT *R,char *buf);
-extern int           UdmResultFromXML(UDM_RESULT *Res, const char *s, size_t l, UDM_CHARSET *cs);
+extern int           UdmResultFromXML(UDM_AGENT *A, UDM_RESULT *Res, const char *s, size_t l, UDM_CHARSET *cs);
 
 #endif
