@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,9 +23,11 @@
 extern __C_LINK int __UDMCALL UdmFindWordsSearchd(UDM_AGENT *query,UDM_RESULT *Res, UDM_DB *searchd);
 extern __C_LINK int __UDMCALL UdmSearchdCatAction(UDM_AGENT *A, UDM_CATEGORY *C, int cmd, void *vdb);
 extern __C_LINK int __UDMCALL UdmSearchdURLAction(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd, void *vdb);
-extern __C_LINK int __UDMCALL UdmResAddDocInfoSearchd(UDM_AGENT * query, UDM_DB *cl, UDM_RESULT * Res, size_t clnum);
-extern int UdmSearchdConnect(UDM_DB *cl);
-extern void UdmSearchdClose(UDM_DB *cl);
+extern __C_LINK int __UDMCALL UdmResAddDocInfoSearchd(UDM_AGENT * query,
+                                                      UDM_RESULT *TmpRes,
+                                                      UDM_DB *db,
+                                                      UDM_RESULT *Res,
+                                                      size_t clnum);
 extern int UdmCloneListSearchd(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc, UDM_RESULT *Res, UDM_DB *db);
 
 #endif
