@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,5 +22,6 @@ extern __C_LINK char * __UDMCALL UdmSGMLUnescape(char * str);
 extern size_t UdmHTMLEncode(char *d, size_t dlen, const char *s, size_t slen);
 extern void UdmSGMLUniUnescape(int * ustr);
 extern int UdmSgmlToUni(const char *sgml);
-
+extern int UdmSGMLScan(int *wc,
+                       const unsigned char *str, const unsigned char *strend);
 #endif
