@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,10 @@
 #define UDM_SQL_HAVE_BIND       32  /* whether to use prepared statements */
 #define UDM_SQL_HAVE_0xHEX	64  /* INSERT INTO t1 VALUES (0xAABBCC)   */
 #define UDM_SQL_HAVE_STDHEX    128  /* INSERT INTO t1 VALUES (X'AABBCC')  */
+#define UDM_SQL_HAVE_ROWNUM    256  /* ROWNUM to limit result size        */
+
+#define UDM_SQL_IGNORE_ERROR  1024
+#define UDM_SQL_DEBUG_QUERY   2048
 
 extern void DecodeHexStr (char *src, UDM_PSTR *dst, size_t size);
 char           *UdmDBEscStr(int DBType,char *to,const char *from,size_t len);
@@ -40,8 +44,18 @@ extern __C_LINK int __UDMCALL UdmSQLStoreResultSimple(UDM_DB *,UDM_SQLRES *res);
 extern __C_LINK int __UDMCALL UdmSQLFreeResultSimple(UDM_DB *,UDM_SQLRES *res);
 extern __C_LINK void __UDMCALL UdmSQLFree(UDM_SQLRES*);
 void		UdmSQLClose(UDM_DB *db);
+#ifdef HAVE_DEBUG
+extern int UdmSQLBeginDebug(UDM_DB *db, const char *file, const int line);
+extern int UdmSQLCommitDebug(UDM_DB *db, const char *file, const int line);
+extern int UdmSQLExecDirectDebug(UDM_DB *db, UDM_SQLRES *R, const char *q, const char *file, const int line);
+#define UdmSQLBegin(db)   UdmSQLBeginDebug(db, __FILE__, __LINE__)
+#define UdmSQLCommit(db)  UdmSQLCommitDebug(db, __FILE__, __LINE__)
+#define UdmSQLExecDirect(db,R,q) UdmSQLExecDirectDebug(db, R, q, __FILE__, __LINE__)
+#else
 extern int UdmSQLBegin(UDM_DB *db);
 extern int UdmSQLCommit(UDM_DB *db);
+extern int UdmSQLExecDirect(UDM_DB *db, UDM_SQLRES *R, const char *query);
+#endif
 
 /*
 extern int UdmOracle8Prepare(UDM_DB *db, const char *query);
@@ -51,7 +65,6 @@ extern int UdmOracle8Exec(UDM_DB *db);
 
 #define UdmSQLQuery(db, R, query) _UdmSQLQuery(db, R, query, __FILE__, __LINE__)
 
-
 #define UDM_SQLMON_DISPLAY_FIELDS	1
 #define UDM_SQLMON_MSG_ERROR		1
 #define UDM_SQLMON_MSG_PROMPT		2
