@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 extern int UdmWordListAdd(UDM_DOCUMENT* Doc,char *word,int where);
 extern int UdmWordListFree(UDM_WORDLIST * List);
+extern int UdmWordListSaveSectionSize(UDM_DOCUMENT *Doc);
 extern UDM_WORDLIST * UdmWordListInit(UDM_WORDLIST * List);
 
 
@@ -28,8 +29,10 @@ extern void UdmWideWordFree(UDM_WIDEWORD *W);
 
 extern UDM_WIDEWORDLIST * UdmWideWordListInit(UDM_WIDEWORDLIST * List);
 extern size_t UdmWideWordListAdd(UDM_WIDEWORDLIST * List,UDM_WIDEWORD * Word);
+extern size_t UdmWideWordListAddLike(UDM_WIDEWORDLIST *WWList,
+                                     UDM_WIDEWORD *orig, char *word);
+extern size_t UdmWideWordListAddForStat(UDM_WIDEWORDLIST * List,UDM_WIDEWORD * Word);
 extern void UdmWideWordListFree(UDM_WIDEWORDLIST * List);
-
-extern __C_LINK int __UDMCALL UdmResCollectWords(UDM_RESULT *, UDM_URLCRDLISTLIST *);
+extern int UdmWideWordListCopy(UDM_WIDEWORDLIST *Dst, UDM_WIDEWORDLIST *Src);
 
 #endif
