@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -47,7 +47,9 @@ __C_LINK UDM_AGENT * __UDMCALL UdmAgentInit(UDM_AGENT *result,UDM_ENV * Env, int
 	result->LangMap = (UDM_LANGMAP*)UdmMalloc(sizeof(UDM_LANGMAP));
 	bzero((void*)result->LangMap, sizeof(UDM_LANGMAP));
 #ifdef USE_TRACE
-	udm_snprintf(filename, sizeof(filename), "/tmp/udm_agent.%d.trace", handle);
+	udm_snprintf(filename, sizeof(filename), "%s/udm_agent.%d.trace",
+                     UdmVarListFindStr(&Env->Vars, "TmpDir", UDM_TMP_DIR),
+	             handle);
 	result->TR = fopen(filename, "w");
 	if (!result->TR)
 	{
@@ -78,3 +80,9 @@ __C_LINK void __UDMCALL UdmAgentSetAction(UDM_AGENT *Indexer,int action){
 	Indexer->action = action;
 }
 
+
+void UdmAgentSetTask(UDM_AGENT *Indexer, const char *task)
+{
+  Indexer->State.task= task;
+  Indexer->State.start_time= time(0);
+}
