@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -138,7 +138,7 @@ int UdmMethod(const char *s)
 
 
 
-void UdmWeightFactorsInit(const char *wf, int *res)
+void UdmWeightFactorsInit(char *res, const char *wf, size_t num)
 {
   size_t len;
   int sn;
@@ -154,9 +154,24 @@ void UdmWeightFactorsInit(const char *wf, int *res)
     for(sec=wf+len-1; sec >= wf; sec--)
       res[len-(sec-wf)]= UdmHex2Int(*sec);
   }
+  
+  for (sn= num + 1 ; sn < 256; sn++)
+    res[sn]= 0;
 }
 
 
+size_t UdmWeightFactorsInit2(char *res,
+                             UDM_VARLIST *V1,
+                             UDM_VARLIST *V2,
+                             const char *name)
+{
+  size_t numsections= UdmVarListFindInt(V1, "NumSections", 256);
+  const char *wf1= UdmVarListFindStr(V1, name, "");
+  const char *wf2= UdmVarListFindStr(V2, name, "");
+  const char *wf3= wf2[0] ? wf2 : wf1;
+  UdmWeightFactorsInit(res, wf3, numsections);
+  return wf3[0] ? numsections : 0;
+}
 
 /* hold the path of the current config file */
 static char current[1024] = "";
@@ -462,7 +477,7 @@ static int add_filter(void *Cfg, size_t ac,char **av)
       M.arg = av[0];
       M.pattern = av[i];
       
-      if(UDM_OK != UdmMatchListAddWithServer(C->Indexer, &Conf->Filters, &M, 
+      if(UDM_OK != UdmMatchListAddWithServer(NULL, &Conf->Filters, &M, 
                                              err, sizeof(err), ++C->ordre))
       {
         udm_snprintf(Conf->errstr,sizeof(Conf->errstr)-1,"%s",err);
@@ -626,7 +641,7 @@ static int add_section(void *Cfg, size_t ac,char **av)
   cdon= strncasecmp(av[1], "url", 3) ? 1 : 0;
   S.section= atoi(av[2]);
   S.maxlen= atoi(av[3]);
-
+  
   av+= 4;
   ac-= 4;
   
@@ -636,6 +651,12 @@ static int add_section(void *Cfg, size_t ac,char **av)
       cdon= 1;
     else if (!strcasecmp(av[0], "cdoff") || !strcasecmp(av[0], "NoDetectClones"))
       cdon= 0;
+    else if (!strcasecmp(av[0], "html"))
+      S.flags|= UDM_VARFLAG_HTMLSOURCE;
+    else if (!strcasecmp(av[0], "wiki"))
+      S.flags|= UDM_VARFLAG_HTMLSOURCE | UDM_VARFLAG_WIKI;
+    else if (!strcasecmp(av[0], "text"))
+      /* do nothing */;
     else if (!strcasecmp(av[0], "afterheaders"))
       SectionMatch= &Conf->SectionHdrMatch;
     else if (!strcasecmp(av[0], "afterguesser"))
@@ -645,7 +666,7 @@ static int add_section(void *Cfg, size_t ac,char **av)
     else
       break;
   }
-  S.flags= cdon ? 0 : UDM_VARFLAG_NOCLONE;
+  S.flags|= cdon ? 0 : UDM_VARFLAG_NOCLONE;
 
   if (ac == 0)
   {
@@ -965,6 +986,23 @@ static int dblist_free(void *Cfg, size_t ac,char **av)
   return UDM_OK;
 }
 
+
+static int
+env_rpl_casefolding(void *Cfg, size_t ac, char **av)
+{
+  UDM_CFG  *C=(UDM_CFG*)Cfg;
+  UDM_ENV  *Conf = C->Indexer->Conf;
+  UDM_UNIDATA  *unidata;
+  if(!(unidata= UdmUnidataGetByName(av[1])))
+  {
+    sprintf(Conf->errstr,"CaseFolding '%s' is not supported", av[1]);
+    return UDM_ERROR;
+  }
+  Conf->unidata= unidata;
+  return UDM_OK;
+}
+
+
 static int env_rpl_charset(void *Cfg, size_t ac,char **av)
 {
   UDM_CFG  *C=(UDM_CFG*)Cfg;
@@ -1269,8 +1307,9 @@ static UDM_CONFCMD commands[] =
   {"ImportEnv",            1,1,   env_rpl_env_var},
   {"DBAddr",               1,1,   env_rpl_var},
   {"DefaultContentType",   1,1,   env_rpl_var},
+  {"ResultContentType",    1,1,   env_rpl_var},
   {"Listen",               1,1,   env_rpl_var},
-  {"UseRemoteContentType", 1,1,   env_rpl_var},
+  {"UseRemoteContentType", 1,1,   env_rpl_bool_var},
   {"UseCRC32URLId",        1,1,   env_rpl_var},
   {"UseCRC32SiteId",       1,1,   env_rpl_var},
   {"NewsExtensions",       1,1,   env_rpl_var},
@@ -1278,7 +1317,7 @@ static UDM_CONFCMD commands[] =
   {"SyslogFacility",       1,1,   env_rpl_var},
   {"AliasProg",            1,1,   env_rpl_var},
   {"ForceIISCharset1251",  1,1,   env_rpl_var},
-  {"GroupBySite",          1,1,   env_rpl_var},
+  {"GroupBySite",          1,1,   env_rpl_bool_var},
   {"Cache",                1,1,   env_rpl_var},
   {"wf",                   1,1,   env_rpl_var},
   {"StrictModeThreshold",  1,1,   env_rpl_var},
@@ -1317,12 +1356,15 @@ static UDM_CONFCMD commands[] =
   {"NumWordFactor",        1,1,   env_rpl_var},
   {"NumDistinctWordFactor",1,1,   env_rpl_num_var},
   {"UserScoreFactor",      1,1,   env_rpl_num_var},
+  {"WordDensityFactor"    ,1,1,   env_rpl_num_var},
+  {"WordFormFactor"       ,1,1,   env_rpl_num_var},
   {"URLDataThreshold",     1,1,   env_rpl_num_var},
   {"DocSizeWeight",        1,1,   env_rpl_num_var},
   {"RelevancyFactor",      1,1,   env_rpl_num_var},
   {"DateFactor",           1,1,   env_rpl_num_var},
   {"MinWordLength",        1,1,   env_rpl_num_var},
   {"MaxWordLength",        1,1,   env_rpl_num_var},
+  {"SubstringMatchMinWordLength", 1,1, env_rpl_num_var},
   {"ExcerptSize",          1,1,   env_rpl_num_var},
   {"ExcerptPadding",       1,1,   env_rpl_num_var},
   {"LogLevel",             1,1,   env_rpl_num_var},
@@ -1334,10 +1376,17 @@ static UDM_CONFCMD commands[] =
   {"ExcerptStopword",      1,1,   env_rpl_bool_var},
   {"UseCookie",            1,1,   env_rpl_bool_var},
   {"UseNumericOperators",  1,1,   env_rpl_bool_var},
+  {"SaveSectionSize",      1,1,   env_rpl_bool_var},
+  {"Dehyphenate",          1,1,   env_rpl_bool_var},
+  {"StripAccents",         1,1,   env_rpl_bool_var},
+  {"LoadURLInfo",          1,1,   env_rpl_bool_var},
+  {"LoadTagInfo",          1,1,   env_rpl_bool_var},
+
   {"ReplaceVar",           2,2,   env_rpl_named_var},
 
   {"LocalCharset",         1,1,   env_rpl_charset},
   {"BrowserCharset",       1,1,   env_rpl_charset},
+  {"CaseFolding",          1,1,   env_rpl_casefolding},
 
   {"XMLEnterHook",         2,2,   rpl_xml_hook},
   {"XMLLeaveHook",         2,2,   rpl_xml_hook},
@@ -1374,6 +1423,7 @@ static UDM_CONFCMD commands[] =
   {"ServerWeight",         1,1,   srv_rpl_num_var},
   {"Robots",               1,1,   srv_rpl_bool_var},
   {"DetectClones",         1,1,   srv_rpl_bool_var},
+  {"CollectLinks",         1,1,   srv_rpl_bool_var},
   {"Index",                1,1,   srv_rpl_bool_var},
   {"FollowSymLinks",       1,1,   srv_rpl_bool_var},
   {"NetErrorDelayTime",    1,1,   srv_rpl_time_var},
@@ -1397,6 +1447,7 @@ static UDM_CONFCMD commands[] =
   {"CheckOnly",            1,100, add_filter},
   {"HrefOnly",             1,100, add_filter},
   {"ImportOnly",           1,100, add_filter},
+  {"Skip",                 1,100, add_filter},
 
   {"IndexIf",              1,100, add_section_filter},
   {"NoIndexIf",            1,100, add_section_filter},
