@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -74,6 +74,8 @@
 #include "udm_xmalloc.h"
 #include "udm_searchcache.h"
 #include "udm_store.h"
+#include "udm_match.h"
+#include "udm_word.h"
 
 #define UDM_THREADINFO(A,s,m)	if(A->Conf->ThreadInfo)A->Conf->ThreadInfo(A,s,m)
 
@@ -111,8 +113,6 @@ void UdmDBFree(void *vdb)
   UDM_FREE(db->where);
   UDM_FREE(db->from);
   
-  if (db->searchd) UdmSearchdClose(db);
-  
 #if HAVE_SQL
   if (db->connected) UdmSQLClose(db);
 #endif
@@ -123,34 +123,6 @@ void UdmDBFree(void *vdb)
 }
 
 
-
-__C_LINK int __UDMCALL UdmLimit8(UDM_ENV *Conf,
-                                 UDM_UINT8URLIDLIST *L,
-                                 const char *field,int type, void *vdb)
-{
-  UDM_DB  *db=vdb;
-  int  rc=UDM_OK;
-#ifdef HAVE_SQL
-  rc=UdmLimit8SQL(Conf, L, field, type, db);
-#endif
-  strcpy(Conf->errstr, db->errstr);
-  return rc;
-}
-
-__C_LINK int __UDMCALL UdmLimit4(UDM_ENV *Conf,
-                                 UDM_UINT4URLIDLIST *L,
-                                 const char *field, int type, void *vdb)
-{
-  UDM_DB  *db=vdb;
-  int  rc=UDM_OK;
-  
-#ifdef HAVE_SQL
-  rc=UdmLimit4SQL(Conf, L, field, type, db);
-#endif
-  strcpy(Conf->errstr, db->errstr);
-  return rc;
-}
-
 /*
 int UdmURLData(UDM_ENV *Conf, UDM_URLDATALIST *L, UDM_DB *db)
 {
@@ -242,7 +214,7 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
   int        rc= UDM_OK;
   int        status=UdmVarListFindInt(&Doc->Sections,"Status",0);
   urlid_t    url_id = (urlid_t)UdmVarListFindInt(&Doc->Sections, "ID", 0);
-  time_t     next_index_time;
+  time_t     next_index_time= time(NULL) + Doc->Spider.period;
   char       dbuf[64];
   int        use_newsext; 
   int        action;
@@ -250,9 +222,14 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
   UDM_LOCK_CHECK_OWNER(Indexer, UDM_LOCK_CONF);
 
   use_newsext= !strcasecmp(UdmVarListFindStr(&Indexer->Conf->Vars,"NewsExtensions","no"),"yes");
+
+  if (Doc->method == UDM_METHOD_VISITLATER)
+  {
+    action= UDM_URL_ACTION_SUPDATE;
+    goto update;
+  }
   
   /* First of all check that URL must be delated */
-  
   if(Doc->method==UDM_METHOD_DISALLOW)
   {
     UdmLog(Indexer,UDM_LOG_ERROR,"Deleting %s", UdmVarListFindStr(&Doc->Sections, "URL", ""));
@@ -260,11 +237,10 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
     return rc;
   }
 
-  next_index_time=time(NULL)+Doc->Spider.period;
   switch(status){
   
   case 0: /* No HTTP code */
-    if (Doc->connp.Host != NULL) Doc->connp.Host->net_errors++;
+    UdmHostErrorIncrement(&Indexer->Conf->Hosts, Doc->connp.hostname);
     UdmLog(Indexer,UDM_LOG_ERROR,"No HTTP response status");
     next_index_time=time(NULL)+Doc->Spider.net_error_delay_time;
     action= UDM_URL_ACTION_SUPDATE;
@@ -277,13 +253,13 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
       UdmLog(Indexer,UDM_LOG_ERROR,"No Content-type header");
       next_index_time=time(NULL)+Doc->Spider.net_error_delay_time;
       UdmVarListReplaceInt(&Doc->Sections,"Status",UDM_HTTP_STATUS_INTERNAL_SERVER_ERROR);
-      if (Doc->connp.Host != NULL) Doc->connp.Host->net_errors++;
+      UdmHostErrorIncrement(&Indexer->Conf->Hosts, Doc->connp.hostname);
       action= UDM_URL_ACTION_SUPDATE;
       break;
     }
     else
     {
-      if (Doc->connp.Host != NULL) Doc->connp.Host->net_errors = 0;
+      UdmHostErrorReset(&Indexer->Conf->Hosts, Doc->connp.hostname);
 
       /* Check clones */
       if(Doc->method == UDM_METHOD_GET && Doc->Spider.use_clones)
@@ -364,7 +340,7 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
   
     /* Keep words in database                */
     /* We'll retry later, maybe host is down */
-    if (Doc->connp.Host != NULL) Doc->connp.Host->net_errors++;
+    UdmHostErrorIncrement(&Indexer->Conf->Hosts, Doc->connp.hostname);
     next_index_time=time(NULL)+Doc->Spider.net_error_delay_time;
     action= UDM_URL_ACTION_SUPDATE;
     break;
@@ -375,6 +351,8 @@ static int DocUpdate(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc)
     break;
   }
 
+update:
+
   UdmTime_t2HttpStr(next_index_time,dbuf);
   UdmVarListReplaceStr(&Doc->Sections,"Next-Index-Time",dbuf);  
   rc= UdmURLActionNoLock(Indexer, Doc, action);
@@ -469,8 +447,10 @@ int UdmURLActionNoLock(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd)
   dbto =  A->Conf->dbl.nitems;
   if (dbnum < 0 && D != NULL)
   {
-    udmhash32_t  url_id=UdmVarListFindInt(&D->Sections,"URL_ID", 0);
-    dbfrom = dbto = ((url_id) ? url_id : UdmStrHash32(UdmVarListFindStr(&D->Sections, "URL", ""))) % A->Conf->dbl.nitems;
+    udmhash32_t url_id=UdmVarListFindInt(&D->Sections,"URL_ID", 0);
+    udmhash32_t seed= (((url_id) ? url_id :
+                       UdmStrHash32(UdmVarListFindStr(&D->Sections, "URL", ""))) & 0xFF);
+    dbfrom= dbto= seed * A->Conf->dbl.nitems / 256;
     dbto++;
   }
 
@@ -559,7 +539,7 @@ __C_LINK int __UDMCALL UdmTargets(UDM_AGENT *A)
 
 __C_LINK int __UDMCALL UdmResAction(UDM_AGENT *A, UDM_RESULT *R, int cmd)
 {
-  int  res=UDM_ERROR;
+  int  rc= UDM_ERROR;
   UDM_DB  *db;
   size_t i, dbfrom = 0, dbto;
   
@@ -569,17 +549,25 @@ __C_LINK int __UDMCALL UdmResAction(UDM_AGENT *A, UDM_RESULT *R, int cmd)
   {
     db = &A->Conf->dbl.db[i];
     UDM_GETLOCK(A, UDM_LOCK_DB);
+    switch(db->DBDriver)
+    {
+      case UDM_DB_SEARCHD:
+        rc= UDM_OK;
+        break;
+
 #ifdef HAVE_SQL
-    res = UdmResActionSQL(A, R, cmd, db, i);
+      default:
+        rc= UdmResActionSQL(A, R, cmd, db, i);
 #endif
-    if(res != UDM_OK)
+    }
+    if(rc != UDM_OK)
     {
       UdmLog(A, UDM_LOG_ERROR, db->errstr);
     }
     UDM_RELEASELOCK(A, UDM_LOCK_DB);
-    if (res != UDM_OK) break;
+    if (rc != UDM_OK) break;
   }
-  return res;
+  return rc;
 }
 
 
@@ -626,7 +614,6 @@ __C_LINK int __UDMCALL UdmSrvAction(UDM_AGENT *A, UDM_SERVERLIST *S, int cmd)
   UDM_GETLOCK(A, UDM_LOCK_CONF);
   dbto =  A->Conf->dbl.nitems;
 
-  strcpy(A->Conf->errstr, "No appropriate storage support compiled");
   for (i = dbfrom; i < dbto; i++)
   {
     db = &A->Conf->dbl.db[i];
@@ -647,8 +634,25 @@ __C_LINK int __UDMCALL UdmSrvAction(UDM_AGENT *A, UDM_SERVERLIST *S, int cmd)
 
 static const int search_cache_size=1000;
 
-static
-int UdmFindWordsDB(UDM_AGENT *A, UDM_RESULT *CurRes, UDM_DB *db)
+/* Allocate an array for documents information */
+static int
+UdmResAllocDoc(UDM_RESULT *Res)
+{
+  size_t i;
+  if (!Res->Doc && Res->num_rows > 0)
+  {
+    Res->Doc = (UDM_DOCUMENT*)UdmMalloc(sizeof(UDM_DOCUMENT) * (Res->num_rows));
+    for (i= 0; i < Res->num_rows; i++)
+      UdmDocInit(&Res->Doc[i]);
+  }
+  return UDM_OK;
+}
+
+
+
+static int
+UdmFindWordsDB(UDM_AGENT *A, UDM_RESULT *CurRes, UDM_DB *db,
+               size_t num_best_rows)
 {
   int rc= UDM_OK;
   const char *dbaddr= UdmVarListFindStr(&db->Vars,"DBAddr","<noaddr>");
@@ -664,10 +668,10 @@ int UdmFindWordsDB(UDM_AGENT *A, UDM_RESULT *CurRes, UDM_DB *db)
     default:
       if (UDM_OK != (rc= UdmQueryCacheGetSQL(A, CurRes, db)))
         return rc;
-      if (!CurRes->CoordList.ncoords)
+      if (!CurRes->URLData.nitems)
       {
-        rc= UdmFindWordsSQL(A, CurRes, db);
-        if (rc == UDM_OK && CurRes->CoordList.ncoords)
+        rc= UdmFindWordsSQL(A, CurRes, db, num_best_rows);
+        if (rc == UDM_OK && CurRes->URLData.nitems)
           rc= UdmQueryCachePutSQL(A, CurRes, db);
       }
       break;
@@ -683,6 +687,7 @@ typedef struct search_param_st
   UDM_DB *db;
   int rc;
   void *thd;
+  size_t num_best_rows;
 } UDM_SPRM;
 
 
@@ -690,24 +695,24 @@ static
 void *SearchThread(void *arg)
 {
   UDM_SPRM *sprm= (UDM_SPRM*) arg;
-  sprm->rc= UdmFindWordsDB(sprm->Agent, sprm->Res, sprm->db);
+  sprm->rc= UdmFindWordsDB(sprm->Agent, sprm->Res, sprm->db, sprm->num_best_rows);
   return NULL;
 }
 
 
 static int
-UdmResultSetMachineNumber(UDM_RESULT *Res, size_t num)
+UdmResultSetMachineNumber(UDM_URLDATALIST *List, size_t num)
 {
   size_t i;
-  UDM_URL_CRD *Coords= Res->CoordList.Coords;
+  UDM_URLDATA *Item= List->Item;
 
-  for(i= 0; i < Res->total_found; i++)
+  for(i= 0; i < List->nitems; i++)
   {
     /* 
       We use (256-i) to sort a document from the first database
       before the same document from the second database.
     */
-    Coords[i].coord = (Coords[i].coord << 8) + (255 - (num & 255));
+    Item[i].coord = (Item[i].coord << 8) + (255 - (num & 255));
    }
   return UDM_OK;
 }
@@ -715,72 +720,70 @@ UdmResultSetMachineNumber(UDM_RESULT *Res, size_t num)
 
 static int
 UdmResultJoin(UDM_AGENT *A,
-              UDM_RESULT *TmpRes, size_t dbfrom, size_t dbto,
+              UDM_RESULT *TmpRes, size_t nresults,
               UDM_RESULT *Res)
 {
-  size_t i;
+  size_t i, max_total_found= 0;
   unsigned long ticks;
   
-  for (Res->total_found= 0, i= dbfrom; i < dbto; i++)
-    Res->total_found+= TmpRes[i].total_found;
-
-  Res->num_rows= Res->CoordList.ncoords= Res->total_found;
-  if (dbto - dbfrom == 1)
+  for (Res->URLData.nitems= 0,
+       Res->num_rows= 0,
+       Res->total_found= 0, i= 0; i < nresults; i++)
   {
-    Res->CoordList.Coords= TmpRes[dbfrom].CoordList.Coords;
-    Res->CoordList.Data= TmpRes[dbfrom].CoordList.Data;
-    UdmResultSetMachineNumber(Res, dbfrom);
-    return UDM_OK;
+    size_t j;
+    if (max_total_found < TmpRes[i].total_found)
+      max_total_found= TmpRes[i].total_found;
+    Res->total_found+= TmpRes[i].total_found;
+    Res->num_rows+= TmpRes[i].num_rows;
+    Res->URLData.nitems+= TmpRes[i].URLData.nitems;
+    for (j= 0; j < TmpRes[i].WWList.nwords; j++)
+      UdmWideWordListAddForStat(&Res->WWList, &TmpRes[i].WWList.Word[j]);
   }
 
   ticks= UdmStartTimer();
-  UdmLog(A, UDM_LOG_DEBUG, "Start joining results from %d dbs", dbto - dbfrom);
+  UdmLog(A, UDM_LOG_DEBUG, "Start joining results from %d dbs", nresults);
 
-  if (Res->total_found > 0)
+  if (Res->URLData.nitems > 0)
   {
     size_t offs;
 
-    Res->CoordList.Coords= (UDM_URL_CRD*)UdmMalloc(sizeof(UDM_URL_CRD)
-                                                   * Res->total_found);
-    Res->CoordList.Data= (UDM_URLDATA*)UdmMalloc(sizeof(UDM_URLDATA)
-                                                 * Res->total_found);
-    for (offs= 0, i= dbfrom; i < dbto; offs+= TmpRes[i].total_found, i++)
+    Res->URLData.Item= (UDM_URLDATA*)UdmMalloc(sizeof(UDM_URLDATA)
+                                               * Res->URLData.nitems);
+    for (offs= 0, i= 0; i < nresults; offs+= TmpRes[i].URLData.nitems, i++)
     {
       UDM_RESULT  *CurRes= &TmpRes[i];
-      UDM_URL_CRD *CurCoords= TmpRes[i].CoordList.Coords;
-      if(CurCoords)
+      UDM_URLDATA *CurData= TmpRes[i].URLData.Item;
+      size_t ncoords= CurRes->URLData.nitems;
+      if(CurData)
       {
-        UdmResultSetMachineNumber(CurRes, i);
-        memcpy(Res->CoordList.Coords + offs,CurCoords,
-               sizeof(CurCoords[0])*CurRes->total_found);
-        if (CurRes->CoordList.Data != NULL)
-        {
-          memcpy(Res->CoordList.Data + offs, CurRes->CoordList.Data,
-                 sizeof(Res->CoordList.Data[0]) * CurRes->total_found);
-        }
-        else
-        {
-          bzero(Res->CoordList.Data + offs,
-                sizeof(Res->CoordList.Data[0]) * CurRes->total_found);
-        }
+        UdmResultSetMachineNumber(&CurRes->URLData, i);
+        memcpy(Res->URLData.Item + offs, CurData,
+               sizeof(CurData[0]) * ncoords);
       }
-      UDM_FREE(CurRes->CoordList.Coords);
-      UDM_FREE(CurRes->CoordList.Data);
+      UDM_FREE(CurRes->URLData.Item);
     }
   }
 
-  if (dbto - dbfrom > 1)
+  if (nresults > 1)
   {
-    int use_site_id = ((!strcasecmp(UdmVarListFindStr(&A->Conf->Vars, "GroupBySite", "no"), "yes"))
-                        && (UdmVarListFindInt(&A->Conf->Vars, "site", 0) == 0));
+    int use_site_id= UdmVarListFindBool(&A->Conf->Vars, "GroupBySite", 0) &&
+                     (UdmVarListFindInt(&A->Conf->Vars, "site", 0) == 0);
     if (use_site_id)
     {
-      UdmSortSearchWordsBySite(&Res->CoordList, Res->CoordList.ncoords);
-      UdmGroupBySite(A, Res);
+      UdmURLDataSortBySite(&Res->URLData);
+      UdmURLDataGroupBySite(&Res->URLData);
+      /*
+        TODO: add better approximation of total results
+        found with cluster. This number will be between
+        max_total_found and Res->total_found, and will
+        depend on overlap between the sites:
+        (Res->URLData.nitems / Res->total_found)
+      */
+      Res->total_found= Res->URLData.nitems;
     }
-    UdmSortSearchWordsByPattern(Res, &Res->CoordList, Res->CoordList.ncoords,
-                                UdmVarListFindStr(&A->Conf->Vars, "s", "RP"));
-    Res->num_rows= Res->total_found= Res->CoordList.ncoords;
+    if (Res->URLData.nitems)
+      UdmURLDataSortByPattern(&Res->URLData,
+                              UdmVarListFindStr(&A->Conf->Vars, "s", "R"));
   }
 
   ticks= UdmStartTimer() - ticks;
@@ -790,68 +793,104 @@ UdmResultJoin(UDM_AGENT *A,
 }
 
 
-int UdmFindWords(UDM_AGENT *A, UDM_RESULT *Res)
+static int
+UdmFindWordsMulDB(UDM_AGENT *A, UDM_RESULT *TmpRes,
+                  UDM_RESULT *Res, size_t num_best_rows)
+{
+  size_t i, ndatabases= A->Conf->dbl.nitems;
+  UDM_SPRM search_param[256];
+  int use_threads= A->Conf->ThreadCreate != NULL && ndatabases > 1;
+  int rc= UDM_OK;
+  
+  /* Check if all drivers are thread safe */
+  for (i= 0; i < ndatabases; i++)
+  {
+    UDM_DB *db = &A->Conf->dbl.db[i];
+    /* TODO: link against libmysqlclient_r */
+    if (db->DBDriver != UDM_DB_SEARCHD)
+    {
+      use_threads= 0;
+      break;
+    }
+  }
+    
+  for (i= 0; i < ndatabases; i++)
+  {
+    UDM_RESULT *CurRes= &TmpRes[i];
+    UDM_DB *db = &A->Conf->dbl.db[i];
+    CurRes[0]= Res[0];    /* Copy bool items */
+    UdmWideWordListCopy(&CurRes->WWList, &Res->WWList);
+
+    if (use_threads)
+    {
+      void *thd;
+      UdmLog(A, UDM_LOG_DEBUG, "Starting thread[%d]", i + 1);
+      search_param[i].Agent= UdmAgentInit(NULL, A->Conf, i + 1);
+      search_param[i].Res= CurRes;
+      search_param[i].db= db;
+      search_param[i].num_best_rows= num_best_rows;
+      A->Conf->ThreadCreate(&thd, SearchThread, (void*) &search_param[i]);
+      search_param[i].thd= thd;
+    }
+    else
+    {
+      if (UDM_OK != (rc= UdmFindWordsDB(A, CurRes, db, num_best_rows)))
+        return rc;
+    }
+  }
+    
+  if (use_threads)
+  {
+    unsigned long long ticks= UdmStartTimer();
+    UdmLog(A, UDM_LOG_DEBUG, "Creating threads");
+    for (i= 0; i < ndatabases; i++)
+    {
+      void *thd= search_param[i].thd;
+      UdmLog(A, UDM_LOG_DEBUG, "Joining thread[%d]", i + 1);
+      A->Conf->ThreadJoin(thd);
+    }
+    ticks= UdmStartTimer() - ticks;
+    UdmLog(A, UDM_LOG_DEBUG, "Threads finished: %.2f", (float) ticks/1000);
+  }
+  
+  if (use_threads)
+  {
+    for (i= 0; i < ndatabases; i++)
+      UdmAgentFree(search_param[i].Agent);
+  }
+
+  UdmResultJoin(A, TmpRes, ndatabases, Res);
+  
+  return rc;
+}
+
+
+static int
+UdmFindWords(UDM_AGENT *A, UDM_RESULT *TmpRes,
+             UDM_RESULT *Res, size_t num_best_rows)
 {
   const char  *cache_mode= UdmVarListFindStr(&A->Conf->Vars, "Cache", "no");
-  size_t      i, dbfrom= 0, dbto= A->Conf->dbl.nitems;
   int res=    UDM_OK;
   size_t      ResultsLimit = UdmVarListFindUnsigned(&A->Conf->Vars, "ResultsLimit", 0);
 
   if( strcasecmp(cache_mode, "yes") || UdmSearchCacheFind(A, Res))
   {
-    /* If not found in search cache       */
-    /* Let's do actual search now:        */
-    /* Get groupped by url_id words array */
-    UDM_RESULT TmpRes[256];
-    UDM_SPRM search_param[256];
-    int use_threads= A->Conf->ThreadCreate != NULL && dbfrom + 1 != dbto;
-
-    use_threads= 0; /* TODO: link against libmysqlclient_r */
-    
-    for (i= dbfrom; i < dbto; i++)
+    if (A->Conf->dbl.nitems > 1)
     {
-      UDM_RESULT *CurRes= &TmpRes[i];
-      UDM_DB *db = &A->Conf->dbl.db[i];
-      CurRes[0]= Res[0];    /* Copy WWList and bool items */
-
-      if (use_threads)
-      {
-        void *thd;
-        UdmLog(A, UDM_LOG_DEBUG, "Starting thread[%d]", i + 1);
-        search_param[i].Agent= UdmAgentInit(NULL, A->Conf, i + 1);
-        search_param[i].Res= CurRes;
-        search_param[i].db= db;
-        A->Conf->ThreadCreate(&thd, SearchThread, (void*) &search_param[i]);
-        search_param[i].thd= thd;
-      }
-      else
-      {
-        if (UDM_OK != (res= UdmFindWordsDB(A, CurRes, db)))
-          return res;
-      }
+      if (UDM_OK  != (res= UdmFindWordsMulDB(A, TmpRes, Res, num_best_rows)))
+        return res;
     }
-    
-    if (use_threads)
+    else
     {
-      for (i= dbfrom; i < dbto; i++)
-      {
-        void *thd= search_param[i].thd;
-        UdmLog(A, UDM_LOG_DEBUG, "Joining thread[%d]", i + 1);
-        A->Conf->ThreadJoin(thd);
-      }
-    }
-    
-    if (use_threads)
-    {
-      for (i= dbfrom; i < dbto; i++)
-        UdmAgentFree(search_param[i].Agent);
+      UDM_DB *db = &A->Conf->dbl.db[0];
+      if (UDM_OK != (res= UdmFindWordsDB(A, Res, db, num_best_rows)))
+        return res;
+      UdmResultSetMachineNumber(&Res->URLData, 0);
     }
 
-    UdmResultJoin(A, TmpRes, dbfrom, dbto, Res);
-
-    if (ResultsLimit > 0 && ResultsLimit < Res->total_found)
+    if (ResultsLimit > 0 && ResultsLimit < Res->URLData.nitems)
     {
-      Res->total_found = Res->CoordList.ncoords = ResultsLimit;
+      Res->URLData.nitems= ResultsLimit;
     }
 
     if((!strcasecmp(cache_mode,"yes"))&&(search_cache_size>-1))
@@ -865,12 +904,37 @@ int UdmFindWords(UDM_AGENT *A, UDM_RESULT *Res)
 }
 
 
+static int CreateAlias(UDM_ENV *Conf, UDM_DOCUMENT *Doc)
+{
+  char *alcopy, *aliastr;
+  UDM_MATCH    *Alias;
+  UDM_MATCH_PART Parts[10];
+
+  /* Create "Alias" variable */
+  alcopy=UdmRemoveHiLightDup(UdmVarListFindStr(&Doc->Sections,"URL",""));
+  if ((Alias=UdmMatchListFind(&Conf->Aliases,alcopy,10,Parts)))
+  {
+    size_t nbytes= strlen(Alias->arg) + strlen(alcopy) + 64;
+    aliastr= (char*)UdmMalloc(nbytes);
+    UdmMatchApply(aliastr, nbytes, alcopy, Alias->arg, Alias, 10, Parts);
+  }
+  else
+  {
+    aliastr = (char*)UdmStrdup(alcopy);
+  }
+  UdmVarListReplaceStr(&Doc->Sections,"Alias", aliastr);
+  UdmFree(aliastr);
+  UdmFree(alcopy);
+  return UDM_OK;
+}
+
+
 UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
 {
-  UDM_RESULT  *Res;
+  UDM_RESULT  *Res, *TmpRes;
   int    res=UDM_OK;
   unsigned long  ticks=UdmStartTimer(), ticks_;
-  size_t  i, dbfrom= 0, dbto=  A->Conf->dbl.nitems;
+  size_t  i, nbytes, numdatabases=  A->Conf->dbl.nitems;
   size_t  page_number= (size_t) UdmVarListFindInt(&A->Conf->Vars, "np", 0);
   size_t  page_size=   (size_t) UdmVarListFindInt(&A->Conf->Vars, "ps", 10);
   size_t  offs=        (size_t) UdmVarListFindInt(&A->Conf->Vars, "offs", 0);
@@ -880,13 +944,27 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
 
   UdmLog(A,UDM_LOG_DEBUG,"Start UdmFind");
   
+  ticks_= UdmStartTimer();
+  UdmLog(A, UDM_LOG_DEBUG, "Start Prepare");  
+  nbytes= sizeof(UDM_RESULT) * numdatabases;
+  TmpRes= (UDM_RESULT*) UdmMalloc(nbytes);
+  bzero((void*) TmpRes, sizeof(UDM_RESULT) * numdatabases);
   Res=UdmResultInit(NULL);
-  UdmPrepare(A, Res);
-  UdmVarListAddStr(&A->Conf->Vars, "orig_m", UdmVarListFindStr(&A->Conf->Vars, "m", "all"));
-
-  if (!Res->WWList.nwords ||
-      UDM_OK != (res= UdmFindWords(A, Res)))
+  if (UDM_OK != (res= UdmPrepare(A, Res)))
     goto conv;
+  UdmVarListAddStr(&A->Conf->Vars, "orig_m", UdmVarListFindStr(&A->Conf->Vars, "m", "all"));
+  ticks_= UdmStartTimer() - ticks_;
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  Prepare:\t\t%.2f", (float)ticks_/1000);
+
+  Res->first = offs ? offs : page_number * page_size;  
+
+  ticks_= UdmStartTimer();
+  UdmLog(A, UDM_LOG_DEBUG, "Start FindWords");  
+  if (!Res->WWList.nwords ||
+      UDM_OK != (res= UdmFindWords(A, TmpRes, Res, Res->first + page_size)))
+    goto conv;
+  ticks_= UdmStartTimer() - ticks_;
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  FindWords:\t\t%.2f", (float)ticks_/1000);
   
   if (!Res->total_found)
   {
@@ -902,47 +980,34 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   UdmVarListDel(&A->Conf->Vars, "orig_m");
   UdmVarListReplaceInt(&A->Conf->Vars, "CurrentTimestamp", (int) time(0));
 
-  Res->first = offs ? offs : page_number * page_size;  
-  if(Res->first >= Res->total_found)
+
+  if(Res->first >= Res->URLData.nitems)
   {
-    /*Res->first = (Res->total_found)? (Res->total_found - 1) : 0;*/
     Res->last= Res->first;
     Res->num_rows= 0;
     goto conv; /* jump to converting variables into BrowserCharset */
   }
-  if((Res->first+page_size)>Res->total_found)
+  if(Res->first + page_size > Res->URLData.nitems)
   {
-    Res->num_rows=Res->total_found-Res->first;
+    Res->num_rows= Res->URLData.nitems - Res->first;
   }
   else
   {
-    Res->num_rows=page_size;
+    Res->num_rows= page_size;
   }
-  Res->last=Res->first+Res->num_rows-1;
+  Res->last= Res->first + Res->num_rows - 1;
 
-  /* Allocate an array for documents information */
-  if (Res->num_rows > 0) Res->Doc = (UDM_DOCUMENT*)UdmMalloc(sizeof(UDM_DOCUMENT) * (Res->num_rows));
+  ticks_= UdmStartTimer();
+  UdmLog(A, UDM_LOG_DEBUG, "Start AddDocInfo");
+
+  UdmResAllocDoc(Res);
   
-  /* Copy url_id and coord to result */
-  for(i=0;i<Res->num_rows;i++)
-  {
-    uint4  score = Res->CoordList.Coords[i + Res->first /* * Res->offset*/].coord;
-    UdmDocInit(&Res->Doc[i]);
-    UdmVarListReplaceInt(&Res->Doc[i].Sections, "ID", Res->CoordList.Coords[i + Res->first /* * Res->offset*/].url_id);
-    udm_snprintf(str, 128, "%.3f%%", ((double)(score >> 8)) / 1000);
-    UdmVarListReplaceStr(&Res->Doc[i].Sections, "Score", str);
-    UdmVarListReplaceInt(&Res->Doc[i].Sections,"Order",(int)(i + Res->first + 1));
-    UdmVarListReplaceInt(&Res->Doc[i].Sections, "dbnum", 255 - (int)(score & 0xFF));
-    UdmVarListReplaceUnsigned(&Res->Doc[i].Sections, "PerSite",
-                              Res->CoordList.Data[i + Res->first].per_site);
-  }
-  
-  for (i = dbfrom; i < dbto; i++)
+  for (i= 0; i < numdatabases; i++)
   {
     UDM_DB *db= &A->Conf->dbl.db[i];
     switch(db->DBDriver){
     case UDM_DB_SEARCHD:
-      res = UdmResAddDocInfoSearchd(A, db, Res, i);
+      res = UdmResAddDocInfoSearchd(A, TmpRes, db, Res, i);
       break;
 #ifdef HAVE_SQL
     default:
@@ -951,46 +1016,66 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
 #endif
     }
   }
+
+  /* Copy url_id and coord to result */
+  for(i=0;i<Res->num_rows;i++)
+  {
+    UDM_URLDATA *Data= &Res->URLData.Item[i + Res->first];
+    uint4 score= Data->coord;
+    UdmVarListReplaceUnsigned(&Res->Doc[i].Sections, "PerSite", Data->per_site);
+    udm_snprintf(str, 128, "%.3f%%", ((double)(score >> 8)) / 1000);
+    UdmVarListReplaceStr(&Res->Doc[i].Sections, "Score", str);
+    UdmVarListReplaceInt(&Res->Doc[i].Sections,"Order",(int)(i + Res->first + 1));
+    UdmVarListReplaceInt(&Res->Doc[i].Sections, "dbnum", UDM_COORD2DBNUM(score));
+  }
   
-  ticks_=UdmStartTimer();
-  UdmLog(A, UDM_LOG_DEBUG, "Start Clones");
   
-  if(UdmVarListFindInt(&A->Conf->Vars, "DetectClones", 1))
+  for (i= 0; i < Res->num_rows; i++)
+    CreateAlias(A->Conf, &Res->Doc[i]);
+    
+  ticks_= UdmStartTimer() - ticks_;
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  AddDocInfo:\t\t%.2f", (float)ticks_/1000);
+  
+  if (UdmVarListFindInt(&A->Conf->Vars, "DetectClones", 0))
   {
     size_t num=Res->num_rows;
-    for(i=0;i<num;i++){
+    ticks_=UdmStartTimer();
+    UdmLog(A, UDM_LOG_DEBUG, "Start Clones");
+  
+    for(i=0;i<num;i++)
+    {
       UDM_RESULT *Cl = UdmCloneList(A, &Res->Doc[i]);
-      if(Cl){
-        Res->Doc=(UDM_DOCUMENT*)UdmRealloc(Res->Doc,sizeof(UDM_DOCUMENT)*(Res->num_rows+Cl->num_rows));
-        memcpy(&Res->Doc[Res->num_rows],Cl->Doc,sizeof(UDM_DOCUMENT)*Cl->num_rows);
-        Res->num_rows+=Cl->num_rows;
-        UDM_FREE(Cl->Doc);
+      if (Cl)
+      {
+        size_t c;
+        UdmVarListReplaceInt(&Res->Doc[i].Sections, "nclones", Cl->num_rows);
+        for (c= 0; c < Cl->num_rows; c++)
+        {
+          char name[32];
+          sprintf(name, "Clone%d", c);
+          UdmVarListReplaceLst(&Res->Doc[i].Sections,
+                               &Cl->Doc[c].Sections, name, "*");
+        }
         UdmResultFree(Cl);
       }
     }
+    ticks_ = UdmStartTimer() - ticks_;
+    UdmLog(A, UDM_LOG_DEBUG, "Stop  Clones:\t\t%.2f", (float)ticks_/1000);
   }
-  ticks_ = UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  Clones:\t\t%.2f", (float)ticks_/1000);
-  
-  ticks_=UdmStartTimer();
-  UdmLog(A, UDM_LOG_DEBUG, "Start adding Order");
   
   /* first and last begins from 0, make it begin from 1 */
   Res->first++;
   Res->last++;
-  for(i=0;i<Res->num_rows;i++)
-  {
-    UdmVarListReplaceInt(&Res->Doc[i].Sections,"Order",(int)(Res->first+i));
-  }
-  ticks_ = UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  Order:\t\t\t%.2f", (float)ticks_/1000);
   
 conv:
+
+#ifndef NO_ADVANCE_CONV
   ticks_=UdmStartTimer();
   UdmLog(A, UDM_LOG_DEBUG, "Start UdmConvert");
   UdmConvert(A->Conf, Res, A->Conf->lcs, A->Conf->bcs);
   ticks_ = UdmStartTimer() - ticks_;
   UdmLog(A, UDM_LOG_DEBUG, "Stop  UdmConvert:\t\t%.2f", (float)ticks_/1000);
+#endif
 
   ticks_=UdmStartTimer();
   UdmLog(A, UDM_LOG_DEBUG, "Start Excerpts");
@@ -1006,11 +1091,18 @@ conv:
   ticks_ = UdmStartTimer() - ticks_;
   UdmLog(A, UDM_LOG_DEBUG, "Stop  Excerpts:\t\t%.2f", (float)ticks_/1000);
 
+  UdmLog(A, UDM_LOG_DEBUG, "Start WordInfo");
+  ticks_=UdmStartTimer();
   UdmResWordInfo(A->Conf, Res);
-  Res->work_time=ticks=UdmStartTimer()-ticks;
-  UdmLog(A,UDM_LOG_DEBUG,"Done  UdmFind %.2f",(float)ticks/1000);
-  UdmTrack(A, Res);
+  ticks_= UdmStartTimer() - ticks_;
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  WordInfo:\t\t%.2f", (float)ticks_/1000);
 
+  Res->work_time= ticks= UdmStartTimer() - ticks;
+
+  UdmTrack(A, Res);
+  UdmLog(A,UDM_LOG_DEBUG,"Done  UdmFind %.2f",(float)ticks/1000);
+
+  UdmFree(TmpRes);
   if(res!=UDM_OK)
   {
     UdmResultFree(Res);
@@ -1026,6 +1118,7 @@ static int UdmStr2DBMode(const char * str1)
   if(!strncasecmp(str1,"single",6))m=UDM_DBMODE_SINGLE;
   else if(!strncasecmp(str1,"multi",5))m=UDM_DBMODE_MULTI;
   else if(!strncasecmp(str1,"blob",4))m=UDM_DBMODE_BLOB;
+  else if(!strncasecmp(str1,"rawblob",7))m=UDM_DBMODE_RAWBLOB;
   return(m);
 }
 
@@ -1058,6 +1151,7 @@ __C_LINK const char* __UDMCALL UdmDBModeToStr(int dbmode)
     case UDM_DBMODE_SINGLE:  return "single";
     case UDM_DBMODE_MULTI:   return "multi";
     case UDM_DBMODE_BLOB:    return "blob";
+    case UDM_DBMODE_RAWBLOB: return "rawblob";
   }
   return "unknown_dbmode";
 }
@@ -1102,13 +1196,13 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "oracle8", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM,
     &udm_sqldb_oracle_handler
   },
   {
     "oracle", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM,
     &udm_sqldb_oracle_handler
   },
 #endif
@@ -1165,7 +1259,7 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "sqlite3",
     UDM_DB_SQLITE3, UDM_DB_SQLITE3, 1,
-    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_STDHEX,
     &udm_sqldb_sqlite3_handler,
   },
 #endif
@@ -1209,13 +1303,13 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "odbc-oracle", UDM_DB_ORACLE8, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-oracle8", UDM_DB_ORACLE8, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM,
     &udm_sqldb_odbc_handler,
   },
   {
@@ -1286,6 +1380,7 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
   UDM_FREE(db->DBName);
   UDM_FREE(db->where);
   UDM_FREE(db->from);
+  db->DBMode= UDM_DBMODE_BLOB;
   
   UdmVarListReplaceStr(&db->Vars, "DBAddr", dbaddr);
   
@@ -1337,15 +1432,12 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
     UdmVarListReplaceStr(&db->Vars, "filename", addr.filename);
   }
   
-  if(!strcasecmp(addr.schema, "searchd"))
+  if(!strcasecmp(addr.schema, "searchd") ||
+     !strcasecmp(addr.schema, "http") ||
+     !strcasecmp(addr.schema, "file"))
   {
     db->DBType=UDM_DB_SEARCHD;
     db->DBDriver=UDM_DB_SEARCHD;
-    if (UDM_OK != UdmSearchdConnect(db))
-    {
-      rc= UDM_ERROR;
-      goto ret;
-    }
   }
   else
   {
@@ -1390,10 +1482,17 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
         db->DBType != UDM_DB_ORACLE8 &&
         db->DBType != UDM_DB_DB2 &&
         db->DBType != UDM_DB_PGSQL &&
-        db->DBType != UDM_DB_IBASE)
+        db->DBType != UDM_DB_IBASE &&
+        db->DBType != UDM_DB_SQLITE3)
       return UDM_ERROR;
   }
 
+  if((v= UdmVarListFindStr(&db->Vars, "debugsql", "no")))
+  {
+    if (!strcasecmp(v, "yes")) 
+      db->flags |= UDM_SQL_DEBUG_QUERY;
+  }
+
   if(db->DBDriver == UDM_DB_IBASE ||
      db->DBDriver == UDM_DB_SQLITE ||
      db->DBDriver == UDM_DB_SQLITE3)
@@ -1444,11 +1543,12 @@ __C_LINK int __UDMCALL UdmStatAction(UDM_AGENT *A, UDM_STATLIST *S)
     res = UdmStatActionSQL(A, S, db);
     UDM_RELEASELOCK(A, UDM_LOCK_DB);
 #endif
-    if (res != UDM_OK) break;
-  }
-  if(res!=UDM_OK)
-  {
-    strcpy(A->Conf->errstr,db->errstr);
+    if (res != UDM_OK)
+    {
+      strcpy(A->Conf->errstr,db->errstr);
+      db->errcode= 0;
+      break;
+    }
   }
   UDM_RELEASELOCK(A, UDM_LOCK_CONF);
   return res;
@@ -1692,12 +1792,7 @@ int UdmMulti2Blob (UDM_AGENT *Indexer)
     int rc;
     UDM_DB *db = &Indexer->Conf->dbl.db[i];
     UDM_GETLOCK(Indexer, UDM_LOCK_DB);
-    if (db->DBMode == UDM_DBMODE_MULTI)
-      rc= UdmMulti2BlobSQL(Indexer, db);
-    else if (db->DBMode == UDM_DBMODE_SINGLE)
-      rc= UdmSingle2BlobSQL(Indexer, db);
-    else if (db->DBMode == UDM_DBMODE_BLOB)
-      rc= UdmBlob2BlobSQL(Indexer, db);
+    rc= UdmConvert2BlobSQL(Indexer, db);
     UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
     if (rc != UDM_OK)
     {
@@ -1777,6 +1872,7 @@ int UdmRewriteLimits(UDM_AGENT *Indexer)
 
 int UdmExport (UDM_AGENT *Indexer)
 {
+  int rc= UDM_OK;
 #ifdef HAVE_SQL
   size_t i;
   unsigned long ticks;
@@ -1788,14 +1884,19 @@ int UdmExport (UDM_AGENT *Indexer)
   {
     UDM_DB *db = &Indexer->Conf->dbl.db[i];
     UDM_GETLOCK(Indexer, UDM_LOCK_DB);
-    UdmExportSQL(Indexer, db);
+    rc= UdmExportSQL(Indexer, db);
     UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
+    if (rc != UDM_OK)
+    {
+      UdmLog(Indexer,UDM_LOG_ERROR,"%s",db->errstr); 
+      break;
+    }
   }
-
+  
   ticks=UdmStartTimer()-ticks;
   UdmLog(Indexer,UDM_LOG_ERROR,"Export finished\t%.2f",(float)ticks/1000);
 #endif
-  return(0);
+  return rc;
 }
 
 
@@ -1824,7 +1925,7 @@ static size_t UdmMultiCacheAddWord (UDM_MULTI_CACHE_WORD *cache, uint4 coord)
 static size_t UdmMultiCacheAddSection (UDM_MULTI_CACHE_SECTION *cache, UDM_WORD *word)
 {
   size_t i;
-  uint4 coord = UDM_WRDPOS(word->coord) & 0xFFFF;
+  uint4 coord = word->pos & 0x1FFFFF;
 
   if (! cache) return(0);
 
@@ -1851,7 +1952,7 @@ static size_t UdmMultiCacheAddSection (UDM_MULTI_CACHE_SECTION *cache, UDM_WORD
 static size_t UdmMultiCacheAddURL (UDM_MULTI_CACHE_URL *cache, UDM_WORD *word)
 {
   size_t i;
-  unsigned char secno = UDM_WRDSEC(word->coord);
+  unsigned char secno= word->secno;
 
   if (! cache) return(0);
 
@@ -2093,7 +2194,8 @@ static int bccmpwrd (UDM_BLOB_CACHE_WORD *s1, UDM_BLOB_CACHE_WORD *s2)
 
 void UdmBlobCacheSort (UDM_BLOB_CACHE *cache)
 {
-  qsort(cache->words, cache->nwords, sizeof(UDM_BLOB_CACHE_WORD), (udm_qsort_cmp)bccmpwrd);
+  if (cache->nwords)
+    UdmSort(cache->words, cache->nwords, sizeof(UDM_BLOB_CACHE_WORD), (udm_qsort_cmp)bccmpwrd);
 }
 
 UDM_WORD_CACHE *UdmWordCacheInit (UDM_WORD_CACHE *cache)
@@ -2149,7 +2251,7 @@ static int wccmpwrd (UDM_WORD_CACHE_WORD *s1, UDM_WORD_CACHE_WORD *s2)
     return -1;
   if ((_= strcmp(s1->word, s2->word)))
     return _;
-  return s1->coord - s2->coord;
+  return s1->pos - s2->pos;
 }
 
 void UdmWordCacheSort (UDM_WORD_CACHE *cache)
@@ -2157,31 +2259,32 @@ void UdmWordCacheSort (UDM_WORD_CACHE *cache)
   UdmSort(cache->words, cache->nwords, sizeof(UDM_WORD_CACHE_WORD), (udm_qsort_cmp)wccmpwrd);
 }
 
-int UdmWordCacheAdd (UDM_WORD_CACHE *cache, urlid_t url_id, const char *word, int intag)
+int UdmWordCacheAdd(UDM_WORD_CACHE *cache, urlid_t url_id, UDM_WORD *W)
 {
-  if (! word) return(UDM_OK);
+  if (!W->word) return(UDM_OK);
 
   if (cache->nwords == cache->awords)
   {
     UDM_WORD_CACHE_WORD *tmp;
-    tmp = UdmRealloc(cache->words, (cache->awords + 256) * sizeof(UDM_WORD_CACHE_WORD));
-    if (! tmp) {
+    tmp= UdmRealloc(cache->words, (cache->awords + 256) * sizeof(UDM_WORD_CACHE_WORD));
+    if (!tmp)
+    {
       fprintf(stderr, "Realloc failed while adding word\n");
       return(UDM_ERROR);
     }
-    cache->words = tmp;
-    cache->awords += 256;
-    cache->nbytes += sizeof(UDM_WORD_CACHE_WORD) * 256;
+    cache->words= tmp;
+    cache->awords+= 256;
+    cache->nbytes+= sizeof(UDM_WORD_CACHE_WORD) * 256;
   }
 
-  cache->words[cache->nwords].word = UdmStrdup(word);
-  if (! cache->words[cache->nwords].word) return(UDM_ERROR);
-  cache->words[cache->nwords].url_id = url_id;
-  cache->words[cache->nwords].secno = UDM_WRDSEC(intag) & 0xFF;
-  cache->words[cache->nwords].coord = UDM_WRDPOS(intag) & 0xFFFF;
-  cache->words[cache->nwords].seed = UdmStrHash32(word) & MULTI_DICTS;
+  if (!(cache->words[cache->nwords].word= UdmStrdup(W->word)))
+    return(UDM_ERROR);
+  cache->words[cache->nwords].url_id= url_id;
+  cache->words[cache->nwords].secno= W->secno & 0xFF;
+  cache->words[cache->nwords].pos= W->pos & 0x1FFFFF;
+  cache->words[cache->nwords].seed= UdmStrHash32(W->word) & MULTI_DICTS;
   cache->nwords++;
-  cache->nbytes += strlen(word) + 1;
+  cache->nbytes+= strlen(W->word) + 1;
   return(UDM_OK);
 }
 
