@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -50,9 +50,6 @@
 /**************************** DBAddr ***********************************/
 
 __C_LINK UDM_ENV * __UDMCALL UdmEnvInit(UDM_ENV *Conf){
-#ifdef MECAB
-        char   *mecab_argv[] = { "mecab", "-F%m\" \"", "-E\" \"", "-B\" \"", NULL };
-#endif
 
 	if(!Conf){
 		Conf=(UDM_ENV *)UdmMalloc(sizeof(UDM_ENV));
@@ -67,9 +64,7 @@ __C_LINK UDM_ENV * __UDMCALL UdmEnvInit(UDM_ENV *Conf){
 	Conf->url_number=0x7FFFFFFF;
 	Conf->lcs=UdmGetCharSet("latin1");
 	Conf->bcs=UdmGetCharSet("latin1");
-#ifdef MECAB
-	Conf->mecab = mecab_new (4, mecab_argv);
-#endif
+	Conf->unidata= udm_unidata_default;
 	
 	return(Conf);
 }
@@ -137,6 +132,18 @@ __C_LINK int __UDMCALL UdmSetLockProc(UDM_ENV * Conf,
 
 int UdmEnvPrepare(UDM_ENV *Env)
 {
+#ifdef MECAB
+   {
+     char   *mecab_argv[] = { "mecab", "-F%m\" \"", "-E\" \"", "-B\" \"", NULL };
+     if (!(Env->mecab= mecab_new (4, mecab_argv)))
+     {
+       udm_snprintf(Env->errstr, sizeof(Env->errstr) - 1,
+                    "mecab_new failed: %s", mecab_strerror(Env->mecab));
+       return UDM_ERROR;
+     }
+   }
+#endif
+
   if(Env->Spells.nitems && Env->Affixes.nitems)
   {
     char *err= Env->errstr;
