@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -187,5 +187,7 @@ void UdmParseHTTPResponse(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
   }
   UDM_FREE(headers);
   
-  UdmVarListInsInt(&Doc->Sections,"Content-Length",Doc->Buf.buf-Doc->Buf.content+(int)Doc->Buf.size);
+  UdmVarListInsInt(&Doc->Sections,"Content-Length",
+                   Doc->Buf.buf-Doc->Buf.content+(int)Doc->Buf.size +
+                   Doc->Buf.content_length);
 }
