@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@
 #include "udm_sqldbms.h"
 #include "udm_uniconv.h"
 #include "udm_parsexml.h"
+#include "udm_http.h"
 
 #ifdef WIN32
 #include "udm_winutils.h"
@@ -64,10 +65,11 @@
        int flags    =0; /* For indexer            */
        int total_threads=0; /* Total threads number         */
        int sleep_threads=0; /* Number of sleepping threads      */
-       int next_thread  =1; /* Handle number for indexer    */
        int max_index_time=-1;
        int cur_url_number=0;
 static int log2stderr=1;
+static int have_loglevel= 0;
+static int loglevel= UDM_LOG_INFO;
 static char cname[1024]="";
 static int add_servers=UDM_FLAG_ADD_SERV;
 static int add_server_urls = UDM_FLAG_ADD_SERVURL;
@@ -79,11 +81,12 @@ static int warnings=1;
 
 static UDM_ENV Conf;
 
-UDM_AGENT **ThreadIndexers;
+UDM_AGENT *ThreadIndexers;
 
 #ifdef HAVE_PTHREAD
 #ifdef WIN32
 HANDLE *threads;
+typedef HANDLE udm_thread_t;
 #ifdef CONSOLE
 
 #include <time.h>
@@ -99,6 +102,7 @@ localtime_r(const time_t *clock, struct tm *result)
 #else
 #define CONSOLE
 pthread_t *threads;
+typedef pthread_t udm_thread_t;
 #endif
 #endif
 
@@ -335,7 +339,7 @@ static int cmpgrp(const void *v1, const void *v2)
   return strcasecmp(c1->name,c2->name);
 }
 
-static void display_charsets(void)
+static void display_charsets(FILE *file)
 {
   UDM_CHARSET *cs=NULL;
   UDM_CHARSET c[100];
@@ -349,19 +353,19 @@ static void display_charsets(void)
     if (cs->family != UDM_CHARSET_UNKNOWN)
       c[n++]=*cs;
   }
-  fprintf(stderr,"\n%d charsets available:\n",n);
+  fprintf(file,"\n%d charsets available:\n",n);
 
   UdmSort(c,n,sizeof(UDM_CHARSET),&cmpgrp);
   for(i=0;i<n;i++)
   {
     if (family!=c[i].family)
     {
-      fprintf(stderr, "\n%19s : ", UdmCsGroup(&c[i]));
+      fprintf(file, "\n%19s : ", UdmCsGroup(&c[i]));
       family=c[i].family;
     }
-    fprintf(stderr,"%s ",c[i].name);
+    fprintf(file,"%s ",c[i].name);
   }
-  fprintf(stderr,"\n");
+  fprintf(file,"\n");
 }
 
 static void UdmFeatures(UDM_VARLIST *V)
@@ -537,93 +541,96 @@ static void UdmFeatures(UDM_VARLIST *V)
 
 static int usage(int level)
 {
-  fprintf(stderr, "\n");
-  fprintf(stderr, "indexer from %s-%s-%s\n", PACKAGE, VERSION, UDM_DBTYPE);
-  fprintf(stderr, "http://www.mnogosearch.org/ (C)1998-2006, LavTech Corp.\n");
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Usage: indexer [OPTIONS]  [configfile]\n");
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Indexing options:\n");
+  FILE *file= stdout;
+  fprintf(file, "\n");
+  fprintf(file, "indexer from %s-%s-%s\n", PACKAGE, VERSION, UDM_DBTYPE);
+  fprintf(file, "http://www.mnogosearch.org/ (C)1998-2007, LavTech Corp.\n");
+  fprintf(file, "\n");
+  fprintf(file, "Usage: indexer [OPTIONS]  [configfile]\n");
+  fprintf(file, "\n");
+  fprintf(file, "Indexing options:\n");
 
 #ifdef HAVE_SQL
-  fprintf(stderr, "  -a              reindex all documents even if not expired (may be\n");
-  fprintf(stderr, "                  limited using -t, -u, -s, -c, -y and -f options)\n");
-  fprintf(stderr, "  -m              reindex expired documents even if not modified (may\n");
-  fprintf(stderr, "                  be limited using -t, -u, -c, -s, -y and -s options)\n");
-  fprintf(stderr, "  -e              index 'most expired' (oldest) documents first\n");
-  fprintf(stderr, "  -o              index documents with less depth (hops value) first\n");
-  fprintf(stderr, "  -r              do not try to reduce remote servers load by randomising\n");
-  fprintf(stderr, "                  url fetch list before indexing (-r recommended for very \n");
-  fprintf(stderr, "                  big number of URLs)\n");
-  fprintf(stderr, "  -n n            index only n documents and exit\n");
-  fprintf(stderr, "  -c n            index only n seconds and exit\n");
-  fprintf(stderr, "  -q              quick startup (do not add Server URLs)\n");
+  fprintf(file, "  -a              reindex all documents even if not expired (may be\n");
+  fprintf(file, "                  limited using -t, -u, -s, -c, -y and -f options)\n");
+  fprintf(file, "  -m              reindex expired documents even if not modified (may\n");
+  fprintf(file, "                  be limited using -t, -u, -c, -s, -y and -f options)\n");
+  fprintf(file, "  -e              index 'most expired' (oldest) documents first\n");
+  fprintf(file, "  -o              index documents with less depth (hops value) first\n");
+  fprintf(file, "  -r              do not try to reduce remote servers load by randomising\n");
+  fprintf(file, "                  url fetch list before indexing (-r recommended for very \n");
+  fprintf(file, "                  big number of URLs)\n");
+  fprintf(file, "  -n n            index only n documents and exit\n");
+  fprintf(file, "  -c n            index only n seconds and exit\n");
+  fprintf(file, "  -q              quick startup (do not add Server URLs)\n");
 #endif
 
-  fprintf(stderr, "  -b              block starting more than one indexer instances\n");
-  fprintf(stderr, "  -i              insert new URLs (URLs to insert must be given using -u or -f)\n");
-  fprintf(stderr, "  -p n            sleep n seconds after each URL\n");
-  fprintf(stderr, "  -w              do not warn before clearing documents from database\n");
+  fprintf(file, "  -b              block starting more than one indexer instances\n");
+  fprintf(file, "  -i              insert new URLs (URLs to insert must be given using -u or -f)\n");
+  fprintf(file, "  -p n            sleep n seconds after each URL\n");
+  fprintf(file, "  -w              do not warn before clearing documents from database\n");
 
 #ifdef HAVE_PTHREAD
-  fprintf(stderr, "  -N n            run N threads\n");
+  fprintf(file, "  -N n            run N threads\n");
 #endif
 
-  fprintf(stderr, "\n");
+  fprintf(file, "\n");
 
 #ifdef HAVE_SQL
-  fprintf(stderr, "Subsection control options (may be combined):\n");
-  fprintf(stderr, "  -s status       limit indexer to documents matching status (HTTP Status code)\n");
-  fprintf(stderr, "  -t tag          limit indexer to documents matching tag\n");
-  fprintf(stderr, "  -g category     limit indexer to documents matching category\n");
-  fprintf(stderr, "  -y content-type limit indexer to documents matching content-type\n");
-  fprintf(stderr, "  -L language     limit indexer to documents matching language\n");
-  fprintf(stderr, "  -u pattern      limit indexer to documents with URLs matching pattern\n");
-  fprintf(stderr, "                  (supports SQL LIKE wildcard '%%')\n");
-  fprintf(stderr, "  -f filename     read URLs to be indexed/inserted/cleared from file (with -a\n");
-  fprintf(stderr, "                  or -C option, supports SQL LIKE wildcard '%%'; has no effect\n");
-  fprintf(stderr, "                  when combined with -m option)\n");
-  fprintf(stderr, "  -f -            Use STDIN instead of file as URL list\n");
+  fprintf(file, "Subsection control options (may be combined):\n");
+  fprintf(file, "  -s status       limit indexer to documents matching status (HTTP Status code)\n");
+  fprintf(file, "  -t tag          limit indexer to documents matching tag\n");
+  fprintf(file, "  -g category     limit indexer to documents matching category\n");
+  fprintf(file, "  -y content-type limit indexer to documents matching content-type\n");
+  fprintf(file, "  -L language     limit indexer to documents matching language\n");
+  fprintf(file, "  -u pattern      limit indexer to documents with URLs matching pattern\n");
+  fprintf(file, "                  (supports SQL LIKE wildcard '%%')\n");
+  fprintf(file, "  -f filename     read URLs to be indexed/inserted/cleared from file (with -a\n");
+  fprintf(file, "                  or -C option, supports SQL LIKE wildcard '%%'; has no effect\n");
+  fprintf(file, "                  when combined with -m option)\n");
+  fprintf(file, "  -f -            Use STDIN instead of file as URL list\n");
 #else
-  fprintf(stderr, "URL options:\n");
-  fprintf(stderr, "  -u URL          insert URL at startup\n");
-  fprintf(stderr, "  -f filename     read URLs to be inserted from file\n");
+  fprintf(file, "URL options:\n");
+  fprintf(file, "  -u URL          insert URL at startup\n");
+  fprintf(file, "  -f filename     read URLs to be inserted from file\n");
 #endif
 
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Logging options:\n");
+  fprintf(file, "\n");
+  fprintf(file, "Logging options:\n");
 
 #ifdef LOG_PERROR
-  fprintf(stderr, "  -l              do not log to stdout/stderr\n");
+  fprintf(file, "  -l              do not log to stdout/stderr\n");
 #endif
 
-  fprintf(stderr, "  -v n            verbose level, 0-5\n");
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Misc. options:\n");
+  fprintf(file, "  -v n            verbose level, 0-5\n");
+  fprintf(file, "\n");
+  fprintf(file, "Misc. options:\n");
 
 #ifdef HAVE_SQL
-  fprintf(stderr, "  -C              clear database and exit\n");
-  fprintf(stderr, "  -S              print statistics and exit\n");
-  fprintf(stderr, "  -j t            set current time for statistic (use with -S),\n");
-  fprintf(stderr, "                  YYYY-MM[-DD[ HH[:MM[:SS]]]]\n");
-  fprintf(stderr, "                  or time offset, e.g. 1d12h (see Period in indexer.conf)\n");
-  fprintf(stderr, "  -I              print referers and exit\n");
-  fprintf(stderr, "  -R              calculate popularity rank\n");
-  fprintf(stderr, "  -Ecreate        create SQL table structure and exit\n");
-  fprintf(stderr, "  -Edrop          drop SQL table structure and exit\n");
-  fprintf(stderr, "  -Ewordstat      create statistics for misspelled word suggestions\n");
+  fprintf(file, "  -C              clear database and exit\n");
+  fprintf(file, "  -S              print statistics and exit\n");
+  fprintf(file, "  -j t            set current time for statistic (use with -S),\n");
+  fprintf(file, "                  YYYY-MM[-DD[ HH[:MM[:SS]]]]\n");
+  fprintf(file, "                  or time offset, e.g. 1d12h (see Period in indexer.conf)\n");
+  fprintf(file, "  -I              print referers and exit\n");
+  fprintf(file, "  -R              calculate popularity rank\n");
+  fprintf(file, "  -Ecreate        create SQL table structure and exit\n");
+  fprintf(file, "  -Edrop          drop SQL table structure and exit\n");
+  fprintf(file, "  -Eblob          create fast search index\n");
+  fprintf(file, "  -Ewordstat      create statistics for misspelled word suggestions\n");
 #endif
+  
+  fprintf(file, "  -F pattern      print compile configuration and exit, e.g. -F '*'\n");
+  fprintf(file, "  -h,-?           print help page and exit\n");
+  fprintf(file, "  -hh             print more help and exit\n");
+  fprintf(file, "  -d configfile   use given configfile instead of default one.\n");
+  fprintf(file, "                  This option is usefull when running indexer as an\n");
+  fprintf(file, "                  interpreter, e.g.: #!/usr/local/sbin/indexer -d\n");
+  fprintf(file, "\n");
+  fprintf(file, "\n");
+  fprintf(file, "Please post bug reports and suggestions at http://www.mnogosearch.org/bugs/\n");
 
-  fprintf(stderr, "  -h,-?           print help page and exit\n");
-  fprintf(stderr, "  -hh             print more help and exit\n");
-  fprintf(stderr, "  -d configfile   use given configfile instead of default one.\n");
-  fprintf(stderr, "                  This option is usefull when running indexer as an\n");
-  fprintf(stderr, "                  interpreter, e.g.: #!/usr/local/sbin/indexer -d\n");
-  fprintf(stderr, "\n");
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Please post bug reports and suggestions at http://www.mnogosearch.org/bugs/\n");
-
-  if (level>1)display_charsets();
+  if (level>1)display_charsets(file);
   return(0);
 }
 
@@ -703,12 +710,13 @@ static char **UdmARGV;
 static enum udm_indcmd cmd = UDM_IND_INDEX;
 static int insert = 0, expire = 0, pop_rank = 0, mkind = 0, block = 0, help = 0;
 static char *url_filename=NULL;
+static int thd_errors= 0;
 
 static void UdmParseCmdLine(void)
 {
   int ch;
 
-  while ((ch = getopt(UdmARGC, UdmARGV, "QUCSIRMabheorlmqiw?E:F:t:u:s:n:v:L:A:D:p:N:f:c:g:y:j:d:")) != -1)
+  while ((ch = getopt(UdmARGC, UdmARGV, "QUCSIRMabheorlmqiw?-:E:F:t:u:s:n:v:L:A:D:p:N:f:c:g:y:j:d:")) != -1)
   {
     switch (ch)
     {
@@ -741,7 +749,7 @@ static void UdmParseCmdLine(void)
       case 'm': flags|=UDM_FLAG_REINDEX;break;
       case 'n': Conf.url_number=atoi(optarg);break;
       case 'c': max_index_time=atoi(optarg);break;
-      case 'v': UdmSetLogLevel(NULL, atoi(optarg)); break;
+      case 'v': loglevel= atoi(optarg); have_loglevel= 1; break;
       case 'p': seconds=atoi(optarg);break;
       case 't': UdmVarListAddStr(&Conf.Vars,"tag" , optarg);break;
       case 'g': UdmVarListAddStr(&Conf.Vars,"cat" , optarg);break;
@@ -766,6 +774,18 @@ static void UdmParseCmdLine(void)
       case 'd': strncpy(cname, optarg, sizeof(cname));
         cname[sizeof(cname) - 1] = '\0';
         break;
+      case '-':
+        {
+          char *arg= strchr(optarg, '=');
+          if (arg)
+          {
+            *arg++= '\0';
+            UdmVarListAddStr(&Conf.Vars, optarg, arg);
+          }
+          else
+            help++;
+        }
+        break;
       case '?':
       case 'h':
       default:
@@ -776,7 +796,154 @@ static void UdmParseCmdLine(void)
 
 
 
+static int
+UdmReloadEnv(UDM_AGENT *Indexer)
+{
+  UDM_ENV   NewConf;
+  int  rc;
 
+  UdmLog(Indexer,UDM_LOG_ERROR,"Reloading config '%s'",cname);
+  UdmEnvInit(&NewConf);
+  UdmSetLockProc(&NewConf,UdmLockProc);
+  UdmSetRefProc(&NewConf,UdmRefProc);
+
+  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+  Indexer->Conf = &NewConf;
+  rc = UdmIndexerEnvLoad(Indexer, cname, add_servers + load_langmaps + UDM_FLAG_SPELL);
+  Indexer->Conf = &Conf;
+               
+  if (rc!=UDM_OK)
+  {
+    UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
+    UdmLog(Indexer,UDM_LOG_ERROR,"Can't load config: %s",UdmEnvErrMsg(&NewConf));
+    UdmLog(Indexer,UDM_LOG_ERROR,"Continuing with old config");
+    UdmEnvFree(&NewConf);
+  }
+  else
+  {
+    UdmEnvFree(&Conf);
+    Conf=NewConf;
+    UdmParseCmdLine();
+#ifndef WIN32
+    UdmOpenLog("indexer", &Conf, log2stderr);
+#endif
+    UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
+  }
+  return UDM_OK;
+}
+  
+
+static int
+httpd_client_handler(int client, UDM_AGENT *A)
+{
+  char request[4096];
+  char response[1024];
+  char speed_info[128]= "";
+  ssize_t nrecv,nsent;
+  size_t i, len, total_docs= 0, total_sec= 0;
+  unsigned long long total_bytes= 0;
+  time_t now= time(0);
+  
+  nrecv= recv(client, &request, sizeof(request), 0);
+  UdmLog(A, UDM_LOG_ERROR, "Received request len=%d", (int) nrecv);
+  udm_snprintf(response, sizeof(response) - 1,
+               "HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n");
+  nsent= UdmSend(client, response, strlen(response), 0);
+  len= sprintf(response,
+               "<table border=1 cellspacing=1 cellpadding=1>\n"
+               "<tr><th>ID</th>"
+               "<th>Docs</th>"
+               "<th>Size</th>"
+               "<th>Task</th>"
+               "<th>Time</th>"
+               "<th>Param</th>"
+               "<th>Extra</th></tr>");
+  nsent= UdmSend(client, response, len, 0);
+  for (i= 0; i < maxthreads; i++)
+  {
+    UDM_AGENT *Tmp= &ThreadIndexers[i];
+    char mutex_owned_info[64]= "";
+    size_t mutex, sec;
+    
+    for (mutex= 0;
+         mutex < Tmp->State.nmutexes && mutex < UDM_AGENT_STATE_MUTEXES;
+         mutex++)
+    {
+      sprintf(UDM_STREND(mutex_owned_info), " #%d",
+              Tmp->State.mutex_owned[mutex]);
+    }
+    
+    /*
+    size_t mutex_owned= Tmp->State.mutex_owned;
+    
+    if (mutex_owned)
+      sprintf(mutex_owned_info, "Owner for mitex #%d", mutex_owned);
+    */
+    
+    len= sprintf(response,
+                 "<tr><td>%d</td>"
+                 "<td align=right>%d</td>"
+                 "<td align=right>%llu</td>"
+                 "<td>%s</td>"
+                 "<td align=right>%d</td>"
+                 "<td>%s&nbsp;</td>"
+                 "<td>%s%s%s&nbsp;</td></tr>\n",
+                 Tmp->handle,
+                 Tmp->ndocs,
+                 Tmp->nbytes,
+                 Tmp->State.task,
+                 (int) (now - Tmp->State.start_time),
+                 UDM_NULL2EMPTY(Tmp->State.param),
+                 mutex_owned_info[0] ? "Owner for mutex: " : "",
+                 mutex_owned_info,
+                 UDM_NULL2EMPTY(Tmp->State.extra));
+    nsent= UdmSend(client, response, len, 0);
+    total_docs+= Tmp->ndocs;
+    total_bytes+= Tmp->nbytes;
+    sec= (size_t) (now - Tmp->start_time);
+    if (sec > total_sec)
+      total_sec= sec;
+  }
+  if (total_sec)
+  {
+    udm_snprintf(speed_info, sizeof(speed_info) - 1,
+                 "%d seconds, %d docs/sec, %d bytes/sec",
+                 total_sec,
+                 total_docs / total_sec,
+                 (int) (total_bytes / total_sec));
+  }
+  
+  len= sprintf(response,
+                 "<tr><td>&nbsp;</td>"
+                 "<td align=right>%d</td>"
+                 "<td align=right>%llu</td>"
+                 "<td>&nbsp;</td>"
+                 "<td align=right>&nbsp;</td>"
+                 "<td>%s&nbsp;</td>"
+                 "<td>&nbsp;</td></tr>\n",
+                 total_docs,
+                 total_bytes,
+                 speed_info);
+  nsent= UdmSend(client, response, len, 0);
+  len= sprintf(response, "</table>\n");
+  nsent= UdmSend(client, response, len, 0);
+  return UDM_OK;
+}
+
+
+#ifdef  WIN32
+unsigned int __stdcall thread_main_httpd(void *arg)
+#else
+static void* thread_main_httpd(void *arg)
+#endif
+{
+  UDM_AGENT *A= (UDM_AGENT*) arg;
+  UdmStartHTTPD(A, httpd_client_handler);
+  return 0;
+}
+
+
+#define UDM_NOTARGETS_SLEEP 10
 
 #ifdef  WIN32
 unsigned int __stdcall thread_main(void *arg)
@@ -804,47 +971,18 @@ static void * thread_main(void *arg)
     UDM_GETLOCK(Indexer, UDM_LOCK_THREAD);
     if (have_sighup)
     {
-      UDM_ENV   NewConf;
-      int  rc;
-               
-      UdmLog(Indexer,UDM_LOG_ERROR,"Reloading config '%s'",cname);
-      UdmEnvInit(&NewConf);
-      UdmSetLockProc(&NewConf,UdmLockProc);
-      UdmSetRefProc(&NewConf,UdmRefProc);
-
-      UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
-      Indexer->Conf = &NewConf;
-      rc = UdmIndexerEnvLoad(Indexer, cname, add_servers + load_langmaps + UDM_FLAG_SPELL);
-      Indexer->Conf = &Conf;
-               
-      if (rc!=UDM_OK)
-      {
-        UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
-        UdmLog(Indexer,UDM_LOG_ERROR,"Can't load config: %s",UdmEnvErrMsg(&NewConf));
-        UdmLog(Indexer,UDM_LOG_ERROR,"Continuing with old config");
-        UdmEnvFree(&NewConf);
-      }
-      else
-      {
-        UdmEnvFree(&Conf);
-        Conf=NewConf;
-        UdmParseCmdLine();
-#ifndef WIN32
-        UdmOpenLog("indexer", &Conf, log2stderr);
-#endif
-        UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
-      }
+      UdmReloadEnv(Indexer);
       have_sighup=0;
     }
-  
+    
     if (have_sigint || have_sigterm)
     {
       int z;
       UdmLog(Indexer, UDM_LOG_ERROR, "%s received. Terminating. Please wait...", (have_sigint) ? "SIGINT" : "SIGTERM");
-      for (z = 0 ; z < total_threads; z++)
+      for (z = 0 ; z < maxthreads; z++)
       {
-        if (ThreadIndexers[z])
-          UdmAgentSetAction(ThreadIndexers[z], UDM_TERMINATED);
+        if (ThreadIndexers[z].handle)
+          UdmAgentSetAction(&ThreadIndexers[z], UDM_TERMINATED);
       }
       UdmAgentSetAction(&Main, UDM_TERMINATED);
       have_sigint = have_sigterm = 0;
@@ -913,6 +1051,7 @@ static void * thread_main(void *arg)
         break;
 #endif
       case UDM_ERROR:
+        thd_errors++;
 #ifdef WIN32
         str_buf = (char *)UdmMalloc(1024);
         udm_snprintf(str_buf, 1024, "Error: %s",  UdmEnvErrMsg(Indexer->Conf));
@@ -954,6 +1093,7 @@ static void * thread_main(void *arg)
 
     if ((seconds)&&(!done))
     {
+      UdmAgentSetTask(Indexer, "Sleeping");
       UdmLog(Indexer,UDM_LOG_DEBUG,"Sleeping %d second(s)",seconds);
 #ifndef WIN32
       Indexer->nsleepsecs+= seconds - UDMSLEEP(seconds);
@@ -965,16 +1105,18 @@ static void * thread_main(void *arg)
   
     if ((i_sleep) && (!done))
     {
-      UdmLog(Indexer, UDM_LOG_ERROR, "%s, sleeping %d seconds",
+      UdmLog(Indexer, UDM_LOG_ERROR, "%s, sleeping %d seconds (%d threads, %d sleeping)",
              (res == UDM_NOTARGET) ? "No targets" : 
-             ((res == UDM_TERMINATED) ? "Terminating" : "An error occured"), 60);
+             ((res == UDM_TERMINATED) ? "Terminating" : "An error occured"),
+             UDM_NOTARGETS_SLEEP, total_threads, sleep_threads);
 #ifndef WIN32
-      Indexer->nsleepsecs+= 60 - UDMSLEEP(60);
+      Indexer->nsleepsecs+= UDM_NOTARGETS_SLEEP - UDMSLEEP(UDM_NOTARGETS_SLEEP);
 #else
-      Indexer->nsleepsecs+= 60;
-      UDMSLEEP(60);
+      Indexer->nsleepsecs+= UDM_NOTARGETS_SLEEP;
+      UDMSLEEP(UDM_NOTARGETS_SLEEP);
 #endif
     }
+    UdmAgentSetTask(Indexer, "Unknown");
   }
 
   if (res!=UDM_ERROR)
@@ -993,21 +1135,20 @@ static void * thread_main(void *arg)
       M= Indexer->nbytes / 1048576.0 / sec;
       if (M < 1.0) K = Indexer->nbytes / 1024.0 / sec;
     }
-    UdmLog(Indexer,UDM_LOG_ERROR,"Done (%d seconds, %u documents, %u bytes, %5.2f %cbytes/sec.)",
+    UdmLog(Indexer,UDM_LOG_ERROR,"Done (%d seconds, %u documents, %llu bytes, %5.2f %cbytes/sec.)",
            sec, Indexer->ndocs, Indexer->nbytes, (M < 1.0) ? K : M, (M < 1.0) ? 'K' : 'M' );
 #if !defined(WIN32) && defined(HAVE_PTHREAD)
-    for (z = 0 ; z < total_threads; z++)
-      if (ThreadIndexers[z])
+    for (z = 0 ; z < maxthreads; z++)
+      if (ThreadIndexers[z].handle)
         pthread_kill(threads[z], SIGALRM); /* wake-up sleeping threads */
 #endif
   }
   UDM_GETLOCK(Indexer, UDM_LOCK_THREAD);
   total_threads--;
-  ThreadIndexers[Indexer->handle - 1] = NULL;
+  UdmAgentFree(Indexer);
+  Indexer->handle= 0;
   UDM_RELEASELOCK(Indexer, UDM_LOCK_THREAD);
 
-  UdmAgentFree(Indexer);
-     
 #ifdef WIN32
   return(0);
 #else
@@ -1107,11 +1248,44 @@ static int UdmClear(UDM_AGENT *A, const char *url_fname)
 }
 
 
+static int
+UdmThreadCreate(udm_thread_t *thd, void *(*start_routine)(void*), void *arg)
+{
+  int res;
+#ifdef WIN32
+  {
+    res= _beginthreadex(NULL, 0, start_routine, arg, 0, NULL);
+    assert(res != -1);
+  }
+#else
+  {
+    pthread_attr_t attr;
+    size_t stksize= 1024 * 512;
+    pthread_attr_init(&attr);
+    pthread_attr_setstacksize(&attr, stksize);
+    pthread_create(thd, &attr, start_routine, arg);
+    pthread_attr_destroy(&attr);
+  }
+#endif
+  return res;
+}
+
+
 static int UdmIndex(UDM_AGENT *A)
 {
-
-  ThreadIndexers = (UDM_AGENT**)UdmMalloc(maxthreads * sizeof(UDM_AGENT*));
-               
+  UDM_AGENT httpd_agent;
+  size_t nbytes= maxthreads * sizeof(UDM_AGENT);
+  ThreadIndexers= (UDM_AGENT*)UdmMalloc(nbytes);
+  bzero((void*) ThreadIndexers, nbytes);
+  UdmAgentInit(&httpd_agent, A->Conf, 0);
+  
+  if (UdmVarListFind(&A->Conf->Vars, "Listen"))
+  {
+    udm_thread_t httpd_thread;
+    UdmThreadCreate(&httpd_thread, thread_main_httpd, &httpd_agent);
+  }
+  
+  
 #ifdef HAVE_PTHREAD
   {
     int i;
@@ -1122,35 +1296,22 @@ static int UdmIndex(UDM_AGENT *A)
     threads= (pthread_t*)UdmMalloc(maxthreads * sizeof(pthread_t));
 #endif
                
-    for(i=0;i<maxthreads;i++)
+    for(i= 0; i < maxthreads; i++)
     {
       UDM_AGENT *Indexer;
       if (seconds) UDMSLEEP(seconds);
 
       UDM_GETLOCK(A, UDM_LOCK_THREAD);
       UDM_GETLOCK(A, UDM_LOCK_CONF);
-      Indexer = UdmAgentInit(NULL, A->Conf, next_thread++);
+      Indexer= UdmAgentInit(&ThreadIndexers[i], A->Conf, i + 1);
       Indexer->flags = flags;
-      ThreadIndexers[Indexer->handle - 1] = Indexer;
       UDM_RELEASELOCK(A, UDM_LOCK_CONF);
       UDM_RELEASELOCK(A, UDM_LOCK_THREAD);
-#ifdef WIN32
-      {
-        int int_res= _beginthreadex(NULL, 0, &thread_main, Indexer, 0, NULL);
-        assert(int_res != -1);
-      }
-#else
-      {
-        pthread_attr_t attr;
-        size_t stksize = 1024 * 512;
-        pthread_attr_init(&attr);
-        pthread_attr_setstacksize(&attr, stksize);
-        pthread_create(&threads[i], &attr, &thread_main, Indexer);
-        pthread_attr_destroy(&attr);
-      }
-#endif
+      
+      UdmThreadCreate(&threads[i], &thread_main, Indexer);
+
       UDM_GETLOCK(A, UDM_LOCK_THREAD);
-      total_threads = i + 1;
+      total_threads= i + 1;
       UDM_RELEASELOCK(A,UDM_LOCK_THREAD);
     }
 #ifndef WIN32        
@@ -1174,10 +1335,12 @@ static int UdmIndex(UDM_AGENT *A)
   thread_main(&Main);
 #endif
 
+  UdmAgentFree(&httpd_agent);
   UDM_FREE(ThreadIndexers);
-  return UDM_OK;
+  return thd_errors ? UDM_ERROR : UDM_OK;
 }
 
+
 static int UdmBitrixIndex(UDM_AGENT *Agent)
 {
   char buf[1024];
@@ -1313,10 +1476,43 @@ ex:
 
 #ifdef CONSOLE
 
+static void
+UdmEnvSetDirs(UDM_ENV *Env)
+{
+#ifdef WIN32
+  {
+    char *conf_dir= WinUdmConfDir();
+    UdmVarListReplaceStr(&Env->Vars,"ConfDir", conf_dir);
+    UdmVarListReplaceStr(&Env->Vars,"ShareDir", conf_dir);
+    UdmVarListReplaceStr(&Env->Vars,"TmpDir", conf_dir);
+    UdmVarListReplaceStr(&Env->Vars,"VarDir", conf_dir);
+    UDM_FREE(conf_dir);
+  }
+#else
+  {
+    char *env;
+    if (!(env= getenv("UDM_CONF_DIR")))
+      env= getenv("UDM_ETC_DIR");
+    UdmVarListReplaceStr(&Env->Vars, "ConfDir", env ?env : UDM_CONF_DIR);
+    
+    env= getenv("UDM_SHARE_DIR");
+    UdmVarListReplaceStr(&Env->Vars, "ShareDir", env ? env : UDM_SHARE_DIR);
+    
+    env= getenv("UDM_VAR_DIR");
+    UdmVarListReplaceStr(&Env->Vars, "VarDir", env ? env : UDM_VAR_DIR);
+    
+    if (!(env= getenv("UDM_TMP_DIR")))
+      env= getenv("TMPDIR");
+    UdmVarListReplaceStr(&Env->Vars, "TmpDir", env ? env : UDM_TMP_DIR);
+  }
+#endif
+
+}
+
 int main(int argc, char **argv)
 {
-  char  *language=NULL,*affix=NULL,*dictionary=NULL, *env;
-  int   pid_fd;
+  char  *language=NULL,*affix=NULL,*dictionary=NULL;
+  int   pid_fd, rc= 0;
   char  pidbuf[1024];
   char  *REQUEST_METHOD= getenv("REQUEST_METHOD");
   FILE  *logfile= REQUEST_METHOD ? stdout : stderr;
@@ -1335,6 +1531,7 @@ int main(int argc, char **argv)
      
   UdmInitMutexes();
   UdmEnvInit(&Conf);
+  UdmEnvSetDirs(&Conf);
   UdmSetLockProc(&Conf,UdmLockProc);
   UdmSetRefProc(&Conf,UdmRefProc);
 #ifdef THINFO_TEST
@@ -1378,24 +1575,9 @@ int main(int argc, char **argv)
   {
     usage(help);
     UdmEnvFree(&Conf);
-    return(1);
+    return 1;
   }
   
-#ifdef WIN32
-  {
-    char *conf_dir= WinUdmConfDir();
-    UdmVarListReplaceStr(&Conf.Vars,"ConfDir", conf_dir);
-    UdmVarListReplaceStr(&Conf.Vars,"ShareDir", conf_dir);
-    UDM_FREE(conf_dir);
-  }
-#else
-  env= getenv("UDM_CONF_DIR");
-  UdmVarListReplaceStr(&Conf.Vars,"ConfDir",env?env:UDM_CONF_DIR);
-     
-  env=getenv("UDM_SHARE_DIR");
-  UdmVarListReplaceStr(&Conf.Vars,"ShareDir",env?env:UDM_SHARE_DIR);
-#endif
- 
   if (!*cname)
   {
     if (argc == 1)
@@ -1415,13 +1597,15 @@ int main(int argc, char **argv)
   {
     fprintf(logfile, "%s\n", UdmEnvErrMsg(&Conf));
     UdmEnvFree(&Conf);
-    exit(1);
+    return 1;
   }
      
   if (cmd == UDM_IND_CHECKCONF)
-    exit(0);
+    return 0;
   
 #ifndef WIN32
+   UdmSetLogLevel(NULL, have_loglevel ? loglevel :
+                  UdmVarListFindInt(&Main.Conf->Vars, "LogLevel", UDM_LOG_INFO));
    UdmOpenLog("indexer",&Conf, log2stderr);
    UdmSigHandlersInit(&Main);
 #endif
@@ -1429,26 +1613,25 @@ int main(int argc, char **argv)
   if (REQUEST_METHOD)
   {
     UdmBitrixIndex(&Main);
-    exit(0);
+    return 0;
   }
   
   if (cmd == UDM_IND_HASHSPELL)
   {
     char errmsg[256];
-    int rc; 
     if (UDM_OK != (rc= UdmSpellListListLoad(&Conf.Spells, errmsg, sizeof(errmsg)))||
         UDM_OK != (rc= UdmSpellListListWriteHash(&Conf.Spells, errmsg, sizeof(errmsg))))
     {
       fprintf(stderr, "error: %s\n", errmsg);
-      exit(1);
+      return 1;
     }
-    exit(0);
+    return 0;
   }
   
   if (cmd == UDM_IND_DUMPSPELL)
   {
     char errmsg[256];
-    int rc, spflags= UDM_SPELL_NOPREFIX;
+    int spflags= UDM_SPELL_NOPREFIX;
     if (UDM_OK != (rc= UdmSpellListListLoad(&Conf.Spells,
                                             errmsg, sizeof(errmsg)))||
         UDM_OK != (rc= UdmAffixListListLoad(&Conf.Affixes, spflags,
@@ -1457,9 +1640,9 @@ int main(int argc, char **argv)
                                     errmsg, sizeof(errmsg))))
     {
       fprintf(stderr, "error: %s\n", errmsg);
-      exit(1);
+      return 1;
     }
-    exit(0);
+    return 0;
   }
   
   if (cmd==UDM_IND_SQLMON)
@@ -1473,33 +1656,33 @@ int main(int argc, char **argv)
     prm.display= UdmDisplaySQLQuery;
     prm.prompt= sqlmonprompt;
     UdmSQLMonitor(&Main, &Conf, &prm);
-    exit(0);
+    return 0;
   }
 
   if (cmd == UDM_IND_MULTI2BLOB)
   {
-    UdmMulti2Blob(&Main);
-    exit(0);
+    rc= UdmMulti2Blob(&Main);
+    return rc;
   }
   else if (cmd == UDM_IND_EXPORT)
   {
-    UdmExport(&Main);
-    exit(0);
+    rc= UdmExport(&Main);
+    return rc;
   }
   else if (cmd == UDM_IND_WRDSTAT)
   {
-    UdmURLAction(&Main, NULL, UDM_URL_ACTION_WRDSTAT);
-    exit(0);
+    rc= UdmURLAction(&Main, NULL, UDM_URL_ACTION_WRDSTAT);
+    return rc;
   }
   else if (cmd == UDM_IND_REWRITEURL)
   {
-    UdmRewriteURL(&Main);
-    exit(0);
+    rc= UdmRewriteURL(&Main);
+    return rc;
   }     
   else if (cmd == UDM_IND_REWRITELIMITS)
   {
-    UdmRewriteLimits(&Main);
-    exit(0);
+    rc= UdmRewriteLimits(&Main);
+    return rc;
   }
      
   if (url_filename && strcmp(url_filename,"-"))
@@ -1617,7 +1800,7 @@ int main(int argc, char **argv)
         }
         UdmLog(&Main,UDM_LOG_ERROR, "indexer from %s-%s-%s started with '%s'", PACKAGE, VERSION, UDM_DBTYPE, cname);
         UdmStoreHrefs(&Main);    /**< store hrefs from config and command line */
-        UdmIndex(&Main);
+        rc= UdmIndex(&Main);
       }
   }
      
@@ -1637,7 +1820,7 @@ ex:
 #ifndef HAVE_ATEXIT
   exitproc();
 #endif
-  return(0);
+  return rc;
 }
 
 #endif /* ifdef CONSOLE */
