@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,15 +27,30 @@
 #include "udm_utils.h"
 #include "udm_wild.h"
 #include "udm_match.h"
-#include "udm_db.h"
 #include "udm_vars.h"
 
+
+static int
+IsSimpleWildEnding(const char *p)
+{
+  if (*p++ != '*' || !*p)
+    return 0;
+  for (; *p; p++)
+  {
+    if (*p == '*' || *p =='?')
+      return 0;
+  }
+  return 1;
+}
+
+
 int UdmMatchComp(UDM_MATCH *Match,char *errstr,size_t errstrsize)
 {
   int flag=REG_EXTENDED;
   int err;
   
   errstr[0]='\0';
+  Match->pattern_length= strlen(Match->pattern);
   
   switch(Match->match_type)
   {
@@ -50,7 +65,16 @@ int UdmMatchComp(UDM_MATCH *Match,char *errstr,size_t errstrsize)
         UDM_FREE(Match->reg);
         return(1);
       }
+      break;
     case UDM_MATCH_WILD:
+      if (IsSimpleWildEnding(Match->pattern))
+      {
+        /* Convert simple UDM_MATCH_WILD cases to UDM_MATCH_END */
+        memmove(Match->pattern, Match->pattern + 1, Match->pattern_length);
+        Match->match_type= UDM_MATCH_END;
+        Match->pattern_length--;
+      }
+      break;
     case UDM_MATCH_BEGIN:
     case UDM_MATCH_END:
     case UDM_MATCH_SUBSTR:
@@ -85,14 +109,14 @@ void UdmMatchFree(UDM_MATCH * Match)
 #define UDM_NSUBS 10
 
 int UdmMatchExec(UDM_MATCH * Match,
-                 const char * string, const char *net_string,
+                 const char *string, size_t string_length,
+                 const char *net_string,
                  size_t nparts,UDM_MATCH_PART * Parts)
 {
   size_t    i;
   int    res=0;
   regmatch_t  subs[UDM_NSUBS];
   const char  *se;
-  size_t    plen,slen;
 
 #ifdef USE_PARANOIA
   void *paran = UdmViolationEnter();
@@ -144,14 +168,13 @@ int UdmMatchExec(UDM_MATCH * Match,
       break;
     case UDM_MATCH_BEGIN:
       for(i=0;i<nparts;i++)Parts[i].beg=Parts[i].end=-1;
-      slen=strlen(Match->pattern);
       if(Match->case_sense)
       {
-        res=strncasecmp(Match->pattern,string,slen);
+        res=strncasecmp(Match->pattern, string, Match->pattern_length);
       }
       else
       {
-        res=strncmp(Match->pattern,string,slen);
+        res=strncmp(Match->pattern, string, Match->pattern_length);
       }
       break;
     case UDM_MATCH_FULL:
@@ -167,14 +190,12 @@ int UdmMatchExec(UDM_MATCH * Match,
       break;
     case UDM_MATCH_END:
       for(i=0;i<nparts;i++)Parts[i].beg=Parts[i].end=-1;
-      plen=strlen(Match->pattern);
-      slen=strlen(string);
-      if(slen<plen)
+      if(string_length < Match->pattern_length)
       {
-        res=1;
+        res= 1;
         break;
       }
-      se=string+slen-plen;
+      se= string + string_length - Match->pattern_length;
       if(Match->case_sense)
       {
         res=strcasecmp(Match->pattern,se);
@@ -196,57 +217,117 @@ int UdmMatchExec(UDM_MATCH * Match,
 }
 
 
-int UdmMatchApply(char *res,size_t size,const char *string,const char *rpl,
-      UDM_MATCH *Match, size_t nparts, UDM_MATCH_PART *Parts)
+static int
+UdmMatchSubNo(const char *str)
+{
+  if (*str == '$' && str[1] >= '0' && str[1] <= '9')
+    return str[1] - '0';
+  return -1;
+}
+
+static size_t
+UdmMatchPartLength(UDM_MATCH_PART *P)
+{
+  return ((P->beg > -1) && (P->end > P->beg)) ? P->end - P->beg : 0;
+}
+
+
+/*
+  Calc length which will be required for UdmMatchRegexApply(),
+  including traling '\0' character.
+*/
+static size_t
+UdmMatchRegexApplyLength(const char *repl, size_t nparts, UDM_MATCH_PART *Parts)
+{
+  size_t length;
+  for (length= 0; *repl; repl++)
+  {
+    int sub;
+    if ((sub= UdmMatchSubNo(repl)) >= 0)
+    {
+      size_t part_length= UdmMatchPartLength(&Parts[sub]);
+      length+= part_length;
+      repl++;
+    }
+    else
+    {
+      length++;
+    }
+  }
+  return length + 1;
+}
+
+
+/*
+  Apply "regex" type of match parts.
+  previously generated by UdmMatchExec result.
+  res    - where to put result to
+  size   - size of res, must be at least 1 byte - for trailing '\0'.
+  string - source string
+  rpl    - replacement pattern, with variables \0, \1, \2, ... \9
+  nparts - how many parts were found by UdmMatchExec
+  Parts  - the parts themself
+  
+  RETURNS - result length.
+*/
+static size_t
+UdmMatchRegexApply(char *res, size_t size,
+                   const char *string,const char *rpl,
+                   size_t nparts, UDM_MATCH_PART *Parts)
+{
+  char *dst= res, *dstend= res + size - 1;
+  const char *repl= rpl;
+  
+  if (!res)
+    return UdmMatchRegexApplyLength(rpl, nparts, Parts);
+  
+  UDM_ASSERT(size>0);
+  
+  while (*repl && dst < dstend)
+  {
+    int sub;
+    if ((sub= UdmMatchSubNo(repl)) >= 0)
+    {
+      size_t part_length;
+      if((part_length= UdmMatchPartLength(&Parts[sub])))
+      {
+        size_t bytes_left= dstend - dst;
+        if (bytes_left <= part_length)
+          part_length= bytes_left;
+        memcpy(dst, string + Parts[sub].beg, part_length);
+        dst+= part_length;
+      }
+      repl+= 2;
+    }
+    else
+    {
+      *dst++= *repl++;
+    }
+  }
+  *dst='\0';
+  return dst - res;
+}
+
+
+size_t
+UdmMatchApply(char *res, size_t size,
+              const char *string,const char *rpl,
+              UDM_MATCH *Match,
+              size_t nparts, UDM_MATCH_PART *Parts)
 {
-  char    *dst;
   int    len=0;
-  const char  *repl=rpl;
   if(!size)return 0;
   
   switch(Match->match_type)
   {
     case UDM_MATCH_REGEX:
-      dst=res;
-
-      while((*repl)&&((size_t)(dst-res)<(size-1)))
-      {
-        if(*repl=='$')
-        {
-          char digit[2];
-          int sub;
-          size_t len1;
-              
-          digit[0]=repl[1];
-          digit[1]='\0';
-          sub=atoi(digit);
-
-          if((Parts[sub].beg>-1)&&(Parts[sub].end>Parts[sub].beg))
-          {
-            size_t bytes_left = size - (dst - res);
-            len1=Parts[sub].end-Parts[sub].beg;
-            if (bytes_left <= len1) len1 = bytes_left;
-            strncpy(dst,string+Parts[sub].beg,len1);
-            dst+=len1;
-          }
-          *dst= '\0';
-          repl+=2;
-        }
-        else
-        {
-          *dst=*repl;
-          dst++;
-          *dst='\0';
-          repl++;
-        }
-      }
-      *dst='\0';
-      len=dst-res;
+      len= UdmMatchRegexApply(res, size, string, rpl, nparts, Parts);
       break;
       
     case UDM_MATCH_BEGIN:
       len = udm_snprintf(res,size-1,"%s%s",rpl,string+strlen(Match->pattern));
       break;
+      
     default:
       *res='\0';
       len=0;
@@ -297,11 +378,11 @@ void UdmMatchListFree(UDM_MATCHLIST *L)
 UDM_MATCH * __UDMCALL UdmMatchListFind(UDM_MATCHLIST *L, const char *str,
                                        size_t nparts,UDM_MATCH_PART *Parts)
 {
-  size_t i;
+  size_t i, slen= strlen(str);
   for(i=0;i<L->nmatches;i++)
   {
     UDM_MATCH *M=&L->Match[i];
-    if(!UdmMatchExec(M, str, str, nparts, Parts))
+    if(!UdmMatchExec(M, str, slen, str, nparts, Parts))
       return M;
   }
   return NULL;
@@ -318,7 +399,8 @@ UDM_MATCH * __UDMCALL UdmMatchSectionListFind(UDM_MATCHLIST *L,
   {
     UDM_MATCH *M=&L->Match[i];
     const char *str = UdmVarListFindStr(&Doc->Sections, M->section, "");
-    if(!UdmMatchExec(M, str, str, nparts, Parts))
+    size_t slen= strlen(str);
+    if(!UdmMatchExec(M, str, slen, str, nparts, Parts))
       return M;
   }
   return NULL;
