@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -48,6 +48,20 @@
 
 /****************************************************************/
 
+static UDM_VAR *
+UdmVarListFindByPrefix(UDM_VARLIST *Vars,
+                       const char *prefix, size_t prefix_length)
+{
+  size_t i;
+  for (i= 0; i < Vars->nvars; i++)
+  {
+    UDM_VAR *V= &Vars->Var[i];
+    if (!strncasecmp(V->name, prefix, prefix_length))
+      return V;
+  }
+  return NULL;
+}
+
 
 static int UdmReallocSection(UDM_AGENT *Indexer, UDM_VAR *Sec)
 {
@@ -91,25 +105,24 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
   UDM_CHARSET  *sys_int;
   UDM_CONV  dc_uni;
   UDM_CONV  uni_lc;
+  UDM_UNIDATA *unidata= Indexer->Conf->unidata;
   UDM_TEXTLIST  *tlist=&Doc->TextList;
   int    res=UDM_OK;
   int    crc32=0;
   int    crossec;
-  int    *uword;    /* Word in UNICODE      */
+  int    strip_accents;
   char   *lcsword;  /* Word in LocalCharset */
   size_t max_word_len, min_word_len, uwordlen = UDM_MAXWORDSIZE;
 #ifdef USE_PARANOIA
   void *paran = UdmViolationEnter();
 #endif
+  UDM_VAR *Sec;
+  int raw_sections= (UdmVarListFindByPrefix(&Doc->Sections, "Raw.", 4) != NULL);
+  const char *content_language= UdmVarListFindStr(&Doc->Sections, "Content-Language", "");
+  const char *seg=  UdmVarListFindStr(&Indexer->Conf->Vars, "Segmenter", NULL);
   
-  if ((uword = (int*)UdmMalloc((uwordlen + 1) * sizeof(int))) == NULL)
-  {
-    UdmViolationExit(paran);
-    return UDM_ERROR;
-  }
   if ((lcsword = (char*)UdmMalloc(12 * uwordlen + 1)) == NULL)
   {
-    UDM_FREE(uword); 
     UdmViolationExit(paran);
     return UDM_ERROR;
   }
@@ -119,6 +132,7 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
     crossec= S ? S->section : 0;
   }
   
+  strip_accents= UdmVarListFindBool(&Indexer->Conf->Vars, "StripAccents", 0);
   doccset=UdmVarListFindStr(&Doc->Sections,"Parser.Charset",NULL);
   if(!doccset)doccset=UdmVarListFindStr(&Doc->Sections,"Charset",NULL);
   if(!doccset||!*doccset)doccset=UdmVarListFindStr(&Doc->Sections,"RemoteCharset","iso-8859-1");
@@ -139,43 +153,53 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
   
   /* Now convert everything to UNICODE format and calculate CRC32 */
   
-  for(i=0;i<tlist->nitems;i++)
+  for(Sec= NULL, i= 0; i< tlist->nitems; i++)
   {
-    size_t srclen;
-    size_t srclen0;
-    size_t dstlen;
+    size_t srclen, dstlen, cnvlen, ustr_char_len;
     char   *src,*dst;
-    int    *lt, *tok, *ustr = NULL, *UStr = NULL;
+    int    *lt, *tok, *ustr = NULL;
     UDM_TEXTITEM  *Item=&tlist->Item[i];
     char    secname[128];
-    UDM_VAR *Sec;
-    
-    Sec=UdmVarListFind(&Doc->Sections,Item->section_name);
-  
-    srclen0=strlen(Item->str);
-    srclen=srclen0+1;      /* with '\0' */
-    dstlen=(3*srclen+1)*sizeof(int);  /* with '\0' */
+
+    if (!Sec || strcasecmp(Sec->name, Item->section_name))
+      Sec= UdmVarListFind(&Doc->Sections, Item->section_name);
+
+    srclen= strlen(Item->str);
+    dstlen= (3 * (srclen + 1))*sizeof(int);  /* with '\0' */
     
     if ((ustr = (int*)UdmMalloc(dstlen)) == NULL)
     {
       UdmLog(Indexer, UDM_LOG_ERROR, "%s:%d Can't alloc %u bytes", __FILE__, __LINE__, dstlen);
-      UDM_FREE(uword); UDM_FREE(lcsword);
-      UdmViolationExit(paran);
-      return UDM_ERROR;
+      res= UDM_ERROR;
+      goto ex;
     }
     
     src=Item->str;
     dst=(char*)ustr;
-    UdmConv(&dc_uni,dst,dstlen,src,srclen);
-    UdmUniRemoveDoubleSpaces(ustr);
-    if ((UStr = UdmUniDup(ustr)) == NULL)
+    cnvlen= UdmConv(&dc_uni,dst,dstlen,src,srclen) / 4;
+    ustr[cnvlen]= 0;
+    
+    ustr_char_len= UdmUniRemoveDoubleSpaces(ustr);
+
+    /* Append section text */
+    if(Sec && Sec->curlen < Sec->maxlen &&
+       !(Item->flags & UDM_TEXTLIST_FLAG_SKIP_ADD_SECTION))
     {
-      UdmLog(Indexer, UDM_LOG_ERROR, "%s:%d Can't UdmUniDup", __FILE__, __LINE__);
-      UDM_FREE(uword); UDM_FREE(lcsword); UDM_FREE(ustr);
-      UdmViolationExit(paran);
-      return UDM_ERROR;
+      int cnvres;
+      UdmReallocSection(Indexer,Sec);
+      src= (char*) ustr;
+      srclen= ustr_char_len * sizeof(int);
+      dstlen= Sec->maxlen - Sec->curlen;
+      cnvres= UdmConv(&uni_lc, Sec->val + Sec->curlen, dstlen, src, srclen);
+      Sec->curlen+= cnvres;
+      Sec->val[Sec->curlen]='\0';
+      if (cnvres < 0)
+      {
+        Sec->curlen= Sec->maxlen;
+      }
     }
     
+    
     /*
     TODO for clones detection:
     Replace any separators into space to ignore 
@@ -185,26 +209,21 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
     
     if (!Sec || !(Sec->flags & UDM_VARFLAG_NOCLONE))
     {
-      size_t reslen= UdmUniLen(ustr);
-      size_t j;
-      for (j=0; j < reslen; j++)
-      {
-        char p[2];
-        p[0]= (char) (ustr[j] >> 8) & 0xFF;
-        p[1]= (char) (ustr[j] & 0xFF);
-        crc32=UdmCRC32Update(crc32, p, 2);
-      }
+      crc32= UdmCRC32UpdateUnicode(crc32, ustr, ustr_char_len);
     }
     
     if(Item->section)
     {
-      UdmUniStrToLower(ustr);
-      ustr = UdmUniSegment(Indexer, ustr, UdmVarListFindStr(&Doc->Sections, "Content-Language", ""));
+      if (strip_accents)
+        UdmUniStrStripAccents(unidata, ustr);
+      UdmUniStrToLower(unidata, ustr);
+      ustr = UdmUniSegment(Indexer, ustr, content_language, seg);
 
-      for(tok=UdmUniGetToken(ustr,&lt); tok ; tok = UdmUniGetToken(NULL, &lt))
+      for(tok= UdmUniGetToken(unidata, ustr,&lt);
+          tok ;
+          tok = UdmUniGetToken(unidata, NULL, &lt))
       {
         size_t  tlen;        /* Word length          */ 
-        int  ures;
         
         tlen=lt-tok;
         
@@ -220,27 +239,17 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
         
           if (tlen > uwordlen)
           {
-            uwordlen = tlen;
-            if ((uword = (int*)UdmRealloc(uword, (uwordlen + 1) * sizeof(int))) == NULL)
+            uwordlen= tlen;
+            if ((lcsword= (char*) UdmRealloc(lcsword, 12 * uwordlen + 1)) == NULL)
             { 
-              UDM_FREE(lcsword); UDM_FREE(ustr); UDM_FREE(UStr); 
-              UdmViolationExit(paran);
-              return UDM_ERROR;
-            }
-            if ((lcsword = (char*)UdmRealloc(lcsword, 12 *uwordlen + 1)) == NULL)
-            { 
-              UDM_FREE(uword); UDM_FREE(ustr); UDM_FREE(UStr); 
-              UdmViolationExit(paran);
-              return UDM_ERROR;
+              res= UDM_ERROR;
+              goto ex;
             }
           }
 
-          memcpy(uword,tok,tlen*sizeof(int));
-          uword[tlen]=0;
-        
-          UdmConv(&uni_lc, lcsword, 12 * uwordlen + 1, (char*)uword, sizeof(*uword) * (tlen + 1));
-          ures=strlen(lcsword);
-        
+          cnvlen= UdmConv(&uni_lc, lcsword, 12 * uwordlen , (char*) tok, sizeof(*tok) * tlen);
+          lcsword[cnvlen]= '\0';
+          
           res=UdmWordListAdd(Doc,lcsword,Item->section);
           if(res!=UDM_OK)break;
         
@@ -249,7 +258,7 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
             UDM_CROSSWORD cw;
             cw.url=Item->href;
             cw.weight = crossec;
-            cw.pos=Doc->CrossWords.wordpos;
+            cw.pos=Doc->CrossWords.wordpos[crossec];
             cw.word=lcsword;
             UdmCrossListAdd(Doc,&cw);
           }
@@ -257,54 +266,40 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
       }
     }
     
-    if(Sec && Sec->curlen < Sec->maxlen &&
-       !(Item->flags & UDM_TEXTLIST_FLAG_SKIP_ADD_SECTION))
+    if (raw_sections)
     {
-      int cnvres;
-      UdmReallocSection(Indexer,Sec);
-
-      src = (char*)UStr;
-      srclen = UdmUniLen(UStr) * sizeof(int);
-      dstlen=Sec->maxlen-Sec->curlen;
-      cnvres=UdmConv(&uni_lc,Sec->val+Sec->curlen,dstlen,src,srclen);
-      Sec->curlen+=uni_lc.obytes;
-      Sec->val[Sec->curlen]='\0';
-      if (cnvres<0)
+      udm_snprintf(secname,sizeof(secname)-1,"Raw.%s",Item->section_name);
+      if ((Sec=UdmVarListFind(&Doc->Sections,secname)))
       {
-        Sec->curlen=Sec->maxlen;
-      }
-    }
-    
-    udm_snprintf(secname,sizeof(secname)-1,"Raw.%s",Item->section_name);
-    if ((Sec=UdmVarListFind(&Doc->Sections,secname)))
-    {
-      if(Sec->curlen < Sec->maxlen)
-      {
-        size_t nbytes;
-        
-        UdmReallocSection(Indexer,Sec);
-        
-        dstlen= Sec->maxlen-Sec->curlen;
-        nbytes= dstlen < srclen0 ? dstlen : srclen0;
-        memcpy(Sec->val+Sec->curlen,Item->str,nbytes);
-        Sec->curlen+=nbytes;
-        Sec->val[Sec->curlen]='\0';
-        
-        if (dstlen<srclen0)
+        if(Sec->curlen < Sec->maxlen)
         {
-          Sec->curlen=Sec->maxlen;
+          size_t nbytes;
+          
+          UdmReallocSection(Indexer,Sec);
+          
+          dstlen= Sec->maxlen-Sec->curlen;
+          nbytes= dstlen < srclen ? dstlen : srclen;
+          memcpy(Sec->val+Sec->curlen,Item->str,nbytes);
+          Sec->curlen+=nbytes;
+          Sec->val[Sec->curlen]='\0';
+          
+          if (dstlen < srclen)
+          {
+            Sec->curlen=Sec->maxlen;
+          }
         }
       }
     }
     
     UDM_FREE(ustr);
-    UDM_FREE(UStr);
     if(res!=UDM_OK)break;
     
   }
+
   UdmVarListReplaceInt(&Doc->Sections,"crc32",crc32);
-  
-  UDM_FREE(uword); UDM_FREE(lcsword);
+
+ex:
+  UDM_FREE(lcsword);
   UdmViolationExit(paran);
   return res;
 }
@@ -327,7 +322,7 @@ UdmTextListAddWithConversion(UDM_DOCUMENT *Doc,
     UDM_TEXTITEM Item;
     char  sec_name[64];
     UdmUnescapeCGIQuery(tmp, src);
-    UdmConv(cnv, dst, dstlen, tmp, strlen(tmp));
+    UdmConv(cnv, dst, dstlen, tmp, strlen(tmp) + 1);
     bzero(&Item, sizeof(Item));
     udm_snprintf(sec_name, sizeof(sec_name), "%s", name);
     Item.str= dst;
@@ -446,24 +441,9 @@ int UdmParseText(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc)
 }
 
 
-static void UdmNextCharB(void *d)
-{
-  UDM_HTMLTOK *t = (UDM_HTMLTOK *)d;
-  (t->b)++;
-}
-
-static void UdmNextCharE(void *d)
-{
-  UDM_HTMLTOK *t = (UDM_HTMLTOK *)d;
-  (t->e)++;
-}
-
-
 int UdmHTMLTOKInit(UDM_HTMLTOK *tag)
 {
   bzero((void*)tag, sizeof(*tag));
-  tag->next_b = &UdmNextCharB;
-  tag->next_e = &UdmNextCharE;
   return UDM_OK;
 }
 
@@ -493,74 +473,82 @@ static inline int IsSpaceChar(int x)
 }
 
 
-const char * UdmHTMLToken(const char * s, const char ** lt,UDM_HTMLTOK *t)
+/*
+  TODO: extend UdmHTMLToken() to unserstand ISO-2022-XX sequences, which
+  can have '<' and '>' as part of the text when in multi-byte mode, and
+  which don't mean the beginning and the end of the tag corrspondingly.
+  See uconv-eucjp.c for more details.
+*/
+const char * UdmHTMLToken(const char *src, const char ** lt,UDM_HTMLTOK *t)
 {
+  const char *b, *e;
   t->ntoks= 0;
-  t->s= s;
   
-  if(t->s == NULL && (t->s = *lt) == NULL)
+  if (!src && !(src= *lt))
     return NULL;
 
-  if (*t->s == '<')
+  if (*src == '<')
   {
-    t->type= strncmp(t->s + 1, "!--", 3) ? UDM_HTML_TAG : UDM_HTML_COM;
+    const char *tmp= src + 1;
+    t->type= (*tmp++ == '!' && *tmp++ == '-' && *tmp++== '-') ?
+             UDM_HTML_COM : UDM_HTML_TAG;
   }
   else
   {
-    if(!*t->s)
+    if(!*src)
       return NULL;
-    t->type=UDM_HTML_TXT;
+    t->type= UDM_HTML_TXT;
   }
   
   switch(t->type)
   {
     case UDM_HTML_TAG:
 
-      for(*lt = t->b = t->s + 1; *t->b; )
+      for(*lt= b= src + 1; *b; )
       {
-        const char * valbeg=NULL;
-        const char * valend=NULL;
-        size_t nt=t->ntoks;
+        const char *valbeg= NULL;
+        const char *valend= NULL;
+        size_t nt= t->ntoks;
         
         /* Skip leading spaces */
-        for ( ; IsSpaceChar((unsigned char)*t->b) ; t->b++);
+        for ( ; IsSpaceChar((unsigned char)*b) ; b++);
 
-        if(*t->b=='>')
+        if(*b == '>')
         {
-          *lt = t->b + 1;
-          return(t->s);
+          *lt= b + 1;
+          return src;
         }
 
-        if(*t->b=='<')
+        if(*b == '<')
         { /* Probably broken tag occure */
-          *lt = t->b;
-          return(t->s);
+          *lt= b;
+          return src;
         }
 
         /* Skip non-spaces, i.e. name */
-        for(t->e = t->b; (*t->e) && !strchr(" =>\t\r\n", *t->e); t->e++);
+        for (e= b; *e && !strchr(" =>\t\r\n", *e); e++);
         
-        if(t->ntoks<UDM_MAXTAGVAL)
+        if(t->ntoks < UDM_MAXTAGVAL)
           t->ntoks++;
         
-        t->toks[nt].val=0;
-        t->toks[nt].vlen=0;
-        t->toks[nt].name = t->b;
-        t->toks[nt].nlen = t->e - t->b;
+        t->toks[nt].val= 0;
+        t->toks[nt].vlen= 0;
+        t->toks[nt].name= b;
+        t->toks[nt].nlen= e - b;
 
         if (nt == 0)
         {
           const char *beg;
           int val;
-          if (t->b[0] == '/')
+          if (b[0] == '/')
           {
             val= 0;
-            beg= t->b + 1;
+            beg= b + 1;
           }
           else
           {
             val= 1;
-            beg= t->b;
+            beg= b;
           }
           if (!strncasecmp(beg,"script",6))
             t->script= val;
@@ -572,76 +560,82 @@ const char * UdmHTMLToken(const char * s, const char ** lt,UDM_HTMLTOK *t)
             t->body= val;
         }
 
-        if(*t->e=='>')
+        if(*e == '>')
         {
-          *lt = t->e + 1;
-          return(t->s);
+          *lt= e + 1;
+          return src;
         }
 
-        if(!(*t->e))
+        if(!*e)
         {
-          *lt = t->e;
-          return(t->s);
+          *lt= e;
+          return src;
         }
         
         /* Skip spaces */
-        for( ; IsSpaceChar((unsigned char) *t->e) ; t->e++);
+        for( ; IsSpaceChar((unsigned char) *e) ; e++);
         
-        if(*t->e != '=')
+        if(*e != '=')
         {
-          t->b = t->e;
-          *lt = t->b;        /* bug when hang on broken inside tag pages fix */
+          b= e;
+          *lt= b;        /* bug when hang on broken inside tag pages fix */
           continue;
         }
         
         /* Skip spaces */
-        for(t->b = t->e + 1; IsSpaceChar((unsigned char) *t->b); t->b++);
+        for(b= e + 1; IsSpaceChar((unsigned char) *b); b++);
         
-        if(*t->b == '"')
+        if(*b == '"')
         {
-          t->b++;
-          valbeg = t->b;
-          for(t->e = t->b; (*t->e) && (*t->e != '"'); t->e++);
-          valend = t->e;
-          t->b = t->e;
-          if(*t->b == '"')t->b++;
+          b++;
+          valbeg= b;
+          for (e= b; *e && *e != '"'; e++);
+          valend= e;
+          b= e;
+          if (*b == '"') b++;
         }
-        else if(*t->b == '\'')
+        else if (*b == '\'')
         {
-          t->b++;
-          valbeg = t->b;
-          for(t->e = t->b; (*t->e) && (*t->e != '\''); t->e++);
-          valend = t->e;
-          t->b = t->e;
-          if(*t->b == '\'')t->b++;
+          b++;
+          valbeg= b;
+          for (e= b; *e && *e != '\''; e++);
+          valend= e;
+          b= e;
+          if (*b == '\'')b++;
         }
         else
         {
-          valbeg = t->b;
-          for(t->e = t->b; (*t->e) && !strchr(" >\t\r\n", *t->e);t->e++);
-          valend = t->e;
-          t->b = t->e;
+          valbeg= b;
+          for (e= b; *e && !strchr(" >\t\r\n", *e); e++);
+          valend= e;
+          b= e;
         }
-        *lt = t->b;
-        t->toks[nt].val=valbeg;
-        t->toks[nt].vlen=valend-valbeg;
+        *lt= b;
+        t->toks[nt].val= valbeg;
+        t->toks[nt].vlen= valend - valbeg;
       }
       break;
 
     case UDM_HTML_COM: /* comment */
       
-      if(!strncasecmp(t->s, "<!--UdmComment-->",17) ||
-         !strncasecmp(t->s, "<!--noindex-->", 14) ||
-         !strncasecmp(t->s, "<!--X-BotPNI-->",15))
+      if(!strncasecmp(src, "<!--UdmComment-->",17) ||
+         !strncasecmp(src, "<!--noindex-->", 14) ||
+         !strncasecmp(src, "<!--X-BotPNI-->",15))
         t->comment=1;
-      else if(!strncasecmp(t->s, "<!--/UdmComment-->",18) ||
-              !strncasecmp(t->s, "<!--/noindex-->", 15) ||
-              !strncasecmp(t->s, "<!--X-BotPNI-End-->", 19))
+      else if(!strncasecmp(src, "<!--/UdmComment-->",18) ||
+              !strncasecmp(src, "<!--/noindex-->", 15) ||
+              !strncasecmp(src, "<!--X-BotPNI-End-->", 19))
         t->comment=0;
 
-      for(t->e = t->s; (*t->e) && (strncmp(t->e, "-->", 3)); t->e++);
-      if(!strncmp(t->e, "-->", 3)) *lt = t->e + 3;
-      else  *lt = t->e;
+      for (e= src; *e ; e++)
+      {
+        if (e[0] == '-' && e[1] == '-' && e[2] == '>')
+        {
+          *lt= e + 3;
+          return src;
+        }
+      }
+      *lt= e;
       break;
 
     case UDM_HTML_TXT: /* text */
@@ -651,19 +645,19 @@ const char * UdmHTMLToken(const char * s, const char ** lt,UDM_HTMLTOK *t)
       /* <script> x="<"; </script> */
       /* We should find </script>  */
       
-      for(t->e = t->s; *t->e; t->e++)
+      for (e= src; *e; e++)
       {
-        if(*t->e == '<')
+        if (*e == '<')
         {
           if(t->script)
           {
-            if(!strncasecmp(t->e, "</script>",9))
+            if (!strncasecmp(e, "</script>",9))
             {
               /* This is when script body  */
               /* is not hidden using <!--  */
               break;
             }
-            else if(!strncmp(t->e, "<!--",4))
+            else if (!strncmp(e, "<!--",4))
             {
               /* This is when script body  */
               /* is hidden but there are   */
@@ -679,10 +673,10 @@ const char * UdmHTMLToken(const char * s, const char ** lt,UDM_HTMLTOK *t)
         }
       }
       
-      *lt = t->e;
+      *lt= e;
       break;
   }
-  return t->s;
+  return src;
 }
 
 
@@ -692,112 +686,119 @@ UdmDocAddHref(UDM_DOCUMENT *Doc, char *href, const char *crosstext)
   UDM_HREF Href;
   UdmSGMLUnescape(href);
   UdmHrefInit(&Href);
-  Href.referrer = UdmVarListFindInt(&Doc->Sections, "Referrer-ID", 0);
-  Href.hops=1+UdmVarListFindInt(&Doc->Sections,"Hops",0);
-  Href.site_id = UdmVarListFindInt(&Doc->Sections, "Site_id", 0);
-  Href.url=href;
-  Href.method=UDM_METHOD_GET;
+  /*
+    referrer, hops and site_id will be set later,
+    in the end of UdmHTMLParse()
+  */
+  Href.url= href;
+  Href.method= UDM_METHOD_GET;
   if (crosstext)
   {
     urlid_t url_id= UdmVarListFindInt(&Doc->Sections, "ID", 0);
     UdmVarListAddStr(&Href.Vars, "CrossText", crosstext);
     UdmVarListAddInt(&Href.Vars, "Referrer-ID", url_id);
   }
-  UdmHrefListAdd(&Doc->Hrefs,&Href);
+  UdmHrefListAdd(&Doc->Hrefs, &Href);
 }
 
 
+static inline int
+udm_strnncasecmp(const char *s, size_t slen, const char *t, size_t tlen)
+{
+  if (slen != tlen)
+    return 1;
+  return strncasecmp(s, t, slen);
+}
+
+
+#define UDM_CSTR_WITH_LEN(x)  (x), ((size_t) (sizeof(x) - 1))
+
+
+
 int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
 {
   UDM_TEXTITEM Item;
   UDM_VAR  *Sec;
-  int opening;
-  char name[128];
-  register char * n;
+  int opening, rel_nofollow= 0;
   char *metaname=NULL;
   char *metacont=NULL;
   char *href=NULL;
   char *lang = NULL;
-  char *secname;
-  size_t i, seclen = 128;
+  size_t i;
+  const char *nameptr= tag->toks[0].name;
+  size_t namelen= tag->toks[0].nlen;
 
 #ifdef USE_PARANOIA
   void *paran = UdmViolationEnter();
 #endif
 
-  if(!tag->ntoks)
-  {
-    UdmViolationExit(paran);
-    return(0);
-  }
-  if(!tag->toks[0].name)
-  {
-    UdmViolationExit(paran);
-    return(0);
-  }
-  if(tag->toks[0].nlen>sizeof(name)-1)
+  if(!tag->ntoks || !nameptr)
   {
     UdmViolationExit(paran);
     return(0);
   }
 
-  secname = (char*)UdmMalloc(seclen);
-  strncpy(name,tag->toks[0].name,tag->toks[0].nlen);
-  name[tag->toks[0].nlen]='\0';
-  
+    
   for(i=0;i<tag->ntoks;i++)
   {
-    if(ISTAG(i,"name"))
+    const char *aname= tag->toks[i].name;
+    size_t alen= tag->toks[i].nlen;
+    if(!udm_strnncasecmp(aname, alen, UDM_CSTR_WITH_LEN("name")))
     {
-      metaname = UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
+      metaname= UdmStrndup(tag->toks[i].val, tag->toks[i].vlen);
     }
-    else if(ISTAG(i,"http-equiv"))
+    else if(!udm_strnncasecmp(aname, alen, UDM_CSTR_WITH_LEN("http-equiv")))
     {
-      metaname = UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
+      metaname= UdmStrndup(tag->toks[i].val, tag->toks[i].vlen);
     }
-    else if(ISTAG(i,"content"))
+    else if(!udm_strnncasecmp(aname, alen, UDM_CSTR_WITH_LEN("content")))
     {
-      metacont = UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
+      metacont= UdmStrndup(tag->toks[i].val, tag->toks[i].vlen);
     }
-    else if(ISTAG(i,"href"))
+    else if(!udm_strnncasecmp(aname, alen, UDM_CSTR_WITH_LEN("href")))
     {
       /* A, LINK, AREA*/
-      char *y = UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
-      href = (char*)UdmStrdup(UdmTrim(y, " \t\r\n"));
+      char *y= UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
+      href= (char*)UdmStrdup(UdmTrim(y, " \t\r\n"));
       UDM_FREE(y);
     }
-    else if(ISTAG(i,"src"))
+    else if(!udm_strnncasecmp(aname, alen, UDM_CSTR_WITH_LEN("rel")))
+    {
+      char param[64], *y;
+      size_t paramlen= udm_min(tag->toks[i].vlen, sizeof(param) - 1);
+      memcpy(param, tag->toks[i].val, paramlen);
+      param[paramlen]= '\0';
+      y= UdmTrim(param, " \t\r\n");
+      rel_nofollow= !strcasecmp(y, "nofollow");
+    }
+    else if(!udm_strnncasecmp(aname, alen, UDM_CSTR_WITH_LEN("src")))
     {
       /* IMG, FRAME */
-      char *y = UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
-      href = (char*)UdmStrdup(UdmTrim(y, " \t\r\n"));
+      char *y= UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
+      href= (char*)UdmStrdup(UdmTrim(y, " \t\r\n"));
       UDM_FREE(y);
     }
-    else if (ISTAG(i, "lang"))
+    else if (!udm_strnncasecmp(aname, alen, UDM_CSTR_WITH_LEN("lang")))
     {
-      char *y = UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
-      lang = (char*)UdmStrdup(UdmTrim(y, " \t\r\n"));
+      char *n;
+      char *y= UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
+      lang= (char*)UdmStrdup(UdmTrim(y, " \t\r\n"));
       for(n = lang; *n; *n = tolower(*n),n++);
       UDM_FREE(y);
     }
-    else
+    else if (!tag->skip_attribute_sections)
     {
-      if (tag->toks[i].nlen + 12 > seclen)
-      {
-        secname = (char*)UdmRealloc(secname, seclen = (tag->toks[i].nlen + 12));
-      }
-      
-      strcpy(secname, "attribute.");
-      strncat(secname + 10, tag->toks[i].name, tag->toks[i].nlen);
-      secname[seclen - 1]='\0';
+      char secname[128];
+      udm_snprintf(secname, sizeof(secname), "attribute.%.*s",
+                   tag->toks[i].nlen, tag->toks[i].name);
 
-      if ((Sec = UdmVarListFind(&Doc->Sections, secname)) && Doc->Spider.index)
+      if ((Sec= UdmVarListFind(&Doc->Sections, secname)) && Doc->Spider.index)
       {
-        char *y = UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
-        Item.str = y;
-        Item.section = Sec->section;
-        Item.section_name = secname;
-        Item.href = NULL;
+        char *y= UdmStrndup(tag->toks[i].val,tag->toks[i].vlen);
+        Item.str= y;
+        Item.section= Sec->section;
+        Item.section_name= secname;
+        Item.href= NULL;
         Item.flags= 0;
         UdmTextListAdd(&Doc->TextList, &Item);
         UDM_FREE(y);
@@ -805,12 +806,11 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
     }
   }
   
-  for(n=name;*n;*n=tolower(*n),n++);
-  
-  if(name[0]=='/')
+  if(nameptr[0] == '/')
   {
-    opening=0;
-    memmove(name,name+1,strlen(name+1)+1);
+    opening= 0;
+    nameptr++;
+    namelen--;
   }
   else
   {
@@ -820,17 +820,18 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
   tag->nonbreaking= 0;
   /* Let's find tag name in order of frequency */
 
-  if(!strcmp(name,"a"))
+  if(!udm_strnncasecmp(nameptr, namelen, "a", 1))
   {
     UDM_FREE(tag->lasthref);      /*117941*/
   }
-  else if(!strcmp(name,"title"))
+  else if(!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("title")))
     tag->title=opening;  /*6192*/
-  else if(!strcmp(name,"html") && opening && (lang != NULL))
+  else if(!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("html")))
   {
-    UdmVarListReplaceStr(&Doc->Sections, "Meta-Language", lang);
+    if (opening && lang != NULL)
+      UdmVarListReplaceStr(&Doc->Sections, "Meta-Language", lang);
   }
-  else if(!strcmp(name,"body"))
+  else if(!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("body")))
   {
     tag->body=opening;  /*5146*/
     if (opening && (lang != NULL))
@@ -838,12 +839,11 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
       UdmVarListReplaceStr(&Doc->Sections, "Meta-Language", lang);
     }
   }
-  else if((!strcmp(name,"meta"))&&(metaname)&&(metacont))
-  { 
-    strcpy(secname,"meta.");
-    strncat(secname + 5 ,metaname, seclen - 5);
-    secname[seclen - 1]='\0';
-    
+  else if(!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("meta")) &&
+          metaname && metacont)
+  {
+    char secname[128];
+    udm_snprintf(secname, sizeof(secname), "meta.%s", metaname);
     if((!tag->comment) && (Sec=UdmVarListFind(&Doc->Sections,secname)) && 
        Doc->Spider.index)
     {
@@ -868,7 +868,7 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
     else if(!strcasecmp(metaname, "Content-Language") ||
             !strcasecmp(metaname, "DC.Language"))
     {
-      char *l;
+      char *l, *n ;
       l = (char*)UdmStrdup(metacont);
       for(n = l; *n; *n = tolower(*n),n++);
       UdmVarListReplaceStr(&Doc->Sections, "Meta-Language", l);
@@ -940,41 +940,56 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
       }
     }
   }
-  else if(!strcmp(name,"script"))  tag->script=opening;
-  else if(!strcmp(name,"style"))  tag->style=opening;
-  else if(!strcmp(name,"noindex"))  tag->comment=opening;
-  else if((!strcmp(name,"base"))&&(href))
+  else if(!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("script")))
+    tag->script=opening;
+  else if(!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("style")))
+    tag->style=opening;
+  else if(!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("noindex")))
+    tag->comment=opening;
+  else if(!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("base")))
   {
-    UdmVarListReplaceStr(&Doc->Sections,"base.href",href);
-    /* Do not add BASE HREF itself into database.      */
-    /* It will be used only to compose relative links. */
-    UDM_FREE(href);
+    if (href)
+    {
+      UdmVarListReplaceStr(&Doc->Sections,"base.href",href);
+      /*
+        Do not add BASE HREF itself into database.
+        It will be used only to compose relative links.
+      */
+      UDM_FREE(href);
+    }
   }
-  else if (!strcmp(name,"font"))
+  else if (!udm_strnncasecmp(nameptr, namelen, UDM_CSTR_WITH_LEN("font")))
   {
     tag->nonbreaking= 1;
   }
 
   if((href)&&(tag->follow!=UDM_FOLLOW_NO))
   {
-    if (strcasecmp(name, "a"))
+    if (udm_strnncasecmp(nameptr, namelen, "a", 1))
     {
-      /* No crosstext, add the link right now*/
-      UdmDocAddHref(Doc, href, NULL);
+      /*Add the link only if there's no 'nofollow'*/
+      if(!rel_nofollow)
+      {
+        UdmDocAddHref(Doc, href, NULL);
+        /* ...No crosstext, add the link right now*/
+      }
     }
     else
     {
       /* TODO: Postpone adding, for crosswords and SQLExportHref */
       UDM_FREE(tag->lasthref);
-      tag->lasthref = (char*)UdmStrdup(href);
-      UdmDocAddHref(Doc, href, NULL);
+      /*Add the link only if there's no 'nofollow'*/
+      if(!rel_nofollow)
+      {
+        tag->lasthref= (char*) UdmStrdup(href);
+        UdmDocAddHref(Doc, href, NULL);
+      }
     }
   }
   UDM_FREE(metaname);
   UDM_FREE(metacont);
   UDM_FREE(href);
   UDM_FREE(lang);
-  UDM_FREE(secname);
   
   UdmViolationExit(paran);
   return 0;
@@ -989,18 +1004,23 @@ typedef struct
   char * ofs;
 } UDM_TAGSTACK;
 
+
 int UdmHTMLParse(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
   UDM_HTMLTOK  tag;
   UDM_TEXTITEM Item;
   const char   *htok;
   const char   *last;
   UDM_VAR      *BSec=UdmVarListFind(&Doc->Sections,"body");
+  UDM_VAR      *NBSec=UdmVarListFind(&Doc->Sections,"nobody");
   UDM_VAR      *TSec=UdmVarListFind(&Doc->Sections,"title");
   int          body_sec  = BSec ? BSec->section : 0;
+  int          nobody_sec= NBSec ? NBSec->section : 0;
   int          title_sec = TSec ? TSec->section : 0;
   int          body_sec_flags= BSec ? BSec->flags : 0;
+  int          nobody_sec_flags= NBSec ? NBSec->flags : 0;
   int          title_sec_flags= TSec ? TSec->flags : 0;
   char         scb[]="body";
+  char         scnb[]="nobody";
   char         sct[]="title";
   /*
     We cannot use BSec and TSec later, as they are
@@ -1014,12 +1034,13 @@ int UdmHTMLParse(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
   UdmHTMLTOKInit(&tag);
   tag.follow=Doc->Spider.follow;
   tag.index=Doc->Spider.index;
+  tag.skip_attribute_sections=
+    (UdmVarListFindByPrefix(&Doc->Sections, "attribute.", 10) == NULL);
 
   htok=UdmHTMLToken(Doc->Buf.content,&last,&tag);
   
   while(htok)
   {
-    char       *tmp=NULL;
     const char *tmpbeg;
     const char *tmpend;
     
@@ -1031,36 +1052,51 @@ int UdmHTMLParse(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
       case UDM_HTML_TXT:
         for( tmpbeg=htok;   tmpbeg<last && strchr(" \r\n\t",tmpbeg[0]) ; tmpbeg++);
         for( tmpend=last-1; htok<tmpend && strchr(" \r\n\t",tmpend[0]) ; tmpend--);
-        if(tmpbeg>=tmpend)break;
+        if (tmpbeg >= tmpend || tag.comment || tag.style ||
+                                tag.script  || !tag.index)
+          break;
         
-        tmp = UdmStrndup(tmpbeg,(size_t)(tmpend-tmpbeg+1));
+        Item.str= UdmStrndup(tmpbeg, (size_t) (tmpend - tmpbeg + 1));
         
-        if (body_sec && !(body_sec_flags & UDM_VARFLAG_USERDEF) &&
-            !tag.comment && tag.body && !tag.script && !tag.style && tag.index)
+        if (tag.body)
         {
-          Item.href=tag.lasthref;
-          Item.str=tmp;
-          Item.section=body_sec;
-          Item.section_name=scb;
-          Item.flags= 0;
+          if (body_sec && !(body_sec_flags & UDM_VARFLAG_USERDEF))
+          {
+            Item.href=tag.lasthref;
+            Item.section=body_sec;
+            Item.section_name=scb;
+            Item.flags= 0;
 #if NOT_READY_YET
-          if (tag.nonbreaking)
-            UdmTextListAppend(&Doc->TextList, &Item);
-          else
+            if (tag.nonbreaking)
+              UdmTextListAppend(&Doc->TextList, &Item);
+            else
 #endif
-            UdmTextListAdd(&Doc->TextList,&Item);
+              UdmTextListAdd(&Doc->TextList,&Item);
+          }
         }
-        if (title_sec && !(title_sec_flags & UDM_VARFLAG_USERDEF) &&
-            !tag.comment && tag.title && tag.index)
+        else if (tag.title)
         {
-          Item.href=NULL;
-          Item.str=tmp;
-          Item.section=title_sec;
-          Item.section_name=sct;
-          Item.flags= 0;
-          UdmTextListAdd(&Doc->TextList,&Item);
+          if (title_sec && !(title_sec_flags & UDM_VARFLAG_USERDEF))
+          {
+            Item.href=NULL;
+            Item.section=title_sec;
+            Item.section_name=sct;
+            Item.flags= 0;
+            UdmTextListAdd(&Doc->TextList,&Item);
+          }
         }
-        UDM_FREE(tmp);
+        else /* Text outside body and title */
+        {
+          if (nobody_sec && !(nobody_sec_flags & UDM_VARFLAG_USERDEF))
+          {
+            Item.href= tag.lasthref;
+            Item.section= nobody_sec;
+            Item.section_name= (nobody_sec == body_sec) ? scb : scnb;
+            Item.flags= 0;
+            UdmTextListAdd(&Doc->TextList, &Item);
+          }
+        }
+        UDM_FREE(Item.str);
         break;
 
       case UDM_HTML_TAG:  
@@ -1068,8 +1104,25 @@ int UdmHTMLParse(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
         break;
     }
     htok=UdmHTMLToken(NULL,&last,&tag);
-    
   }
+  
+  /* Set size_id, referrer and hops */
+  {
+    size_t i;
+    UDM_HREF Tmp;
+    Tmp.referrer= UdmVarListFindInt(&Doc->Sections, "Referrer-ID", 0);
+    Tmp.hops= 1 + UdmVarListFindInt(&Doc->Sections,"Hops",0);
+    Tmp.site_id= UdmVarListFindInt(&Doc->Sections, "Site_id", 0);
+
+    for (i= 0; i < Doc->Hrefs.nhrefs; i++)
+    {
+      UDM_HREF *Href= &Doc->Hrefs.Href[i];
+      Href->referrer= Tmp.referrer;
+      Href->hops= Tmp.hops;
+      Href->site_id= Tmp.site_id;
+    }
+  }
+  
   UDM_FREE(tag.lasthref);  
   UdmViolationExit(paran);
   return UDM_OK;
