@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -156,7 +156,7 @@ static int StoreDocCGI (UDM_AGENT *Agent, UDM_ENV *Env, UDM_VARLIST *tmpl)
   char         *HEnd= NULL;
   char         *last= NULL;
   char         ch;
-  const char   *content_type, *charset, *htok;
+  const char   *content_type, *charset, *htok, *qstring;
   UDM_CHARSET  *cs;
 #ifdef USE_PARSER
   UDM_PARSER   *Parser;
@@ -167,6 +167,22 @@ static int StoreDocCGI (UDM_AGENT *Agent, UDM_ENV *Env, UDM_VARLIST *tmpl)
   UdmPrepare(Agent,Res);
   UdmVarListReplaceStr(&Doc->Sections, "URL", UdmVarListFindStr(&Env->Vars, "URL", "0"));
   UdmVarListReplaceStr(&Doc->Sections, "dbnum", UdmVarListFindStr(&Env->Vars, "dbnum", "0"));
+  if ((qstring= UdmVarListFindStr(&Env->Vars, "ENV.QUERY_STRING", NULL)))
+  {
+    /* Remove dbnum from query string */
+    if (!strncmp(qstring, "dbnum=", 6))
+    {
+      char tmp[256];
+      for (qstring+= 6 ; (qstring[0] >= '0' && qstring[0] <= '9') || qstring[0]=='&' ; qstring++);
+      udm_snprintf(tmp, sizeof(tmp), "%s", qstring);
+      UdmVarListReplaceStr(&Doc->Sections, "ENV.QUERY_STRING", tmp);
+      UdmVarListReplaceStr(&Env->Vars, "ENV.QUERY_STRING", tmp);
+    }
+    else
+    {
+      UdmVarListReplaceStr(&Doc->Sections, "ENV.QUERY_STRING", UdmVarListFindStr(&Env->Vars, "ENV.QUERY_STRING", NULL));
+    }
+  }
   UdmURLAction(Agent, Doc, UDM_URL_ACTION_GET_CACHED_COPY);
   UdmVarListReplaceLst(&Env->Vars, &Doc->Sections, NULL, "*");
   
@@ -297,27 +313,16 @@ int UdmThreadJoin(void *thd)
 #endif
 
 
-static int CreateAlias(UDM_ENV *Conf, UDM_DOCUMENT *Doc)
+static int
+UdmHTTPHeadersSend(int httpd, const char *content_type, const char *bcs)
 {
-  char *alcopy, *aliastr;
-  UDM_MATCH    *Alias;
-  UDM_MATCH_PART Parts[10];
-
-  /* Create "Alias" variable */
-  alcopy=UdmRemoveHiLightDup(UdmVarListFindStr(&Doc->Sections,"URL",""));
-  if ((Alias=UdmMatchListFind(&Conf->Aliases,alcopy,10,Parts)))
+  if (httpd && strcasecmp(content_type, "none"))
   {
-    size_t nbytes= strlen(Alias->arg) + strlen(alcopy) + 64;
-    aliastr= (char*)UdmMalloc(nbytes);
-    UdmMatchApply(aliastr, nbytes, alcopy, Alias->arg, Alias, 10, Parts);
+    if (bcs)
+      printf("Content-type: %s; charset=%s\r\n\r\n", content_type, bcs);
+    else
+      printf("Content-Type: %s\r\n\r\n", content_type);
   }
-  else
-  {
-    aliastr = (char*)UdmStrdup(alcopy);
-  }
-  UdmVarListReplaceStr(&Doc->Sections,"Alias", aliastr);
-  UdmFree(aliastr);
-  UdmFree(alcopy);
   return UDM_OK;
 }
 
@@ -346,7 +351,7 @@ int main(int argc, char ** argv)
   /* REQUEST_METHOD too     to be safe     */
   
   httpd=(getenv("QUERY_STRING")||getenv("REQUEST_METHOD"));
-  
+
 #if ( (defined WIN32) && (defined _DEBUG) )
   if(httpd)
   {
@@ -358,6 +363,8 @@ int main(int argc, char ** argv)
   UdmInit();
   Env=UdmEnvInit(NULL);
 #if defined(HAVE_PTHREAD) && !defined(WIN32)
+  UdmInitMutexes();
+  UdmSetLockProc(Env, UdmLockProc);
   Env->ThreadCreate= UdmThreadCreate;
   Env->ThreadJoin= UdmThreadJoin;
 #endif
@@ -365,6 +372,7 @@ int main(int argc, char ** argv)
   UdmVarListInit(&query_vars);
   Agent = UdmAgentInit(NULL, Env, 0);
   UdmVarListAddEnviron(&Env->Vars,"ENV");
+  UdmVarListReplaceStr(&Env->Vars, "version", VERSION);
   
   /* Detect self */
   if((env= getenv("UDMSEARCH_SELF")) || (env= getenv("REQUEST_URI")))
@@ -415,7 +423,8 @@ int main(int argc, char ** argv)
         if (s[0]=='\\')s[0]='/';
     }
     
-    conf_dir= (env= getenv("UDM_ETC_DIR")) ? (char *)UdmStrdup(env) : UdmConfDir();
+    conf_dir= ((env= getenv("UDM_CONF_DIR")) || (env= getenv("UDM_ETC_DIR"))) ?
+               (char *)UdmStrdup(env) : UdmConfDir();
     /* Take from the config directory */
     udm_snprintf(template_name, sizeof(template_name)-5, "%s%s%s", 
                  conf_dir,UDMSLASHSTR,name);
@@ -428,7 +437,8 @@ int main(int argc, char ** argv)
     char *end, *conf_dir;
     size_t length;
     
-    conf_dir= (env= getenv("UDM_ETC_DIR")) ? (char *)UdmStrdup(env) : UdmConfDir();
+    conf_dir= ((env= getenv("UDM_CONF_DIR")) || (env= getenv("UDM_ETC_DIR"))) ?
+               (char *)UdmStrdup(env) : UdmConfDir();
     
     /* Take from the config directory */
     length= udm_snprintf(template_name, sizeof(template_name)-5, "%s%s%s", 
@@ -450,6 +460,7 @@ int main(int argc, char ** argv)
   {
     query_string = (char*)UdmRealloc(query_string, strlen(argv[1]) + 10);
     sprintf(query_string, "q=%s", argv[1]);
+    UdmVarListReplaceStr(&Env->Vars, "ENV.QUERY_STRING", query_string);
   }
   else
   {
@@ -482,8 +493,7 @@ int main(int argc, char ** argv)
   
   if((res=UdmTemplateLoad(Agent,template_name,&tmpl)))
   {
-    if(httpd)
-      printf("Content-Type: text/plain\r\n\r\n");
+    UdmHTTPHeadersSend(httpd, "text/plain", NULL);
     printf("%s\n",Env->errstr);
     UdmVarListFree(&tmpl);
     UdmVarListFree(&query_vars);
@@ -534,16 +544,14 @@ int main(int argc, char ** argv)
   
   if (! Env->bcs)
   {
-    if (httpd)
-      printf("Content-Type: text/plain\r\n\r\n");
+    UdmHTTPHeadersSend(httpd, "text/plain", NULL);
     printf("Unknown BrowserCharset '%s' in template '%s'\n",bcharset,template_name);
     exit(0);
   }
   
   if(! Env->lcs)
   {
-    if (httpd)
-      printf("Content-Type: text/plain\r\n\r\n");
+    UdmHTTPHeadersSend(httpd, "text/plain", NULL);
     printf("Unknown LocalCharset '%s' in template '%s'\n",lcharset,template_name);
     exit(0);
   }
@@ -555,8 +563,9 @@ int main(int argc, char ** argv)
     return(StoreDocCGI(Agent, Env, &tmpl));
   }
   
-  if (httpd)
-    printf("Content-type: text/html; charset=%s\r\n\r\n",bcharset);
+  UdmHTTPHeadersSend(httpd,
+                     UdmVarListFindStr(&Env->Vars, "ResultContentType", "text/html"),
+                     bcharset);
   
   /* These parameters taken from "variable section of template"*/
   
@@ -773,52 +782,10 @@ int main(int argc, char ** argv)
   {
     UDM_DOCUMENT *Doc=&Res->Doc[i];
     UDM_CATEGORY C;
-    char         *clist, *eu;
-    const char   *u;
-    size_t       cl, sc, clistsize;
-    urlid_t      dc_url_id = (urlid_t)UdmVarListFindInt(&Doc->Sections, "ID", 0);
-    urlid_t      dc_origin_id = (urlid_t)UdmVarListFindInt(&Doc->Sections, "Origin-ID", 0);
-    
-    CreateAlias(Agent->Conf, Doc);
-    
-    /* Skip clones */
-    if (dc_origin_id)
-      continue;
-    
-    clist= (char*)UdmMalloc(2048); /* FIXME */
-    clist[0]= '\0';
-    clistsize= 0;
-    for (cl=0;cl<Res->num_rows;cl++)
-    {
-      UDM_DOCUMENT *Clone=&Res->Doc[cl];
-      urlid_t      cl_origin_id = (urlid_t)UdmVarListFindInt(&Clone->Sections, "Origin-ID", 0);
-      
-      if ((dc_url_id == cl_origin_id) && cl_origin_id)
-      {
-        UDM_VARLIST  CloneVars;
-        
-        UdmVarListInit(&CloneVars);
-        UdmVarListAddLst(&CloneVars,&Env->Vars,NULL,"*");
-        UdmVarListReplaceLst(&CloneVars,&Clone->Sections,NULL,"*");
-        CreateAlias(Agent->Conf, Clone);
-        clist = (char*)UdmRealloc(clist, (clistsize = strlen(clist)) + 2048);
-        UdmTemplatePrint(Agent, NULL, clist + clistsize, 2048, &CloneVars, &tmpl, "clone");
-        UdmVarListFree(&CloneVars);
-      }
-    }
-    clistsize += 2048;
-    
-    UdmVarListReplaceStr(&Env->Vars,"CL",clist);
-    UdmVarListReplaceInt(&Env->Vars, "ID", dc_url_id);
-    UdmVarListReplaceStr(&Env->Vars, "URL", UdmVarListFindStr(&Doc->Sections, "URL", ""));
-    UdmVarListReplaceStr(&Env->Vars, "title", "[no title]");
-    
-    /* Pass all found user-defined sections */
-    for(sc=0;sc<Doc->Sections.nvars;sc++)
-    {
-      UDM_VAR *S=&Doc->Sections.Var[sc];
-      UdmVarListReplaceStr(&Env->Vars,S->name,S->val);
-    }
+    const char   *stored_href;
+    size_t       sc;
+    UDM_VARLIST Merge;
+    int dbnum= UdmVarListFindInt(&Doc->Sections, "dbnum", 0);
     
     bzero((void*)&C, sizeof(C));
     strcpy(C.addr,UdmVarListFindStr(&Doc->Sections,"Category",""));
@@ -842,27 +809,41 @@ int main(int argc, char ** argv)
     }
     UDM_FREE(C.Category);
     
-    u= UdmVarListFindStr(&Env->Vars, "URL", "");
-    eu= (char*)UdmMalloc(strlen(u)*10 + 30);
-    UdmEscapeURL(eu, u);
-    if (UdmVarListFindInt(&Doc->Sections, "dbnum", 0))
+    stored_href= UdmVarListFindStr(&Doc->Sections, "stored_href", NULL);
+    if ((!(stored_href= UdmVarListFindStr(&Doc->Sections, "stored_href", NULL)) ||
+         !stored_href[0]) &&
+         UdmVarListFindStr(&Doc->Sections, "CachedCopy", NULL))
     {
-      udm_snprintf(storedstr, storedlen, "?cc=1&amp;dbnum=%s&amp;URL=%s&amp;q=%s&amp;wm=%s",
-        UdmVarListFindStr(&Doc->Sections, "dbnum", "0"),
-        eu,
-        searchwords,
-        UdmVarListFindStr(&Env->Vars, "wm", "wrd"));
+      const char *u= UdmVarListFindStr(&Doc->Sections, "URL", "");
+      char *eu= (char*)UdmMalloc(strlen(u)*10 + 30);
+      UdmEscapeURL(eu, u);
+      if (dbnum)
+      {
+        udm_snprintf(storedstr, storedlen, "?cc=1&amp;dbnum=%d&amp;URL=%s&amp;q=%s&amp;wm=%s",
+                     dbnum, eu, searchwords,
+                     UdmVarListFindStr(&Env->Vars, "wm", "wrd"));
+      }
+      else
+      {
+        udm_snprintf(storedstr, storedlen, "?cc=1&amp;URL=%s&amp;q=%s&amp;wm=%s",
+                     eu, searchwords,
+                     UdmVarListFindStr(&Env->Vars, "wm", "wrd"));
+      }
+      UdmFree(eu);
+      
+      UdmVarListReplaceStr(&Doc->Sections, "stored_href",  storedstr);
     }
     else
     {
-      udm_snprintf(storedstr, storedlen, "?cc=1&amp;URL=%s&amp;q=%s&amp;wm=%s",
-        eu,
-        searchwords,
-        UdmVarListFindStr(&Env->Vars, "wm", "wrd"));
+      /* add dbnum to stored_href returned from searchd */
+      if (stored_href && dbnum)
+      {
+        char tmp[256];
+        udm_snprintf(tmp, sizeof(tmp), "?dbnum=%d&%s", dbnum, stored_href + 1);
+        UdmVarListReplaceStr(&Doc->Sections, "stored_href", tmp);
+      }
     }
-    UdmFree(eu);
     
-    UdmVarListReplaceStr(&Env->Vars, "stored_href", UdmVarListFindStr(&Doc->Sections, "CachedCopy", NULL) ? storedstr : "");
     if ( (sc = UdmVarListFindInt(&Env->Vars, "site", 0)) == 0)
     {
       UdmVarListReplaceOrDelInt(&query_vars,"np", 0);
@@ -871,16 +852,11 @@ int main(int argc, char ** argv)
       UdmVarListReplaceStr(&Env->Vars, "sitelimit_href", url);
     }
     
-    UdmTemplatePrint(Agent, stdout, NULL, 0, &Env->Vars, &tmpl, "res");
+    UdmVarListInit(&Merge);
+    UdmVarListMerge(&Merge, &Env->Vars, &Doc->Sections);
+    UdmTemplatePrint(Agent, stdout, NULL, 0, &Merge, &tmpl, "res");
+    UdmVarListFree(&Merge);
     
-    /* Revoke all found user-defined sections */
-    for(sc = 0; sc < Doc->Sections.nvars; sc++)
-    {
-      UDM_VAR *S = &Doc->Sections.Var[sc];
-      UdmVarListDel(&Env->Vars, S->name);
-    }
-    
-    UdmFree(clist);
   }
   UdmTemplatePrint(Agent, stdout, NULL, 0, &Env->Vars, &tmpl, "resbot");
   UDM_FREE(searchwords);
