@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@ __C_LINK int __UDMCALL UdmServerAdd(UDM_AGENT *A, UDM_SERVER *srv)
   size_t         i;
   UDM_SERVERLIST S;
   UDM_ENV        *Conf = A->Conf;
+  char regerrstr[ERRSTRSIZ]="";
 
   if(srv->Match.match_type==UDM_MATCH_BEGIN)
   {
@@ -136,7 +137,6 @@ __C_LINK int __UDMCALL UdmServerAdd(UDM_AGENT *A, UDM_SERVER *srv)
   else if(srv->Match.match_type==UDM_MATCH_REGEX)
   {
     int err;
-    char regerrstr[ERRSTRSIZ]="";
     if(UDM_OK!=(err=UdmMatchComp(&srv->Match,regerrstr,sizeof(regerrstr)-1)))
     {
       udm_snprintf(Conf->errstr,sizeof(Conf->errstr),"Wrong regex in config file: %s: %s", urlstr,regerrstr);
@@ -184,6 +184,7 @@ __C_LINK int __UDMCALL UdmServerAdd(UDM_AGENT *A, UDM_SERVER *srv)
   new->Match.arg = srv->Match.arg;
   srv->Match.reg = NULL;
   srv->Match.arg = NULL;
+  UdmMatchComp(&new->Match, regerrstr, sizeof(regerrstr));
   new->command = srv->command;
   new->ordre = srv->ordre;
   new->weight = srv->weight;
@@ -226,6 +227,7 @@ UDM_SERVER * UdmServerFind(UDM_ENV *Conf, UDM_SERVERLIST *List, const char *url,
   char        *robots= NULL;
   UDM_SERVER  *Res= NULL;
   char        net[32];
+  size_t      urllen= strlen(url);
   
   /* If it's a robot.txt, cut to hostinfo and find result */
   if((robots=strstr(url,"/robots.txt")))
@@ -233,7 +235,7 @@ UDM_SERVER * UdmServerFind(UDM_ENV *Conf, UDM_SERVERLIST *List, const char *url,
     if(!strcmp(robots,"/robots.txt"))
     {
       robots = (char*)UdmStrdup(url);
-      robots[strlen(url)-10]='\0';
+      robots[urllen-10]='\0';
     }
     else
     {
@@ -272,12 +274,12 @@ UDM_SERVER * UdmServerFind(UDM_ENV *Conf, UDM_SERVERLIST *List, const char *url,
       UdmURLFree(&URL);
     }
      
-    if(follow == UDM_FOLLOW_WORLD || !UdmMatchExec(&srv->Match, url, net, 10, P))
+    if(follow == UDM_FOLLOW_WORLD || !UdmMatchExec(&srv->Match, url, urllen, net, 10, P))
     {
       Res=srv;
       if((aliastr != NULL) && (alias != NULL))
       {
-        aliastrlen = 128 + strlen(url) + strlen(alias) + strlen(srv->Match.pattern);
+        aliastrlen = 128 + urllen + strlen(alias) + strlen(srv->Match.pattern);
         *aliastr = (char*)UdmMalloc(aliastrlen);
         if (*aliastr != NULL)
           UdmMatchApply(*aliastr, aliastrlen, url, alias, &srv->Match, 10, P);
