@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -300,7 +300,7 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
                      &Field[col].Size,
                      &Field[col].Scale,
                      &Field[col].Nullable);
-      result->Fields[col].sqlname= (char*)UdmStrdup(Field[col].Name);
+      result->Fields[col].sqlname= (char*)UdmStrdup((const char*)Field[col].Name);
     }
 
     rc= SQL_NO_DATA_FOUND;
@@ -311,7 +311,11 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
       if (!SQL_OK(rc))
       {
         if (rc != SQL_NO_DATA_FOUND)
+        {
           db->errcode=1;
+          if (db->DBType == UDM_DB_DB2) /* FIXME: check with all databases */
+            return UDM_ERROR;
+        }
         break;
       }
       if (mRows <= (result->nRows+1) * iResColumns)
@@ -350,6 +354,7 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
               db->DBType == UDM_DB_IBASE ||
               db->DBType == UDM_DB_ORACLE8 ||
               db->DBType == UDM_DB_SYBASE ||
+              db->DBType == UDM_DB_DB2 ||
               db->DBType == UDM_DB_MYSQL)
             get_type= SQL_C_BINARY;
           else
@@ -474,6 +479,13 @@ static int UdmODBCInitDB(UDM_DB *db)
   if( !SQL_OK(db->errcode)) return(-5);
   else db->errcode=0;
   db->connected=1;
+  /*
+  if (db->DBType == UDM_DB_DB2)
+  {
+    1253 = SQL_ATTR_LONGDATA_COMPAT
+    int rc= SQLSetConnectAttr(sdb->hDbc, 1253, (SQLPOINTER) 1, 0);
+  }
+  */
   if (db->DBType == UDM_DB_ORACLE8)
     execDB(db, &SQLRes, "ALTER SESSION SET NLS_NUMERIC_CHARACTERS='. '");
   return 0;
@@ -1664,6 +1676,7 @@ static int UdmSQLite3InitDB(UDM_DB *db)
   return UDM_OK;
 }
 
+#if 0
 static int SQLite3CallBack(void *pArg, int argc, char **argv, char **name)
 {
   UDM_SQLRES *res= (UDM_SQLRES*) pArg;
@@ -1684,6 +1697,7 @@ static int SQLite3CallBack(void *pArg, int argc, char **argv, char **name)
       }
     }
   }
+
   res->nRows++;
   res->Items=(UDM_PSTR*)UdmRealloc(res->Items,res->nRows*res->nCols*sizeof(UDM_PSTR));
   
@@ -1697,12 +1711,14 @@ static int SQLite3CallBack(void *pArg, int argc, char **argv, char **name)
   } 
   return 0;
 }
-
+#endif
 
 static int UdmSQLite3Query(UDM_DB *db, UDM_SQLRES *res, const char *q)
 {
-  char *errmsg;
   int rc;
+  sqlite3_stmt *pStmt;
+  const char *Tail;
+  int columns= 0;
   
   if (res)
   {
@@ -1714,28 +1730,104 @@ static int UdmSQLite3Query(UDM_DB *db, UDM_SQLRES *res, const char *q)
   /** Remove heading "/" from DBName (in case of absolute path) */
   if(db->DBName[2] == ':')
   {
-    char *tmp_dbname = UdmStrdup(db->DBName + 1);
+    char *tmp_dbname= UdmStrdup(db->DBName + 1);
     UDM_FREE(db->DBName);
-    db->DBName = tmp_dbname;
+    db->DBName= tmp_dbname;
   }
 #endif /* #ifdef WIN32 */
 
   db->errcode= 0;
-  db->errstr[0]='\0';
-  if(!db->connected && UDM_OK != UdmSQLite3InitDB(db))
+  db->errstr[0]= '\0';
+
+  if (!db->connected && UDM_OK != UdmSQLite3InitDB(db))
     return UDM_ERROR;
-  
-  if ((rc= sqlite3_exec((sqlite3 *)db->specific, q, SQLite3CallBack, (void*)res, &errmsg)))
+
+  if ((rc= sqlite3_prepare((sqlite3*) db->specific, q, -1, &pStmt, &Tail)))
   {
     udm_snprintf(db->errstr, sizeof(db->errstr),
-                 "sqlite3 driver: %s",errmsg ? errmsg : "<NOERROR>");
-    sqlite3_free(errmsg);
-    if (!strstr(db->errstr,"unique"))
-    {
-      db->errcode=1;
-      return UDM_ERROR;
-    }
+                 "sqlite3 driver: (%d) %s", 
+                 sqlite3_errcode((sqlite3*) db->specific),
+                 sqlite3_errmsg((sqlite3*) db->specific));
+    db->errcode= 1;
+    return UDM_ERROR;
   }
+  
+  for (rc= sqlite3_step(pStmt) ; ; rc= sqlite3_step(pStmt))
+  {
+    int i;
+    switch (rc)
+    {
+      case SQLITE_DONE:
+        goto fin;
+
+      case SQLITE_ROW:
+        if (!columns)
+        {
+          columns= sqlite3_column_count(pStmt);
+          res->nCols= columns;
+          res->Fields=(UDM_SQLFIELD*)UdmMalloc(res->nCols*sizeof(UDM_SQLFIELD));
+          bzero(res->Fields,res->nCols*sizeof(UDM_SQLFIELD));
+          for (i=0 ; i < res->nCols; i++)
+          {
+            const char *name= sqlite3_column_name(pStmt, i);
+            res->Fields[i].sqlname= (char*)UdmStrdup(name);
+            res->Fields[i].sqllen= 0;
+            res->Fields[i].sqltype= 0;
+          }
+        }
+        res->nRows++;
+        res->Items=(UDM_PSTR*)UdmRealloc(res->Items,res->nRows*res->nCols*sizeof(UDM_PSTR));
+
+        for (i= 0 ; i < columns; i++)
+        {
+          const unsigned char *value;
+          int type, bytes;
+          size_t offs= (res->nRows-1)*res->nCols+(size_t)i;
+          UDM_PSTR *Item= &res->Items[offs];
+          type= sqlite3_column_type(pStmt, i);
+          if (type == SQLITE_BLOB)
+            value= sqlite3_column_blob(pStmt, i);
+          else
+            value= sqlite3_column_text(pStmt, i);
+          bytes= sqlite3_column_bytes(pStmt, i);
+          Item->len= bytes;
+          Item->val= (char*)UdmMalloc(bytes + 1);
+          if (bytes)
+            memcpy(Item->val, value, bytes);
+          res->Items[offs].val[bytes]= '\0';
+        }
+        break;
+
+      case SQLITE_ERROR:
+        sqlite3_finalize(pStmt);
+        udm_snprintf(db->errstr, sizeof(db->errstr),
+                     "sqlite3 driver: (%d) %s",
+                     sqlite3_errcode((sqlite3*) db->specific),
+                     sqlite3_errmsg((sqlite3*) db->specific));
+        if (!strstr(db->errstr,"unique"))
+        {
+          db->errcode=1;
+          return UDM_ERROR;
+        }
+        return UDM_OK;
+        break;
+    
+      case SQLITE_MISUSE:
+      case SQLITE_BUSY:
+      default:
+        udm_snprintf(db->errstr, sizeof(db->errstr),
+                     "sqlite3_step() returned MISUSE or BUSY");
+        db->errcode= 1;
+        sqlite3_finalize(pStmt);
+        return UDM_ERROR;
+        break;
+      }
+    }
+
+fin:
+  if ((rc= sqlite3_finalize(pStmt)))
+    return UDM_ERROR;
+
   return UDM_OK;
 }
 
@@ -1857,29 +1949,80 @@ char * UdmDBEscStr(int DBType,char *to,const char *from,size_t len)
 
 char *UdmSQLEscStr(UDM_DB *db, char *to, const char *from, size_t len)
 {
+  size_t multiply= db->DBType == UDM_DB_PGSQL ? 4 : 2;
   if(!from)return(NULL);
-  if(!to)to=(char*)UdmMalloc(len*2+1);
+  if(!to)to=(char*)UdmMalloc(len * multiply + 1);
   if(!to)return(NULL);
   return db->sql->SQLEscStr ? db->sql->SQLEscStr(db, to, from, len) : UdmDBEscStr(db->DBType, to, from, len);
 }
 
-int UdmSQLBegin(UDM_DB *db)
+#ifdef HAVE_DEBUG
+int
+UdmSQLBeginDebug(UDM_DB *db, const char *file, const int line)
+{
+  if (db->flags & UDM_SQL_DEBUG_QUERY)
+    fprintf(stderr, "[%d] SQLBegin {%s:%d}\n", getpid(), file, line);
+  return db->sql->SQLBegin(db);
+}
+
+int
+UdmSQLCommitDebug(UDM_DB *db, const char *file, const int line)
+{
+  if (db->flags & UDM_SQL_DEBUG_QUERY)
+    fprintf(stderr, "[%d] SQLCommit {%s:%d}\n", getpid(), file, line);
+  return db->sql->SQLCommit(db);
+}
+int
+UdmSQLExecDirectDebug(UDM_DB *db, UDM_SQLRES *R, const char *query,
+                      const char *file, const int line)
+{
+  if (db->flags & UDM_SQL_DEBUG_QUERY)
+  {
+    int rc;
+    unsigned long long ticks= UdmStartTimer();
+    rc= db->sql->SQLExecDirect(db, R, query);
+    ticks= UdmStartTimer() - ticks;
+    fprintf(stderr,"[%d] %.2fs SQL {%s:%d}: %s\n",
+            getpid(), (float)ticks/1000, file, line, query);
+    return rc;
+  }
+  else
+    return db->sql->SQLExecDirect(db, R, query);
+}
+
+#else
+int
+UdmSQLBegin(UDM_DB *db)
 {
   return db->sql->SQLBegin(db);
 }
 
-int UdmSQLCommit(UDM_DB *db)
+int
+UdmSQLCommit(UDM_DB *db)
 {
   return db->sql->SQLCommit(db);
 }
 
+int
+UdmSQLExecDirect(UDM_DB *db, UDM_SQLRES *R, const char *q)
+{
+  return db->sql->SQLExecDirect(db, R, q);
+}
+#endif
 
-int __UDMCALL _UdmSQLQuery(UDM_DB *db, UDM_SQLRES *SQLRes, const char * query, const char *file, const int line) {
+
+int __UDMCALL
+_UdmSQLQuery(UDM_DB *db, UDM_SQLRES *SQLRes, const char * query,
+             const char *file, const int line)
+{
   UDM_SQLRES res;
   
-#ifdef DEBUG_SQL
+#ifdef HAVE_DEBUG
   unsigned long ticks;
-  ticks=UdmStartTimer();
+  if (db->flags & UDM_SQL_DEBUG_QUERY)
+  {
+    ticks= UdmStartTimer();
+  }
 #endif
   
   /* FIXME: call UdmSQLFree at exit if SQLRes = NULL */
@@ -1887,11 +2030,17 @@ int __UDMCALL _UdmSQLQuery(UDM_DB *db, UDM_SQLRES *SQLRes, const char * query, c
   
   db->sql->SQLQuery(db, SQLRes, query);
 
-#ifdef DEBUG_SQL
-  ticks=UdmStartTimer()-ticks;
-  fprintf(stderr,"[%d] %.2fs SQL {%s:%d}: %s\n", getpid(), (float)ticks/1000, file, line, query);
+#ifdef HAVE_DEBUG
+  if (db->flags & UDM_SQL_DEBUG_QUERY)
+  {
+    ticks= UdmStartTimer() - ticks;
+    fprintf(stderr,"[%d] %.2fs SQL {%s:%d}: %s\n", getpid(), (float)ticks/1000, file, line, query);
+  }
 #endif
   
+  if (db->errcode && (db->flags & UDM_SQL_IGNORE_ERROR))
+    db->errcode= 0;
+  
 #ifdef DEBUG_ERR_QUERY
   if (db->errcode)
     fprintf(stderr, "{%s:%d} Query: %s\n\n", file, line, query);
@@ -1901,8 +2050,6 @@ int __UDMCALL _UdmSQLQuery(UDM_DB *db, UDM_SQLRES *SQLRes, const char * query, c
 }
 
 
-
-
 size_t __UDMCALL UdmSQLNumRows(UDM_SQLRES * res)
 {
   return(res?res->nRows:0);
@@ -2013,6 +2160,38 @@ void UdmSQLClose(UDM_DB *db)
 }
 
 
+void
+UdmSQLResListInit(UDM_SQLRESLIST *List)
+{
+  bzero((void*)List, sizeof(*List));
+}
+
+
+int
+UdmSQLResListAdd(UDM_SQLRESLIST *List, UDM_SQLRES *Res)
+{
+  size_t nbytes= (List->nitems + 1) * sizeof(UDM_SQLRES);
+  if (!(List->Item= (UDM_SQLRES*) UdmRealloc(List->Item, nbytes)))
+    return UDM_ERROR;
+  List->Item[List->nitems]= Res[0];
+  List->nitems++;
+  return UDM_OK;
+}
+
+
+void
+UdmSQLResListFree(UDM_SQLRESLIST *List)
+{
+  size_t i;
+  for (i= 0; i < List->nitems; i++)
+  {
+    UdmSQLFree(&List->Item[i]);
+  }
+  UdmFree(List->Item);
+  UdmSQLResListInit(List);
+}
+
+
 #include "udm_mutex.h"
 
 /*
