@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -123,12 +123,20 @@ size_t UdmTemplatePrintVar(UDM_ENV *Env,
   switch(type)
   {
     case '(': 
+#ifndef NO_ADVANCE_CONV
       eval = UdmRemoveHiLightDup(value);
       res= out_string(stream, dst, dstlen, eval);
+#else
+      res= out_string(stream, dst, dstlen, value);
+#endif
       break;
     case '&':
       {
+#ifndef NO_ADVANCE_CONV
         char *cval= UdmConvDup(value, Env->bcs, Env->bcs, UDM_RECODE_HTML);
+#else
+        char *cval= UdmConvDup(value, Env->lcs, Env->bcs, UDM_RECODE_HTML);
+#endif
         eval= HiLightDup(cval,HlBeg,HlEnd);
         UDM_FREE(cval);
       }
@@ -148,6 +156,42 @@ size_t UdmTemplatePrintVar(UDM_ENV *Env,
   return res;
 }
 
+
+/*
+  Html-aware LEFT() function:
+  Finds byte length of "pos" characters in a safe way,
+  not to break entities like &#1234; or &acute; in the middle.
+*/
+static size_t
+UdmHtmlStrLeft(const char *str, size_t length, size_t pos)
+{
+  const char *p= str + pos;
+  const char *b;
+  
+  if (pos > length)
+    return length;
+  
+  /*
+    '&' '#' alnum* ';'
+  */
+  
+  for (b= (p > str && *p == ';') ? p - 1 : p ;
+       isalnum(*b) && b > str ; b--) /* nop */;
+  
+  /* Check if we found a valid entity beginning */
+  if (*b == '&' || (*b == '#' && b > str && *--b == '&'))
+  {
+    const char *strend= str + length;
+    const char *e;
+    for (e= (p < strend && (*p == '#' || *p == '&')) ? p + 1 : p ;
+         e < strend && isalnum(*e) ; e++) /* nop */;
+    if (e < strend && *e == ';')
+      p= b;
+  }
+  return p - str;
+}
+
+
 static size_t PrintTextTemplate(UDM_AGENT *A, FILE *stream, char *dst, 
                                 size_t dst_len, UDM_VARLIST *vars,
                                 const char *templ,
@@ -157,9 +201,11 @@ static size_t PrintTextTemplate(UDM_AGENT *A, FILE *stream, char *dst,
   const char * s;
   size_t dlen=0;
   
+  if (dst)
+    *dst= '\0'; /* out_string uses strcat */
   for(s=templ; (*s) && ((stream) || (dlen < dst_len)); s++)
   {
-    int type=0;
+    int type= 0, neg= 0;
     char *value=NULL, *newvalue = NULL;
     char empty[]="";
     size_t maxlen=0;
@@ -211,8 +257,8 @@ static size_t PrintTextTemplate(UDM_AGENT *A, FILE *stream, char *dst,
         strncpy(name,vbeg,len);name[len]='\0';
         if((sem=strchr(name,':')))
         {
-          *sem=0;
-          maxlen=atoi(sem+1);
+          *sem++= 0;
+          maxlen= atoi((neg= (*sem == '-')) ? sem + 1 : sem);
         }
         
         if ((A->doccount == 0) && !strcasecmp(name, "ndocs"))
@@ -250,64 +296,43 @@ static size_t PrintTextTemplate(UDM_AGENT *A, FILE *stream, char *dst,
     
     if((curlen>maxlen)&&(maxlen>0))
     {
-      char *p = value + maxlen, *S, *e;
-      S = e = p;
-      if (isdigit(*e)) {
-        while(isdigit(*S) && S > (value + 1)) S--;
-        S--;
-        if (*S == '&' && S[1] == '#')
-        {
-          while(isdigit(*e)) e++;
-          if (*e == ';') p = e + 1;
-        } 
-      }
-      else if (isalpha(*e))
+      if ((newvalue= (char*) UdmMalloc(maxlen + 8)))
       {
-        while(isalpha(*S) && S > value) S--;
-        if (*S == '&')
+        if (!neg)
         {
-          while(isalpha(*e)) e++;
-          if (*e == ';') p = e + 1;
-        }
-      }
-      else if (*e == ';' && S < value )
-      {
-        S--;
-        if (isdigit(*S))
-        {
-          while(isdigit(*S) && S > (value + 1)) S--;
-          S--;
-          if (*S == '&' && S[1] == '#')
+          char *c2, *c3;
+          size_t pos= UdmHtmlStrLeft(value, curlen, maxlen);
+          strncpy(newvalue, value, pos);
+          newvalue[pos] = '\0';
+          c2 = strrchr(newvalue, '\2');
+          c3 = strrchr(newvalue, '\3');
+          if ((c2 != NULL) && ((c3 == NULL) || (c2 > c3)))
           {
-            p = e + 1;
+            strcpy(newvalue + pos, "\3...");
           }
-        }
-        else if (isalpha(*S))
-        {
-          while(isalpha(*S) && S > value) S--;
-          if (*S == '&')
+          else
           {
-            p = e + 1;
+            strcpy(newvalue + pos, "...");
           }
         }
-      }
-      if ((newvalue = (char*)UdmMalloc((size_t)(p - value) + 8)))
-      {
-        char *c2, *c3;
-        strncpy(newvalue, value, (size_t)(p - value));
-        newvalue[(p - value)] = '\0';
-        c2 = strrchr(newvalue, '\2');
-        c3 = strrchr(newvalue, '\3');
-        if ((c2 != NULL) && ((c3 == NULL) || (c2 > c3)))
-        {
-          strcpy(newvalue + (p - value), "\3...");
-        }
         else
         {
-          strcpy(newvalue + (p - value), "...");
+          char *c2, *c3;
+          size_t pos= UdmHtmlStrLeft(value, curlen, curlen - maxlen);
+          size_t len= curlen - pos;
+          size_t offs= 0;
+          c2= strchr(value + pos, '\2');
+          c3= strchr(value + pos, '\3');
+          if (c3 && (!c2 || c2 > c3))
+          {
+            *newvalue= '\2';
+            offs= 1;
+          }
+          memcpy(newvalue + offs, value + pos, len);
+          newvalue[offs + len]= '\0';
         }
         if (value != empty) UDM_FREE(value);
-        value = newvalue;
+        value= newvalue;
       }
     }
     
@@ -332,17 +357,21 @@ static size_t PrintTextTemplate(UDM_AGENT *A, FILE *stream, char *dst,
 }
 
 
+#if 0
 static char *
-GetHtmlTok(const char *src, const char **last, UDM_HTMLTOK *t)
+GetHtmlTok(const char *src, const char **last, UDM_HTMLTOK *t, size_t *length)
 {
   const char *beg= UdmHTMLToken(src, last, t);
   if (beg)
   {
-    size_t length= last[0] - beg;
-    return UdmStrndup(beg, length);
+    *length= last[0] - beg;
+    return UdmStrndup(beg, *length);
   }
+  *length= 0;
   return NULL;
 }
+#endif
+
 
 #define UDM_TMPL_IF		1
 #define UDM_TMPL_IFLIKE		2
@@ -386,6 +415,8 @@ GetHtmlTok(const char *src, const char **last, UDM_HTMLTOK *t)
 #define UDM_TMPL_IFGE		40
 #define UDM_TMPL_IFGT		41
 #define UDM_TMPL_IFLT		42
+#define UDM_TMPL_FOR		43
+#define UDM_TMPL_ENDFOR		44
 
 struct udm_template_st;
 
@@ -427,8 +458,8 @@ typedef struct udm_template_st
   UDM_AGENT *Agent;
   FILE *stream;
   UDM_VARLIST *vars;
-  const char *HlBeg;
-  const char *HlEnd;
+  char *HlBeg;
+  char *HlEnd;
   UDM_TMPL_PRG prg;
   char *dst;
   size_t dstlen;
@@ -528,7 +559,7 @@ static int TemplateFunc2(UDM_TEMPLATE *T)
       M.pattern= it->arg[3];
       M.match_type= UDM_MATCH_REGEX;
       if (!UdmMatchComp(&M, errstr, sizeof(errstr)) &&
-          !UdmMatchExec(&M, vurl, NULL, 10, P))
+          !UdmMatchExec(&M, vurl, strlen(vurl), NULL, 10, P))
       {
         char *res= (char*) UdmMalloc(len);
         res[0]= '\0';
@@ -785,7 +816,7 @@ static size_t PrintTagTemplate(UDM_AGENT *Agent,FILE *stream,
 
   if(vname)
   {
-    var = UdmVarListFindWithValue(vars, UdmTrim(vname, "$()"), value ? value:"");
+    var= UdmVarListFindWithValue(vars, UdmTrim(vname, "$&()"), value ? value:"");
   }
 
   sprintf(UDM_STREND(opt), "%s>", var ? " selected=\"selected\"":"");
@@ -803,8 +834,6 @@ static int TemplateTagOrText(UDM_TEMPLATE *tmpl)
 {
   size_t tmplen;
   UDM_TMPL_PRGITEM *it= &tmpl->prg.Items[tmpl->prg.curr];
-  char *hlbeg= UdmRemoveHiLightDup(tmpl->HlBeg);
-  char *hlend= UdmRemoveHiLightDup(tmpl->HlEnd);
 
   switch (it->cmdnum)
   {
@@ -812,19 +841,19 @@ static int TemplateTagOrText(UDM_TEMPLATE *tmpl)
         
         tmplen= PrintTextTemplate(tmpl->Agent, tmpl->stream,
                                   tmpl->dst , tmpl->dstlen, 
-                                  tmpl->vars, it->arg[0], hlbeg,hlend);
+                                  tmpl->vars, it->arg[0],
+                                  tmpl->HlBeg, tmpl->HlEnd);
         break;
       
       case UDM_TMPL_TAG:
         tmplen= PrintTagTemplate(tmpl->Agent, tmpl->stream,
                                  tmpl->dst, tmpl->dstlen,
-                                 tmpl->vars, it->arg[0], hlbeg, hlend);
+                                 tmpl->vars, it->arg[0],
+                                 tmpl->HlBeg, tmpl->HlEnd);
         break;
   }
   tmpl->dst+= tmplen;
   tmpl->dstlen-= tmplen;
-  UDM_FREE(hlbeg);
-  UDM_FREE(hlend);
   return UDM_OK;
 }
 
@@ -851,18 +880,24 @@ static int TemplateCreateResult(UDM_TEMPLATE *tmpl)
   return UdmVarListCreateObject(tmpl->vars, it->arg[0], UDM_VAR_RESULT, var, 1);
 }
 
-static void HTMLTokToVarList(UDM_VARLIST *vars,UDM_HTMLTOK *tag)
+
+static char
+*UdmHTMLTokAttrDup(UDM_HTMLTOK *tag,
+                   const char *name,
+                   const char *def)
 {
-  size_t  toks;
-  
-  for(toks=0;toks<tag->ntoks;toks++)
+  size_t toks;
+  for (toks= 0; toks < tag->ntoks; toks++)
   {
-    char *vr=tag->toks[toks].name ? UdmStrndup(tag->toks[toks].name,tag->toks[toks].nlen) : (char*)UdmStrdup("");
-    char *vl=tag->toks[toks].val ? UdmStrndup(tag->toks[toks].val,tag->toks[toks].vlen) : (char*)UdmStrdup("");
-    UdmVarListAddStr(vars,vr,vl);
-    UDM_FREE(vr);
-    UDM_FREE(vl);
+    if (tag->toks[toks].name &&
+        !strncasecmp(tag->toks[toks].name, name, tag->toks[toks].nlen))
+    {
+      if (tag->toks[toks].val && tag->toks[toks].vlen)
+        return UdmStrndup(tag->toks[toks].val, tag->toks[toks].vlen);
+      return def ? UdmStrdup(def) : NULL;
+    }
   }
+  return def ? UdmStrdup(def) : NULL;
 }
 
 
@@ -886,6 +921,8 @@ UDM_TMPL_CMD tnames[]=
   {"WHILENOT", 8, UDM_TMPL_WHILENOT , 0, NULL },
   {"WHILE",    5, UDM_TMPL_WHILE    , 0, NULL },
   {"ENDWHILE", 8, UDM_TMPL_ENDWHILE , 0, NULL },
+  {"FOR",      3, UDM_TMPL_FOR      , 0, NULL },
+  {"ENDFOR",   6, UDM_TMPL_ENDFOR   , 0, NULL },
   
   {"COPY",     4, UDM_TMPL_COPY     , 2, TemplateCopy },
   {"SET",      3, UDM_TMPL_SET      , 2, TemplateSetOrPut },
@@ -909,6 +946,12 @@ UDM_TMPL_CMD tnames[]=
 };
 
 
+UDM_TMPL_CMD udm_tmpl_set=
+{"SET", 3, UDM_TMPL_SET, 2, TemplateSetOrPut};
+
+UDM_TMPL_CMD udm_tmpl_inc=
+{"INC", 3, UDM_TMPL_INC, 1, TemplateIncOrDec};
+
 UDM_TMPL_CMD udm_tmpl_txt=
 {"TXT", 3, UDM_TMPL_PRINT, 1, TemplateTagOrText };
 
@@ -953,14 +996,14 @@ static UDM_TMPL_CMD *udm_tmpl_cmpcmd(int cmdnum)
   }
 }
 
-static UDM_TMPL_CMD *HtmlTemplateCmd(const char *str)
+static UDM_TMPL_CMD *HtmlTemplateCmd(const char *str, const char *strend)
 {
   UDM_TMPL_CMD *t;
   size_t len;
-  for (len= 0; str[len] && isalnum(str[len]) ; len++);
+  for (len= 0; str+len < strend && isalnum(str[len]) ; len++);
   for (t= tnames; t->name; t++)
   {
-    if (t->len == len && !strncasecmp(str, t->name, t->len))
+    if (t->len == len && !strncasecmp(str, t->name, len))
       return t;
   }
   return NULL;
@@ -971,6 +1014,7 @@ typedef struct udm_tmpl_compile_stack_item_st
 {
   size_t beg;
   size_t jmp;
+  size_t cmd; /* IF FOR WHILE */
 } UDM_TMPL_COMPILE_STACK_ITEM;
 
 
@@ -1060,15 +1104,19 @@ static void HtmlTemplatePrgFree(UDM_TMPL_PRG *prg)
 
 static int HtmlTemplatePrgAddFunc(UDM_TMPL_PRG *prg,
                                   UDM_TMPL_CMD *cmd,
-                                  const char **args)
+                                  UDM_HTMLTOK *tag)
 {
   UDM_TMPL_PRGITEM i;
   i.cmdnum= cmd->cmdnum;
   i.cmd= cmd;
-  i.arg[0]= args[0] ? UdmStrdup(args[0]) : NULL;
-  i.arg[1]= args[1] ? UdmStrdup(args[1]) : NULL;
-  i.arg[2]= args[2] ? UdmStrdup(args[2]) : NULL; 
-  i.arg[3]= args[3] ? UdmStrdup(args[3]) : NULL; 
+
+  i.arg[0]= UdmHTMLTokAttrDup(tag, "Name", "");
+  if (!(i.arg[1]= UdmHTMLTokAttrDup(tag, "Content", NULL)))
+    i.arg[1]= UdmHTMLTokAttrDup(tag, "From", "");
+  if (!(i.arg[2]= UdmHTMLTokAttrDup(tag, "Result", NULL)))
+    i.arg[2]= UdmHTMLTokAttrDup(tag, "To", "");
+  i.arg[3]= UdmHTMLTokAttrDup(tag, "Match", "");
+  
   i.method= NULL;
   i.jump= 0;
   return HtmlTemplatePrgAdd(prg, &i);
@@ -1077,12 +1125,13 @@ static int HtmlTemplatePrgAddFunc(UDM_TMPL_PRG *prg,
 
 static int HtmlTemplatePrgAdd1Arg(UDM_TMPL_PRG *prg, 
                                   UDM_TMPL_CMD *cmd,
-                                  const char *arg1)
+                                  const char *str,
+                                  const char *strend)
 {
   UDM_TMPL_PRGITEM i;
   i.cmdnum= cmd->cmdnum;
   i.cmd= cmd;
-  i.arg[0]= arg1 ? UdmStrdup(arg1) : NULL;
+  i.arg[0]= str ? UdmStrndup(str, strend - str) : NULL;
   i.arg[1]= NULL;
   i.arg[2]= NULL;
   i.arg[3]= NULL;
@@ -1113,67 +1162,42 @@ static void HtmlTemplateErrorUnexpectedSym(UDM_TMPL_PRG *prg, const char *sym)
 
 static int HtmlTemplatePrgAddInvokeMethod(UDM_TMPL_PRG *prg,
                                           UDM_TMPL_COMPILE_STACK *st,
-                                          const char *template)
+                                          UDM_HTMLTOK *tag,
+                                          const char *str,
+                                          const char *strend)
 {
-  UDM_HTMLTOK tag;
-  UDM_VARLIST attr;
-  UDM_TMPL_PRGITEM i;
-  const char *hlast= NULL;
-  const char *args[4];
-  
-  UdmHTMLTOKInit(&tag);
-  UdmHTMLToken(template, &hlast, &tag);
-  UdmVarListInit(&attr);
-  HTMLTokToVarList(&attr, &tag);
-  
-  args[0]= UdmVarListFindStr(&attr, "Name", "");
-  args[1]= UdmVarListFindStr(&attr, "Content", "");
-  args[2]= UdmVarListFindStr(&attr, "Result", "");
-  args[3]= UdmVarListFindStr(&attr, "Match", "");
+  if (tag->ntoks > 1)
+  {
+    UDM_TMPL_PRGITEM i;
+    i.arg[0]= UdmHTMLTokAttrDup(tag, "Name", "");
+    if (i.arg[0][0])
+    {
+      i.arg[1]= UdmHTMLTokAttrDup(tag, "Content", "");
+      i.arg[2]= UdmHTMLTokAttrDup(tag, "Result", "");
+      i.arg[3]= UdmHTMLTokAttrDup(tag, "Match", "");
+      i.cmdnum= UDM_TMPL_METHOD;
+      i.cmd= &udm_tmpl_method;
+      i.method= UdmStrndup(tag->toks[0].name+1,tag->toks[0].nlen-1);
+      i.jump= 0;
+      HtmlTemplatePrgAdd(prg, &i);
+      return UDM_OK;
+    }
+    UdmFree(i.arg[0]);
+  }
 
-  if (tag.ntoks > 1 && args[0][0])
-  {
-    i.cmdnum= UDM_TMPL_METHOD;
-    i.cmd= &udm_tmpl_method;
-    i.arg[0]= args[0] ? UdmStrdup(args[0]) : NULL;
-    i.arg[1]= args[1] ? UdmStrdup(args[1]) : NULL; 
-    i.arg[2]= args[2] ? UdmStrdup(args[2]) : NULL;
-    i.arg[3]= args[3] ? UdmStrdup(args[3]) : NULL;
-    i.method= UdmStrndup(tag.toks[0].name+1,tag.toks[0].nlen-1);
-    i.jump= 0;
-    HtmlTemplatePrgAdd(prg, &i);
-  }
-  else
-  {
-    /* Just print special tags:  <!DOCTYPE ... > */
-    HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_txt, template);
-  }
-  UdmVarListFree(&attr);
+  /* Just print special tags:  <!DOCTYPE ... > */
+  HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_txt, str, strend);
   return UDM_OK;
 }
 
 static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
                                       UDM_TMPL_COMPILE_STACK *st,
-                                      const char *template,
+                                      UDM_HTMLTOK *tag,
                                       UDM_TMPL_CMD *cmd)
 {
-  UDM_HTMLTOK  tag;
-  UDM_VARLIST  attr;
   UDM_TMPL_COMPILE_STACK_ITEM sitem;
-  const char *hlast=NULL;
-  const char *args[4];
   int rc= UDM_OK;
-  
-  UdmHTMLTOKInit(&tag);
-  UdmHTMLToken(template,&hlast,&tag);
-  UdmVarListInit(&attr);
-  HTMLTokToVarList(&attr,&tag);
-  
-  args[0]= UdmVarListFindStr(&attr,"Name","");
-  args[1]= UdmVarListFindStr(&attr,"Content","");
-  args[2]= UdmVarListFindStr(&attr,"Result","");
-  args[3]= UdmVarListFindStr(&attr,"Match","");
-  
+
   switch (cmd->cmdnum)
   {
     case UDM_TMPL_WHILE:
@@ -1185,8 +1209,9 @@ static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
     case UDM_TMPL_IFLT:
       sitem.beg= prg->nitems;
       sitem.jmp= prg->nitems + 1;
+      sitem.cmd= cmd->cmdnum == UDM_TMPL_WHILE ? UDM_TMPL_WHILE : UDM_TMPL_IF;
       CompileStackPush(st, &sitem);
-      HtmlTemplatePrgAddFunc(prg, udm_tmpl_cmpcmd(cmd->cmdnum), args);
+      HtmlTemplatePrgAddFunc(prg, udm_tmpl_cmpcmd(cmd->cmdnum), tag);
       HtmlTemplatePrgAddJmp(prg, UDM_TMPL_JNE, 0);
       break;
     
@@ -1194,21 +1219,31 @@ static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
     case UDM_TMPL_IFNOT:
       sitem.beg= prg->nitems;
       sitem.jmp= prg->nitems + 1;
+      sitem.cmd= cmd->cmdnum == UDM_TMPL_WHILENOT ? UDM_TMPL_WHILE : UDM_TMPL_IF;
       CompileStackPush(st, &sitem);
-      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_cmp, args);
+      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_cmp, tag);
       HtmlTemplatePrgAddJmp(prg, UDM_TMPL_JE, 0);
       break;
     
+    case UDM_TMPL_FOR:
+      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_set, tag);
+      sitem.beg= prg->nitems;
+      sitem.jmp= prg->nitems + 1;
+      sitem.cmd= UDM_TMPL_FOR;
+      CompileStackPush(st, &sitem);
+      break;
+      
     case UDM_TMPL_IFLIKE:
       sitem.beg= prg->nitems;
       sitem.jmp= prg->nitems + 1;
+      sitem.cmd= UDM_TMPL_IF;
       CompileStackPush(st, &sitem);
-      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_wld, args);
+      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_wld, tag);
       HtmlTemplatePrgAddJmp(prg, UDM_TMPL_JNE, 0);
       break;
     
     case UDM_TMPL_ELSEIF:
-      if (!st->nitems)
+      if (!st->nitems || st->Items[st->nitems-1].cmd != UDM_TMPL_IF)
       {
         HtmlTemplateErrorUnexpectedSym(prg, "<!ELSEIF>");
         rc= UDM_ERROR; 
@@ -1217,12 +1252,12 @@ static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
       HtmlTemplatePrgAddJmp(prg, UDM_TMPL_JMP, 0);
       prg->Items[st->Items[st->nitems-1].jmp].jump= prg->nitems;
       st->Items[st->nitems-1].jmp= prg->nitems + 1;
-      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_cmp, args);
+      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_cmp, tag);
       HtmlTemplatePrgAddJmp(prg, UDM_TMPL_JNE, 0);
       break;
     
     case UDM_TMPL_ELSELIKE:
-      if (!st->nitems)
+      if (!st->nitems || st->Items[st->nitems-1].cmd != UDM_TMPL_IF)
       {
         HtmlTemplateErrorUnexpectedSym(prg, "<!ELSELIKE>");
         rc= UDM_ERROR; 
@@ -1231,12 +1266,12 @@ static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
       HtmlTemplatePrgAddJmp(prg, UDM_TMPL_JMP, 0);
       prg->Items[st->Items[st->nitems-1].jmp].jump= prg->nitems;
       st->Items[st->nitems-1].jmp= prg->nitems + 1;
-      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_wld, args);
+      HtmlTemplatePrgAddFunc(prg, &udm_tmpl_wld, tag);
       HtmlTemplatePrgAddJmp(prg, UDM_TMPL_JNE, 0);
       break;
     
     case UDM_TMPL_ELSE:
-      if (!st->nitems)
+      if (!st->nitems || st->Items[st->nitems-1].cmd != UDM_TMPL_IF)
       {
         HtmlTemplateErrorUnexpectedSym(prg, "<!ELSE>");
         rc= UDM_ERROR; 
@@ -1246,8 +1281,37 @@ static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
       prg->Items[st->Items[st->nitems-1].jmp].jump= prg->nitems;
       break;
     
+    case UDM_TMPL_ENDFOR:
+      if (!st->nitems || st->Items[st->nitems-1].cmd != UDM_TMPL_FOR)
+      {
+        HtmlTemplateErrorUnexpectedSym(prg, "<!ENDFOR>");
+        rc= UDM_ERROR;
+        goto ret;
+      }
+      {
+        UDM_TMPL_PRGITEM *FromItem= &prg->Items[st->Items[st->nitems-1].beg-1];
+        UDM_TMPL_PRGITEM N;
+
+        bzero((void*)&N, sizeof(N));
+        N.cmdnum= UDM_TMPL_INC;
+        N.cmd= &udm_tmpl_inc;
+        N.arg[0]= UdmStrdup(FromItem->arg[0]);
+        HtmlTemplatePrgAdd(prg, &N);
+
+        bzero((void*)&N, sizeof(N));
+        N.cmdnum= UDM_TMPL_IFGT;
+        N.cmd= &udm_tmpl_cmp;
+        N.arg[0]= UdmStrdup(FromItem->arg[0]);
+        N.arg[1]= UdmStrdup(FromItem->arg[2]);
+        HtmlTemplatePrgAdd(prg, &N);
+        
+        HtmlTemplatePrgAddJmp(prg, UDM_TMPL_JNE, st->Items[st->nitems-1].beg);
+      }
+      CompileStackPop(st);
+      break;
+    
     case UDM_TMPL_ENDWHILE:
-      if (!st->nitems)
+      if (!st->nitems || st->Items[st->nitems-1].cmd != UDM_TMPL_WHILE)
       {
         HtmlTemplateErrorUnexpectedSym(prg, "<!ENDWHILE>");
         rc= UDM_ERROR; 
@@ -1259,7 +1323,7 @@ static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
       break;
     
     case UDM_TMPL_ENDIF:
-      if (!st->nitems)
+      if (!st->nitems || st->Items[st->nitems-1].cmd != UDM_TMPL_IF)
       {
         HtmlTemplateErrorUnexpectedSym(prg, "<!ENDIF>");
         rc= UDM_ERROR; 
@@ -1285,60 +1349,69 @@ static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
       break;
     
     default:
-      HtmlTemplatePrgAddFunc(prg, cmd, args);
+      HtmlTemplatePrgAddFunc(prg, cmd, tag);
       break;
   }
   
 ret:
-  UdmVarListFree(&attr);
   return rc;
 }
 
 
-static int HtmlTemplateCompile(UDM_TMPL_PRG *prg, const char *template)
+static int HtmlTemplateCompile(UDM_TMPL_PRG *prg, const char *src)
 {
   const char *lt;
-  char *tok;
+  const char *tok;
   UDM_TMPL_COMPILE_STACK st;
   int rc= UDM_OK;
+
   UDM_HTMLTOK HtmlTok;
   UdmHTMLTOKInit(&HtmlTok);
   CompileStackInit(&st);
   
-  tok= GetHtmlTok(template, &lt, &HtmlTok);
-  while (tok)
+  
+  for (tok= UdmHTMLToken(src, &lt, &HtmlTok) ;
+       tok ;
+       tok= UdmHTMLToken(NULL , &lt, &HtmlTok))
   {
     if (tok[0]=='<' && tok[1] == '!' && isalpha(tok[2]))
     {
       UDM_TMPL_CMD *cmd;
-      if ((cmd= HtmlTemplateCmd(tok+2)))
-        rc= HtmlTemplateCompileCommand(prg, &st, tok, cmd);
+      if ((cmd= HtmlTemplateCmd(tok+2, lt)))
+        rc= HtmlTemplateCompileCommand(prg, &st, &HtmlTok, cmd);
       else if (tok[2] == '-') /* Comment */
-        rc= HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_txt, tok);
+        rc= HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_txt, tok, lt);
       else
-        rc= HtmlTemplatePrgAddInvokeMethod(prg, &st, tok);
+        rc= HtmlTemplatePrgAddInvokeMethod(prg, &st, &HtmlTok, tok, lt);
     }
     else if(!(strncasecmp(tok,"<OPTION",7)))
     {
-      rc= HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_tag, tok);
+      rc= HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_tag, tok, lt);
     }
     else if(!(strncasecmp(tok,"<INPUT",6)))
     {
-      rc= HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_tag, tok);
+      rc= HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_tag, tok, lt);
     }
     else
     {
-      rc= HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_txt, tok);
+      rc= HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_txt, tok, lt);
     }
-    UDM_FREE(tok);
     if (rc != UDM_OK)
       break;
-    tok=GetHtmlTok(NULL, &lt, &HtmlTok);
   }
   
   if (rc == UDM_OK && st.nitems)
   {
-    udm_snprintf(prg->errstr, sizeof(prg->errstr), "Missing ENDIF or ENDWHILE");
+    const char *missing;
+    if (st.Items[st.nitems-1].cmd == UDM_TMPL_IF)
+      missing= "ENDIF";
+    else if (st.Items[st.nitems-1].cmd == UDM_TMPL_FOR)
+      missing= "ENDFOR";
+    else if (st.Items[st.nitems-1].cmd == UDM_TMPL_WHILE)
+      missing= "ENDWHILE";
+    else
+      missing= "ENDUNKNOWN";
+    udm_snprintf(prg->errstr, sizeof(prg->errstr), "Missing %s", missing);
     rc= UDM_ERROR;
   }
   CompileStackFree(&st);
@@ -1449,8 +1522,8 @@ void __UDMCALL UdmTemplatePrint(UDM_AGENT * Agent,
   tmpl.Agent= Agent;
   tmpl.stream= stream;
   tmpl.vars= vars;
-  tmpl.HlBeg= UdmVarListFindStr(vars,"HlBeg","");
-  tmpl.HlEnd= UdmVarListFindStr(vars,"HlEnd","");
+  tmpl.HlBeg= UdmRemoveHiLightDup(UdmVarListFindStr(vars,"HlBeg",""));
+  tmpl.HlEnd= UdmRemoveHiLightDup(UdmVarListFindStr(vars,"HlEnd",""));
   tmpl.dst= dst;
   tmpl.dstlen= dst_len;
   
@@ -1463,12 +1536,15 @@ void __UDMCALL UdmTemplatePrint(UDM_AGENT * Agent,
       if(matches==format)
       {
         PrintHtmlTemplate(&tmpl, tm->Var[t].val);
-        return;
+        goto ret;
       }
       matches++;
     }
   }
   if (First) PrintHtmlTemplate(&tmpl, First->val);
+ret:
+  UDM_FREE(tmpl.HlBeg);
+  UDM_FREE(tmpl.HlEnd);
   return;
 }
 
@@ -1519,14 +1595,14 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
   char    str[1024];
   char    ostr[1024];
   const char  *dbaddr=NULL;
-  int    variables=0;
+  int    variables=0, rc= UDM_OK;
   char    cursection[128]="";
-  char    *cur=NULL;
   char    nameletter[]=
         "abcdefghijklmnopqrstuvwxyz"
         "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
         "0123456789._";
-  
+  UDM_DSTR dstr;
+  UdmDSTRInit(&dstr, 16*1024);
   UdmServerInit(&Srv);
   bzero((void*)&Cfg, sizeof(Cfg));
   Cfg.Indexer = Agent;
@@ -1537,7 +1613,8 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
   if(!(file=fopen(tname,"r")))
   {
     udm_snprintf(Env->errstr,sizeof(Env->errstr)-1,"Unable to open template '%s': %s",tname,strerror(errno));
-    return(1);
+    rc= UDM_ERROR;
+    goto ret;
   }
   
   while(fgets(str,sizeof(str)-1,file))
@@ -1568,7 +1645,10 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
       if(*s=='#')continue;
 
       if(UDM_OK!=(r=ParseHl(&Env->Vars, s)) && UDM_OK!=(r=UdmEnvAddLine(&Cfg,s)))
-        return r;
+      {
+        rc= r;
+        goto ret;
+      }
       continue;
     }
     
@@ -1591,9 +1671,9 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
             UdmVarListAdd(tmpl, NULL);
             I=&tmpl->Var[tmpl->nvars-1];
             I->name = (char*)UdmStrdup(cursection);
-            I->val = (char*)UdmStrdup(cur?cur:"");
+            I->val = (char*) UdmStrdup(dstr.size_data ? dstr.data : "");
             cursection[0]='\0';
-            UDM_FREE(cur);
+            UdmDSTRReset(&dstr);
             continue;
           }
         }
@@ -1609,23 +1689,17 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
     if(!cursection[0])
       continue;
     
-    if(!cur)
-    {
-      cur = (char*)UdmStrdup(ostr);
-    }
-    else
-    {
-      cur=(char*)UdmRealloc(cur,strlen(cur)+strlen(ostr)+1);
-      strcat(cur,ostr);
-    }
+    UdmDSTRAppend(&dstr, ostr, strlen(ostr));
   }
   fclose(file);
   UdmVarListReplaceLst(&Env->Vars, &Srv.Vars, NULL, "*");
   UdmServerFree(&Srv);
-  UDM_FREE(cur);
     
   if (UDM_OK != UdmEnvPrepare(Env))
-    return UDM_ERROR;
+  {
+    rc= UDM_ERROR;
+    goto ret;
+  }
   
 #ifdef HAVE_SQL
   if(Env->dbl.nitems == 0)
@@ -1633,7 +1707,8 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
     if (UDM_OK!=UdmDBListAdd(&Env->dbl, dbaddr = "mysql://localhost/mnogosearch", UDM_OPEN_MODE_READ))
     {
       sprintf(Env->errstr,"Invalid DBAddr: '%s'", (!dbaddr)?"NULL":dbaddr);
-      return UDM_ERROR;
+      rc= UDM_ERROR;
+      goto ret;
     }
   }
 #endif
@@ -1642,9 +1717,12 @@ int UdmTemplateLoad(UDM_AGENT *Agent,
     if (UDM_OK!=UdmDBListAdd(&Env->dbl, dbaddr =  "searchd://localhost/", UDM_OPEN_MODE_READ))
     {
       sprintf(Env->errstr,"Invalid DBAddr: '%s'", (!dbaddr)?"NULL":dbaddr);
-      return UDM_ERROR;
+      rc= UDM_ERROR;
+      goto ret;
     }
   }
-  
-  return UDM_OK;
+
+ret:  
+  UdmDSTRFree(&dstr);
+  return rc;
 }
