@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,20 +34,24 @@ UDM_CHARSET udm_charset_sys_int=
   udm_mb_wc_sys_int,
   udm_wc_mb_sys_int,
   lcase_mb,
+  NULL, /* septoken */
   "sys-int",
   UDM_CHARSET_UNICODE,
   NULL,
+  NULL,
   NULL
 };
 
 static UDM_CHARSET built_charsets[]={
 {
   UDM_CHARSET_8859_1,
-  udm_mb_wc_8bit,
-  udm_wc_mb_8bit,
+  udm_mb_wc_latin1,
+  udm_wc_mb_latin1,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-1",
   UDM_CHARSET_WESTERN,
+  ctype_iso_8859_1,
   tab_8859_1_uni,
   idx_uni_8859_1
 },
@@ -56,8 +60,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-10",
   UDM_CHARSET_NORDIC,
+  ctype_iso_8859_10,
   tab_8859_10_uni,
   idx_uni_8859_10
 },
@@ -66,8 +72,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-11",
   UDM_CHARSET_THAI,
+  ctype_iso_8859_11,
   tab_8859_11_uni,
   idx_uni_8859_11
 },
@@ -76,8 +84,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-13",
   UDM_CHARSET_BALTIC,
+  ctype_iso_8859_13,
   tab_8859_13_uni,
   idx_uni_8859_13
 },
@@ -86,8 +96,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-14",
   UDM_CHARSET_CELTIC,
+  ctype_iso_8859_14,
   tab_8859_14_uni,
   idx_uni_8859_14
 },
@@ -96,8 +108,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-15",
   UDM_CHARSET_WESTERN,
+  ctype_iso_8859_15,
   tab_8859_15_uni,
   idx_uni_8859_15
 },
@@ -106,8 +120,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-16",
   UDM_CHARSET_CENTRAL,
+  ctype_iso_8859_16,
   tab_8859_16_uni,
   idx_uni_8859_16
 },
@@ -116,8 +132,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-2",
   UDM_CHARSET_CENTRAL,
+  ctype_iso_8859_2,
   tab_8859_2_uni,
   idx_uni_8859_2
 },
@@ -126,8 +144,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-3",
   UDM_CHARSET_SOUTHERN,
+  ctype_iso_8859_3,
   tab_8859_3_uni,
   idx_uni_8859_3
 },
@@ -136,8 +156,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-4",
   UDM_CHARSET_BALTIC,
+  ctype_iso_8859_4,
   tab_8859_4_uni,
   idx_uni_8859_4
 },
@@ -146,8 +168,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-5",
   UDM_CHARSET_CYRILLIC,
+  ctype_iso_8859_5,
   tab_8859_5_uni,
   idx_uni_8859_5
 },
@@ -156,8 +180,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-6",
   UDM_CHARSET_ARABIC,
+  ctype_iso_8859_6,
   tab_8859_6_uni,
   idx_uni_8859_6
 },
@@ -166,8 +192,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-7",
   UDM_CHARSET_GREEK,
+  ctype_iso_8859_7,
   tab_8859_7_uni,
   idx_uni_8859_7
 },
@@ -176,8 +204,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-8",
   UDM_CHARSET_HEBREW,
+  ctype_iso_8859_8,
   tab_8859_8_uni,
   idx_uni_8859_8
 },
@@ -186,8 +216,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "iso-8859-9",
   UDM_CHARSET_TURKISH,
+  ctype_iso_8859_9,
   tab_8859_9_uni,
   idx_uni_8859_9
 },
@@ -196,8 +228,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "armscii-8",
   UDM_CHARSET_ARMENIAN,
+  ctype_armscii_8,
   tab_armscii_8_uni,
   idx_uni_armscii_8
 },
@@ -206,8 +240,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1250",
   UDM_CHARSET_CENTRAL,
+  ctype_windows_1250,
   tab_cp1250_uni,
   idx_uni_cp1250
 },
@@ -216,8 +252,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1251",
   UDM_CHARSET_CYRILLIC,
+  ctype_windows_1251,
   tab_cp1251_uni,
   idx_uni_cp1251
 },
@@ -226,8 +264,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1252",
   UDM_CHARSET_WESTERN,
+  ctype_windows_1252,
   tab_cp1252_uni,
   idx_uni_cp1252
 },
@@ -236,8 +276,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1253",
   UDM_CHARSET_GREEK,
+  ctype_windows_1253,
   tab_cp1253_uni,
   idx_uni_cp1253
 },
@@ -246,8 +288,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1254",
   UDM_CHARSET_TURKISH,
+  ctype_windows_1254,
   tab_cp1254_uni,
   idx_uni_cp1254
 },
@@ -256,8 +300,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1255",
   UDM_CHARSET_HEBREW,
+  ctype_windows_1255,
   tab_cp1255_uni,
   idx_uni_cp1255
 },
@@ -266,8 +312,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1256",
   UDM_CHARSET_ARABIC,
+  ctype_windows_1256,
   tab_cp1256_uni,
   idx_uni_cp1256
 },
@@ -276,8 +324,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1257",
   UDM_CHARSET_BALTIC,
+  ctype_windows_1257,
   tab_cp1257_uni,
   idx_uni_cp1257
 },
@@ -286,8 +336,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "windows-1258",
   UDM_CHARSET_VIETNAMESE,
+  ctype_windows_1258,
   tab_cp1258_uni,
   idx_uni_cp1258
 },
@@ -296,8 +348,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp437",
   UDM_CHARSET_WESTERN,
+  ctype_cp437,
   tab_cp437_uni,
   idx_uni_cp437
 },
@@ -306,8 +360,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp850",
   UDM_CHARSET_WESTERN,
+  ctype_cp850,
   tab_cp850_uni,
   idx_uni_cp850
 },
@@ -316,8 +372,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp852",
   UDM_CHARSET_CENTRAL,
+  ctype_cp852,
   tab_cp852_uni,
   idx_uni_cp852
 },
@@ -326,8 +384,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp855",
   UDM_CHARSET_CYRILLIC,
+  ctype_cp855,
   tab_cp855_uni,
   idx_uni_cp855
 },
@@ -336,8 +396,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp857",
   UDM_CHARSET_TURKISH,
+  ctype_cp857,
   tab_cp857_uni,
   idx_uni_cp857
 },
@@ -346,8 +408,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp860",
   UDM_CHARSET_WESTERN,
+  ctype_cp860,
   tab_cp860_uni,
   idx_uni_cp860
 },
@@ -356,8 +420,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp861",
   UDM_CHARSET_ICELANDIC,
+  ctype_cp861,
   tab_cp861_uni,
   idx_uni_cp861
 },
@@ -366,8 +432,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp862",
   UDM_CHARSET_HEBREW,
+  ctype_cp862,
   tab_cp862_uni,
   idx_uni_cp862
 },
@@ -376,8 +444,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp863",
   UDM_CHARSET_WESTERN,
+  ctype_cp863,
   tab_cp863_uni,
   idx_uni_cp863
 },
@@ -386,8 +456,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp864",
   UDM_CHARSET_ARABIC,
+  ctype_cp864,
   tab_cp864_uni,
   idx_uni_cp864
 },
@@ -396,8 +468,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp865",
   UDM_CHARSET_NORDIC,
+  ctype_cp865,
   tab_cp865_uni,
   idx_uni_cp865
 },
@@ -406,8 +480,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp866",
   UDM_CHARSET_CYRILLIC,
+  ctype_cp866,
   tab_cp866_uni,
   idx_uni_cp866
 },
@@ -416,8 +492,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp869",
   UDM_CHARSET_GREEK,
+  ctype_cp869,
   tab_cp869_uni,
   idx_uni_cp869
 },
@@ -426,8 +504,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "cp874",
   UDM_CHARSET_THAI,
+  ctype_cp874,
   tab_cp874_uni,
   idx_uni_cp874
 },
@@ -436,8 +516,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "koi8-r",
   UDM_CHARSET_CYRILLIC,
+  ctype_koi8_r,
   tab_koi8_r_uni,
   idx_uni_koi8_r
 },
@@ -446,8 +528,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "koi8-u",
   UDM_CHARSET_CYRILLIC,
+  ctype_koi8_u,
   tab_koi8_u_uni,
   idx_uni_koi8_u
 },
@@ -456,8 +540,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacArabic",
   UDM_CHARSET_ARABIC,
+  ctype_MacArabic,
   tab_macarabic_uni,
   idx_uni_macarabic
 },
@@ -466,8 +552,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacCE",
   UDM_CHARSET_CENTRAL,
+  ctype_MacCE,
   tab_macce_uni,
   idx_uni_macce
 },
@@ -476,8 +564,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacCroatian",
   UDM_CHARSET_CENTRAL,
+  ctype_MacCroatian,
   tab_maccroatian_uni,
   idx_uni_maccroatian
 },
@@ -486,8 +576,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacCyrillic",
   UDM_CHARSET_CYRILLIC,
+  ctype_MacCyrillic,
   tab_maccyrillic_uni,
   idx_uni_maccyrillic
 },
@@ -496,8 +588,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacGreek",
   UDM_CHARSET_GREEK,
+  ctype_MacGreek,
   tab_macgreek_uni,
   idx_uni_macgreek
 },
@@ -506,8 +600,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacHebrew",
   UDM_CHARSET_HEBREW,
+  ctype_MacHebrew,
   tab_machebrew_uni,
   idx_uni_machebrew
 },
@@ -516,8 +612,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacIceland",
   UDM_CHARSET_ICELANDIC,
+  ctype_MacIceland,
   tab_maciceland_uni,
   idx_uni_maciceland
 },
@@ -526,8 +624,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacRoman",
   UDM_CHARSET_WESTERN,
+  ctype_MacRoman,
   tab_macroman_uni,
   idx_uni_macroman
 },
@@ -536,8 +636,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacRomania",
   UDM_CHARSET_CENTRAL,
+  ctype_MacRomania,
   tab_macromania_uni,
   idx_uni_macromania
 },
@@ -546,8 +648,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacThai",
   UDM_CHARSET_THAI,
+  ctype_MacThai,
   tab_macthai_uni,
   idx_uni_macthai
 },
@@ -556,18 +660,22 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "MacTurkish",
   UDM_CHARSET_TURKISH,
+  ctype_MacTurkish,
   tab_macturkish_uni,
   idx_uni_macturkish
 },
 {
   UDM_CHARSET_US_ASCII,
-  udm_mb_wc_8bit,
-  udm_wc_mb_8bit,
+  udm_mb_wc_ascii,
+  udm_wc_mb_ascii,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "us-ascii",
   UDM_CHARSET_WESTERN,
+  ctype_us_ascii,
   tab_us_ascii_uni,
   idx_uni_us_ascii
 },
@@ -576,8 +684,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "viscii",
   UDM_CHARSET_VIETNAMESE,
+  ctype_viscii,
   tab_viscii_uni,
   idx_uni_viscii
 },
@@ -586,8 +696,10 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_8bit,
   udm_wc_mb_8bit,
   lcase_8bit,
+  UdmStrGetSepToken8bit,
   "geostd8",
   UDM_CHARSET_GEORGIAN,
+  ctype_geostd8,
   tab_geostd8_uni,
   idx_uni_geostd8
 },
@@ -596,9 +708,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_utf8,
   udm_wc_mb_utf8,
   lcase_mb,
+  NULL, /* septoken */
   "utf-8",
   UDM_CHARSET_UNICODE,
   NULL,
+  NULL,
   NULL
 },
 {
@@ -606,9 +720,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_sys_int,
   udm_wc_mb_sys_int,
   lcase_mb,
+  NULL, /* septok */
   "sys-int",
   UDM_CHARSET_UNICODE,
   NULL,
+  NULL,
   NULL
 },
 
@@ -618,9 +734,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_gb2312,
   udm_wc_mb_gb2312,
   lcase_mb,
+  NULL, /* septok */
   "gb2312",
   UDM_CHARSET_CHINESE_SIMPLIFIED,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -631,9 +749,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_big5,
   udm_wc_mb_big5,
   lcase_mb,
+  NULL, /* septok */
   "big5",
   UDM_CHARSET_CHINESE_TRADITIONAL,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -644,9 +764,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_sjis,
   udm_wc_mb_sjis,
   lcase_mb,
+  NULL, /* septok */
   "Shift-JIS",
   UDM_CHARSET_JAPANESE,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -657,9 +779,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_euc_kr,
   udm_wc_mb_euc_kr,
   lcase_mb,
+  NULL, /* septok */
   "EUC-KR",
   UDM_CHARSET_KOREAN,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -670,9 +794,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_euc_jp,
   udm_wc_mb_euc_jp,
   lcase_mb,
+  NULL, /* septok */
   "EUC-JP",
   UDM_CHARSET_JAPANESE,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -684,9 +810,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_gbk,
   udm_wc_mb_gbk,
   lcase_mb,
+  NULL, /* septok */
   "GBK",
   UDM_CHARSET_CHINESE_SIMPLIFIED,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -697,9 +825,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_gujarati,
   udm_wc_mb_gujarati,
   lcase_mb,
+  NULL, /* septok */
   "MacGujarati",
   UDM_CHARSET_INDIAN,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -710,9 +840,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_tscii,
   udm_wc_mb_tscii,
   lcase_mb,
+  NULL, /* septok */
   "tscii",
   UDM_CHARSET_INDIAN,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -723,9 +855,11 @@ static UDM_CHARSET built_charsets[]={
   udm_mb_wc_iso2022jp,
   udm_wc_mb_iso2022jp,
   lcase_mb,
+  NULL, /* septok */
   "iso-2022-jp",
   UDM_CHARSET_JAPANESE,
   NULL,
+  NULL,
   NULL
 },
 #endif
@@ -736,8 +870,10 @@ static UDM_CHARSET built_charsets[]={
   NULL,
   NULL,
   NULL,
+  NULL,
   0,
   NULL,
+  NULL,
   NULL
 }
 
