@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,15 +23,10 @@
 __C_LINK int __UDMCALL
 udm_wc_mb_sys_int (UDM_CONV *conv, UDM_CHARSET *cs, int *wc, unsigned char *s, unsigned char *e)
 {
-  
-  int code = *wc;
-  
-  conv->icodes = conv->ocodes = 1;
-
-  if ( s+sizeof(int) > e)
+  if (s + sizeof(int) > e)
     return UDM_CHARSET_TOOSMALL;
   
-  memcpy(s,&code,sizeof(code));
+  memcpy(s, wc, sizeof(int));
   
   return sizeof(int);
 }
@@ -43,8 +38,6 @@ udm_mb_wc_sys_int (UDM_CONV *conv, UDM_CHARSET *cs,int *pwc,
           const unsigned char *e)
 {
   
-  conv->icodes = conv->ocodes = 1;
-
   if (s+sizeof(int)>e)
     return UDM_CHARSET_TOOFEW(0);
   
