@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -55,17 +55,37 @@ typedef struct udm_var_handler_st
 } UDM_VARHANDLER;
 
 
-static int varcmp(const UDM_VAR *uv1, const UDM_VAR *uv2)
+int udm_l1tolower[256]=
 {
-  if (uv1->name == NULL)
-  {
-    if (uv2->name == NULL)
+  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
+  0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,
+  0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,
+  0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,
+  0x40,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,
+  0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x5B,0x5C,0x5D,0x5E,0x5F,
+  0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,
+  0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x7B,0x7C,0x7D,0x7E,0x7F,
+  0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F,
+  0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F,
+  0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,
+  0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF,
+  0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF,
+  0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xD7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xDF,
+  0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF,
+  0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF
+};
+
+
+static int
+varcmp(const UDM_VAR *uv1, const UDM_VAR *uv2)
+{
+  const char *us1= uv1->name;
+  const char *us2= uv2->name;
+  while (udm_l1tolower[(unsigned char)*us1] == udm_l1tolower[(unsigned char)*us2++])
+    if (*us1++ == '\0')
       return 0;
-    return 1;
-  }
-  if (uv2->name == NULL)
-    return -1;
-  return strcasecmp(uv1->name, uv2->name);
+  return udm_l1tolower[(unsigned char)*us1] -
+         udm_l1tolower[(unsigned char)*--us2];
 }
 
 
@@ -122,6 +142,18 @@ UDM_VARHANDLER SimpleVar=
 };
 
 
+static int UdmVarCopyValueEmptyStr(UDM_VAR *D)
+{
+  D->handler= &SimpleVar;
+  D->section= 0;
+  D->maxlen= 0;
+  D->curlen= 0;
+  D->flags= 0;
+  D->val= NULL;
+  return UDM_OK;
+}
+
+
 
 /********************************************************/
 
@@ -156,6 +188,7 @@ static void UdmVarFreeEnv(UDM_VAR *V)
 
 static int UdmVarCopyValueEnv(UDM_VAR *D, UDM_VAR *S)
 {
+  UdmVarCopyValueEmptyStr(D);
   return UDM_OK;
 }
 
@@ -193,12 +226,105 @@ static int UdmVarListPrint(UDM_VARLIST *Vars)
   for (i= 0; i < Vars->nvars; i++)
   {
   UDM_VAR *v= &Vars->Var[i];
-    fprintf(stderr, "[%d] '%s'='%s'\n", i, v->name, v->val);
+  /*fprintf(stderr, "[%d] '%s'='%s'\n", i, v->name, v->val);*/
   }
   return UDM_OK;
 }
 #endif
 
+
+/*
+  Check if a VarList is sorted
+  Return
+    0 - sorted
+    1 - almost sorted, only the last cooord is on a wrong position
+    2 - very unsorted
+*/
+/*
+static int
+UdmVarListSortStatus(UDM_VARLIST *Vars)
+{
+  size_t var, nwrong;
+  int res;
+  for (nwrong=0, var=1; var < Vars->nvars; var++)
+  {
+    UDM_VAR *Var= &Vars->Var[var];
+    res= varcmp(&Var[-1], &Var[0]);
+    if (res > 0)
+    {
+      nwrong++;
+      if (nwrong > 1)
+        return nwrong;
+    }
+  }
+  return nwrong ? (res > 0 ? 1 : 2) : 0;
+}
+*/
+
+/*
+  This function is called after UdmVarListAdd().
+  Move the last added variable to its sorted place.
+  
+  If there are variables with the same name, the
+  new variable is inserted after them.
+*/
+static void
+UdmVarListMoveLast(UDM_VARLIST *Vars)
+{
+  size_t movevars;
+  UDM_VAR *Last= &Vars->Var[Vars->nvars-1];
+  UDM_VAR *Left= &Vars->Var[0];
+  UDM_VAR *Right= &Vars->Var[Vars->nvars-1];
+
+  for ( ; Left < Right ; )
+  {
+    UDM_VAR *Middle= Left + (Right - Left)/2;
+    if (varcmp(Middle, Last) <= 0) /* We are looking for the last, see above */
+      Left= Middle + 1;
+    else
+     Right= Middle;
+  }
+
+  if ((movevars= Vars->nvars - (Right - Vars->Var) - 1))
+  {
+    UDM_VAR Tmp= *Last;
+    memmove(Right + 1, Right, movevars * sizeof(UDM_VAR));
+    *Right= Tmp;
+  }
+}
+
+
+static void
+UdmVarListSort(UDM_VARLIST *Vars)
+{
+/*
+  int res;
+*/  
+#ifdef DEBUG_VARS
+  fprintf(stderr, "VarListSort: %d\n", Vars->nvars);
+#endif
+
+  if (Vars->nvars < 2)
+    return;
+
+/*
+  if (!(res= UdmVarListSortStatus(Vars)))
+    return;
+
+  if (res == 1)
+  {
+    UdmVarListMoveLast(Vars);
+    return;
+  }
+*/
+  if (Vars->nvars == Vars->svars + 1)
+    UdmVarListMoveLast(Vars);
+  else
+    UdmSort(Vars->Var, Vars->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+  Vars->svars= Vars->nvars;
+}
+
+
 static int UdmVarListMethodEnvFind(UDM_VARLIST *Vars, UDM_VAR *Var,
                                    UDM_VAR **args, size_t nargs)
 {
@@ -228,7 +354,7 @@ static int UdmVarListMethodEnvFind(UDM_VARLIST *Vars, UDM_VAR *Var,
       New->val= (char*) Res;
       New->handler= &ResultVar;
       New->name= UdmStrdup(resname);
-      UdmSort(Vars->Var, Vars->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+      UdmVarListSort(Vars);
       
       udm_snprintf(prmname, sizeof(prmname), "%s.first", resname);
       UdmVarListReplaceInt(Vars, prmname, (int)Res->first);
@@ -271,7 +397,7 @@ static int UdmVarListMethodEnvSQLQuery(UDM_VARLIST *Vars, UDM_VAR *Var,
       New->val= (char*) Res;
       New->handler= &SQLResultVar;
       New->name= UdmStrdup(resname);
-      UdmSort(Vars->Var, Vars->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+      UdmVarListSort(Vars);
       udm_snprintf(prmname, sizeof(prmname), "%s.num_rows", resname);
       UdmVarListReplaceUnsigned(Vars, prmname, UdmSQLNumRows(Res));
     }
@@ -322,6 +448,7 @@ static void UdmVarFreeResult(UDM_VAR *V)
 
 static int UdmVarCopyValueResult(UDM_VAR *D, UDM_VAR *S)
 {
+  UdmVarCopyValueEmptyStr(D);
   return UDM_OK;
 }
 
@@ -371,6 +498,7 @@ static void UdmVarFreeSQLResult(UDM_VAR *V)
 
 static int UdmVarCopyValueSQLResult(UDM_VAR *D, UDM_VAR *S)
 {
+  UdmVarCopyValueEmptyStr(D);
   return UDM_OK;
 }
 
@@ -471,7 +599,7 @@ int UdmVarListCreateObject(UDM_VARLIST *Vars, const char *name, int type,
     New= &Vars->Var[Vars->nvars-1];
     h->Create(h, New, args, nargs);
     New->name= UdmStrdup(name);
-    UdmSort(Vars->Var, Vars->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+    UdmVarListSort(Vars);
   }
   return UDM_OK;
 }
@@ -520,6 +648,10 @@ static int UdmVarCopy(UDM_VAR *D, UDM_VAR *S, const char *name)
 {
   UDM_VARHANDLER *handler= S->handler ? S->handler : &SimpleVar;
   UdmVarCopyName(D, S, name);
+#ifdef DEBUG_VARS
+  fprintf(stderr, "Copy: '%s:%s'\n", name, S->name);
+#endif
+  UDM_ASSERT(name || S->name);
   handler->CopyValue(D, S);
   return UDM_OK;
 }
@@ -538,7 +670,7 @@ int UdmVarListAdd(UDM_VARLIST * Lst,UDM_VAR * S)
   if (!New->handler)
     New->handler= &SimpleVar;
   Lst->nvars++;
-  if(S) UdmSort(Lst->Var, Lst->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+  if(S) UdmVarListSort(Lst);
   return UDM_OK;
 }
 
@@ -548,17 +680,19 @@ static int UdmVarListAddNamed(UDM_VARLIST *Lst,UDM_VAR *S, const char *name)
   UdmVarListAdd(Lst,NULL);
   v=&Lst->Var[Lst->nvars-1];
   UdmVarCopy(v,S,name);
-  UdmSort(Lst->Var, Lst->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+  UdmVarListSort(Lst);
   return UDM_OK;
 }
 
 UDM_VAR * UdmVarListFind(UDM_VARLIST * vars,const char * name)
 {
   UDM_VAR key;
-  
+#ifdef DEBUG_VARS
+  fprintf(stderr, "Find: '%s'\n", name);
+#endif
   if (!vars->nvars) return NULL;
   key.name= (char*)name;
-  return (UDM_VAR*)bsearch(&key, vars->Var, vars->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+  return (UDM_VAR*)UdmBSearch(&key, vars->Var, vars->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
 }
 
 int UdmVarListDelBySection(UDM_VARLIST *vars, int sec)
@@ -646,7 +780,7 @@ static int UdmVarListReplaceNamed(UDM_VARLIST *Lst,UDM_VAR *S, const char *name)
     UdmVarListAdd(Lst,NULL);
     v=&Lst->Var[Lst->nvars-1];
     rc=UdmVarCopy(v,S,name);
-    UdmSort(Lst->Var, Lst->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+    UdmVarListSort(Lst);
   }
   return rc;
 }
@@ -660,7 +794,7 @@ static int UdmVarListInsNamed(UDM_VARLIST *Lst,UDM_VAR *S, const char *name)
     UdmVarListAdd(Lst,NULL);
     v=&Lst->Var[Lst->nvars-1];
     rc=UdmVarCopy(v,S,name);
-    UdmSort(Lst->Var, Lst->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+    UdmVarListSort(Lst);
   }
   return rc;
 }
@@ -707,9 +841,10 @@ void UdmVarListFree(UDM_VARLIST * vars)
     UDM_FREE(vars);
 }
 
-static
-int UdmVarListAddStrWithSection(UDM_VARLIST *vars,
-                                const char *name, const char *val, int sec)
+static int
+UdmVarListAddStrWithSection(UDM_VARLIST *vars,
+                            const char *name, const char *val, int sec,
+                            int sort_flag)
 {
   UDM_VAR *New;
   UdmVarListAdd(vars, NULL);
@@ -720,19 +855,23 @@ int UdmVarListAddStrWithSection(UDM_VARLIST *vars,
   New->maxlen=0;
   New->curlen= (val != NULL) ? strlen(val) : 0;  
   New->name= name ? (char*)UdmStrdup(name) : NULL;
+#ifdef DEBUG_VARS
+  fprintf(stderr, "AddSec: '%s'\n", name);
+#endif
   New->val= val ? (char*)UdmStrdup(val) : NULL;
-  UdmSort(vars->Var, vars->nvars, sizeof(UDM_VAR), (udm_qsort_cmp)varcmp);
+  if (sort_flag)
+    UdmVarListSort(vars);
   return vars->nvars;
 }
 
 int UdmVarListAddStr(UDM_VARLIST *vars, const char *name, const char *val)
 {
-  return UdmVarListAddStrWithSection(vars, name, val, 0);
+  return UdmVarListAddStrWithSection(vars, name, val, 0, 1);
 }
 
 int UdmVarListAddQueryStr(UDM_VARLIST *vars, const char *name, const char *val)
 {
-  return UdmVarListAddStrWithSection(vars, name, val, UDM_VARSRC_QSTRING);
+  return UdmVarListAddStrWithSection(vars, name, val, UDM_VARSRC_QSTRING, 1);
 }
 
 
@@ -768,6 +907,18 @@ int UdmVarListInsInt(UDM_VARLIST * vars,const char * name,int val)
 }
 
 
+
+int UdmVarAppendStrn(UDM_VAR *var, const char *val, size_t len)
+{
+  size_t newlen= var->curlen + len;
+  var->val= (char*) UdmRealloc(var->val, newlen + 1);
+  memcpy(var->val + var->curlen, val, len);
+  var->val[newlen]= '\0';
+  var->curlen= newlen;
+  return UDM_OK;
+}
+
+
 __C_LINK int __UDMCALL UdmVarListReplaceStr(UDM_VARLIST *vars,
                                             const char *name, const char *val)
 {
@@ -941,6 +1092,31 @@ int UdmVarListInsLst(UDM_VARLIST *D, UDM_VARLIST *S,
 }
 
 
+/*
+  Merge two varlists with single sorting
+*/
+int UdmVarListMerge(UDM_VARLIST * Lst,UDM_VARLIST *Src1, UDM_VARLIST *Src2)
+{
+  size_t i;
+  Lst->nvars= Lst->mvars= Src1->nvars + Src2->nvars;
+  if (!(Lst->Var= (UDM_VAR*) UdmMalloc(Lst->nvars * sizeof(UDM_VAR))))
+    return UDM_ERROR;
+
+  for (i= 0; i < Src1->nvars; i++)
+  {
+    UdmVarCopy(&Lst->Var[i], &Src1->Var[i], NULL);
+  }
+  for (i= 0; i < Src2->nvars; i++)
+  {
+    UdmVarCopy(&Lst->Var[i+Src1->nvars], &Src2->Var[i], NULL);
+  }
+  if (Lst->nvars)
+    UdmVarListSort(Lst);
+  return UDM_OK;
+}
+
+
+
 /* 
   Add environment variables 
   into a varlist
@@ -971,9 +1147,10 @@ int UdmVarListAddEnviron(UDM_VARLIST *Vars, const char *name)
     if((val=strchr(str,'=')))
     {
       *val++='\0';
-      UdmVarListAddStrWithSection(Vars,str,val,UDM_VARSRC_ENV);
+      UdmVarListAddStrWithSection(Vars,str,val,UDM_VARSRC_ENV,0);
     }
   }
+  UdmVarListSort(Vars);
   UDM_FREE(str);
   return UDM_OK;
 }
@@ -988,7 +1165,8 @@ int UdmVarListConvert(UDM_VARLIST *Vars, UDM_CONV *conv)
     {
       size_t len= strlen(Var->val), newlen;
       char *newval= (char*)UdmMalloc(len * 12 + 1);
-      newlen= UdmConv(conv, newval, len * 12 + 1, Var->val, len + 1);
+      newlen= UdmConv(conv, newval, len * 12 + 1, Var->val, len);
+      newval[newlen]= '\0';
       UDM_FREE(Var->val);
       Var->val= newval;
       Var->curlen= newlen;
